// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/grpc_json_reverse_transcoder/v3/transcoder.proto

package grpc_json_reverse_transcoderv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#next-free-field: 7]
// “GrpcJsonReverseTranscoder“ is the filter configuration for the gRPC JSON
// reverse transcoder. The reverse transcoder acts as a bridge between a gRPC
// client and an HTTP/JSON server, converting the gRPC request into HTTP/JSON
// for the HTTP backend and the HTTP/JSON response back to gRPC for the gRPC
// client. This effectively reverses the behavior of the
// :ref:`grpc_json_transcoder filter <config_http_filters_grpc_json_transcoder>`,
// allowing a gRPC client to communicate with an HTTP/JSON server.
type GrpcJsonReverseTranscoder struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Supplies the filename of
	// :ref:`the proto descriptor set
	// <config_grpc_json_reverse_transcoder_generate_proto_descriptor_set>` for the gRPC services.
	// If set, takes precedence over the “descriptor_binary“ field.
	DescriptorPath string `protobuf:"bytes,1,opt,name=descriptor_path,json=descriptorPath,proto3" json:"descriptor_path,omitempty"`
	// Supplies the binary content of
	// :ref:`the proto descriptor set
	// <config_grpc_json_reverse_transcoder_generate_proto_descriptor_set>` for the gRPC services.
	// If “descriptor_path“ is set, this field is not used.
	DescriptorBinary []byte `protobuf:"bytes,2,opt,name=descriptor_binary,json=descriptorBinary,proto3" json:"descriptor_binary,omitempty"`
	// The maximum size of a request body to be transcoded, in bytes. A body exceeding this size will
	// provoke a “gRPC status: ResourceExhausted“ response.
	//
	// Large values may cause envoy to use a lot of memory if there are many
	// concurrent requests.
	//
	// If unset, the current stream buffer size is used.
	MaxRequestBodySize *wrapperspb.UInt32Value `protobuf:"bytes,3,opt,name=max_request_body_size,json=maxRequestBodySize,proto3" json:"max_request_body_size,omitempty"`
	// The maximum size of a response body to be transcoded, in bytes. A body exceeding this size will
	// provoke a “gRPC status: Internal“ response.
	//
	// Large values may cause envoy to use a lot of memory if there are many
	// concurrent requests.
	//
	// If unset, the current stream buffer size is used.
	MaxResponseBodySize *wrapperspb.UInt32Value `protobuf:"bytes,4,opt,name=max_response_body_size,json=maxResponseBodySize,proto3" json:"max_response_body_size,omitempty"`
	// The name of the header field that has the API version of the request.
	ApiVersionHeader string `protobuf:"bytes,5,opt,name=api_version_header,json=apiVersionHeader,proto3" json:"api_version_header,omitempty"`
	// Control options for upstream request JSON. These options are passed directly to
	// `JsonPrintOptions <https://developers.google.com/protocol-buffers/docs/reference/cpp/
	// google.protobuf.util.json_util#JsonPrintOptions>`_.
	RequestJsonPrintOptions *GrpcJsonReverseTranscoder_PrintOptions `protobuf:"bytes,6,opt,name=request_json_print_options,json=requestJsonPrintOptions,proto3" json:"request_json_print_options,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *GrpcJsonReverseTranscoder) Reset() {
	*x = GrpcJsonReverseTranscoder{}
	mi := &file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GrpcJsonReverseTranscoder) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GrpcJsonReverseTranscoder) ProtoMessage() {}

func (x *GrpcJsonReverseTranscoder) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GrpcJsonReverseTranscoder.ProtoReflect.Descriptor instead.
func (*GrpcJsonReverseTranscoder) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDescGZIP(), []int{0}
}

func (x *GrpcJsonReverseTranscoder) GetDescriptorPath() string {
	if x != nil {
		return x.DescriptorPath
	}
	return ""
}

func (x *GrpcJsonReverseTranscoder) GetDescriptorBinary() []byte {
	if x != nil {
		return x.DescriptorBinary
	}
	return nil
}

func (x *GrpcJsonReverseTranscoder) GetMaxRequestBodySize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxRequestBodySize
	}
	return nil
}

func (x *GrpcJsonReverseTranscoder) GetMaxResponseBodySize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxResponseBodySize
	}
	return nil
}

func (x *GrpcJsonReverseTranscoder) GetApiVersionHeader() string {
	if x != nil {
		return x.ApiVersionHeader
	}
	return ""
}

func (x *GrpcJsonReverseTranscoder) GetRequestJsonPrintOptions() *GrpcJsonReverseTranscoder_PrintOptions {
	if x != nil {
		return x.RequestJsonPrintOptions
	}
	return nil
}

type GrpcJsonReverseTranscoder_PrintOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Whether to always print primitive fields. By default primitive
	// fields with default values will be omitted in JSON output. For
	// example, an int32 field set to 0 will be omitted. Setting this flag to
	// true will override the default behavior and print primitive fields
	// regardless of their values. Defaults to false.
	AlwaysPrintPrimitiveFields bool `protobuf:"varint,1,opt,name=always_print_primitive_fields,json=alwaysPrintPrimitiveFields,proto3" json:"always_print_primitive_fields,omitempty"`
	// Whether to always print enums as ints. By default they are rendered
	// as strings. Defaults to false.
	AlwaysPrintEnumsAsInts bool `protobuf:"varint,2,opt,name=always_print_enums_as_ints,json=alwaysPrintEnumsAsInts,proto3" json:"always_print_enums_as_ints,omitempty"`
	// Whether to convert the proto field names to “json_name“ annotation value, or lower camel case,
	// in absence of “json_name“. By default the field names will be preserved after conversion.
	// Setting this flag will convert the field names to their canonical form. Defaults to false.
	//
	// Example:
	//
	// .. code-block:: proto
	//
	//	message Author {
	//	  int64 id = 1;
	//	  enum Gender {
	//	    UNKNOWN = 0;
	//	    MALE = 1;
	//	    FEMALE = 2;
	//	  };
	//	  Gender gender = 2;
	//	  string first_name = 3;
	//	  string last_name = 4 [json_name = "lname"];
	//	}
	//
	// The above proto message after being transcoded to JSON with
	// “use_canonical_field_names“ set to “false“ will have the
	// field names same as in the proto message, as follows:
	//
	// .. code-block:: json
	//
	//	{
	//	  "id": "12345",
	//	  "gender": "MALE",
	//	  "first_name": "John",
	//	  "last_name": "Doe"
	//	}
	//
	// and with the “use_canonical_field_names“ set to “true“, the
	// transcoded JSON will have “first_name“ converted to camelCase
	// and “last_name“ converted to its “json_name“ annotation value,
	// as follows:
	//
	// .. code-block:: json
	//
	//	{
	//	  "id": "12345",
	//	  "gender": "MALE",
	//	  "firstName": "John",
	//	  "lname": "Doe"
	//	}
	UseCanonicalFieldNames bool `protobuf:"varint,3,opt,name=use_canonical_field_names,json=useCanonicalFieldNames,proto3" json:"use_canonical_field_names,omitempty"`
	unknownFields          protoimpl.UnknownFields
	sizeCache              protoimpl.SizeCache
}

func (x *GrpcJsonReverseTranscoder_PrintOptions) Reset() {
	*x = GrpcJsonReverseTranscoder_PrintOptions{}
	mi := &file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GrpcJsonReverseTranscoder_PrintOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GrpcJsonReverseTranscoder_PrintOptions) ProtoMessage() {}

func (x *GrpcJsonReverseTranscoder_PrintOptions) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GrpcJsonReverseTranscoder_PrintOptions.ProtoReflect.Descriptor instead.
func (*GrpcJsonReverseTranscoder_PrintOptions) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDescGZIP(), []int{0, 0}
}

func (x *GrpcJsonReverseTranscoder_PrintOptions) GetAlwaysPrintPrimitiveFields() bool {
	if x != nil {
		return x.AlwaysPrintPrimitiveFields
	}
	return false
}

func (x *GrpcJsonReverseTranscoder_PrintOptions) GetAlwaysPrintEnumsAsInts() bool {
	if x != nil {
		return x.AlwaysPrintEnumsAsInts
	}
	return false
}

func (x *GrpcJsonReverseTranscoder_PrintOptions) GetUseCanonicalFieldNames() bool {
	if x != nil {
		return x.UseCanonicalFieldNames
	}
	return false
}

var File_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDesc = "" +
	"\n" +
	"Nenvoy/extensions/filters/http/grpc_json_reverse_transcoder/v3/transcoder.proto\x12=envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xc5\x05\n" +
	"\x19GrpcJsonReverseTranscoder\x12'\n" +
	"\x0fdescriptor_path\x18\x01 \x01(\tR\x0edescriptorPath\x12+\n" +
	"\x11descriptor_binary\x18\x02 \x01(\fR\x10descriptorBinary\x12X\n" +
	"\x15max_request_body_size\x18\x03 \x01(\v2\x1c.google.protobuf.UInt32ValueB\a\xfaB\x04*\x02 \x00R\x12maxRequestBodySize\x12Z\n" +
	"\x16max_response_body_size\x18\x04 \x01(\v2\x1c.google.protobuf.UInt32ValueB\a\xfaB\x04*\x02 \x00R\x13maxResponseBodySize\x12,\n" +
	"\x12api_version_header\x18\x05 \x01(\tR\x10apiVersionHeader\x12\xa2\x01\n" +
	"\x1arequest_json_print_options\x18\x06 \x01(\v2e.envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3.GrpcJsonReverseTranscoder.PrintOptionsR\x17requestJsonPrintOptions\x1a\xc8\x01\n" +
	"\fPrintOptions\x12A\n" +
	"\x1dalways_print_primitive_fields\x18\x01 \x01(\bR\x1aalwaysPrintPrimitiveFields\x12:\n" +
	"\x1aalways_print_enums_as_ints\x18\x02 \x01(\bR\x16alwaysPrintEnumsAsInts\x129\n" +
	"\x19use_canonical_field_names\x18\x03 \x01(\bR\x16useCanonicalFieldNamesB\xee\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Kio.envoyproxy.envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3B\x0fTranscoderProtoP\x01Z\x83\x01github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/grpc_json_reverse_transcoder/v3;grpc_json_reverse_transcoderv3b\x06proto3"

var (
	file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDesc), len(file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDescData
}

var file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_goTypes = []any{
	(*GrpcJsonReverseTranscoder)(nil),              // 0: envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3.GrpcJsonReverseTranscoder
	(*GrpcJsonReverseTranscoder_PrintOptions)(nil), // 1: envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3.GrpcJsonReverseTranscoder.PrintOptions
	(*wrapperspb.UInt32Value)(nil),                 // 2: google.protobuf.UInt32Value
}
var file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3.GrpcJsonReverseTranscoder.max_request_body_size:type_name -> google.protobuf.UInt32Value
	2, // 1: envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3.GrpcJsonReverseTranscoder.max_response_body_size:type_name -> google.protobuf.UInt32Value
	1, // 2: envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3.GrpcJsonReverseTranscoder.request_json_print_options:type_name -> envoy.extensions.filters.http.grpc_json_reverse_transcoder.v3.GrpcJsonReverseTranscoder.PrintOptions
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_init()
}
func file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_init() {
	if File_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDesc), len(file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto = out.File
	file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_goTypes = nil
	file_envoy_extensions_filters_http_grpc_json_reverse_transcoder_v3_transcoder_proto_depIdxs = nil
}
