// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/network/connection_balance/dlb/v3alpha/dlb.proto

package v3alpha

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The fallback policy if any error occurs.
// The default policy is None.
type Dlb_FallbackPolicy int32

const (
	// No fallback policy.
	Dlb_None Dlb_FallbackPolicy = 0
	// Fall back to Nop Connection Balance.
	Dlb_NopConnectionBalance Dlb_FallbackPolicy = 1
	// Fall back to Exact Connection Balance.
	Dlb_ExactConnectionBalance Dlb_FallbackPolicy = 2
)

// Enum value maps for Dlb_FallbackPolicy.
var (
	Dlb_FallbackPolicy_name = map[int32]string{
		0: "None",
		1: "NopConnectionBalance",
		2: "ExactConnectionBalance",
	}
	Dlb_FallbackPolicy_value = map[string]int32{
		"None":                   0,
		"NopConnectionBalance":   1,
		"ExactConnectionBalance": 2,
	}
)

func (x Dlb_FallbackPolicy) Enum() *Dlb_FallbackPolicy {
	p := new(Dlb_FallbackPolicy)
	*p = x
	return p
}

func (x Dlb_FallbackPolicy) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Dlb_FallbackPolicy) Descriptor() protoreflect.EnumDescriptor {
	return file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_enumTypes[0].Descriptor()
}

func (Dlb_FallbackPolicy) Type() protoreflect.EnumType {
	return &file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_enumTypes[0]
}

func (x Dlb_FallbackPolicy) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use Dlb_FallbackPolicy.Descriptor instead.
func (Dlb_FallbackPolicy) EnumDescriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDescGZIP(), []int{0, 0}
}

// The Dlb is a hardware managed system of queues and arbiters connecting producers and consumers. It is a PCIE device
// in the CPU package. It interacts with software running on cores and potentially other devices. The Dlb implements the
// following balancing features:
//
// -  Lock-free multi-producer/multi-consumer operation.
// -  Multiple priorities for varying traffic types.
// -  Various distribution schemes.
//
// Dlb connection balancer uses Dlb hardware to balance connections, and can significantly reduce latency.
//
// As the Dlb connection balancer provides assistance from dedicated Dlb hardware, it can be used for a proxy with a large number of connections
// (e.g., a gateway).
type Dlb struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The ID of the Dlb hardware, start from 0.
	// If not specified, use the first available device as default.
	Id uint32 `protobuf:"varint,1,opt,name=id,proto3" json:"id,omitempty"`
	// Maximum number of retries when sending to DLB device fails.
	// No retry as default.
	MaxRetries     uint32             `protobuf:"varint,2,opt,name=max_retries,json=maxRetries,proto3" json:"max_retries,omitempty"`
	FallbackPolicy Dlb_FallbackPolicy `protobuf:"varint,3,opt,name=fallback_policy,json=fallbackPolicy,proto3,enum=envoy.extensions.network.connection_balance.dlb.v3alpha.Dlb_FallbackPolicy" json:"fallback_policy,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *Dlb) Reset() {
	*x = Dlb{}
	mi := &file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Dlb) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Dlb) ProtoMessage() {}

func (x *Dlb) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Dlb.ProtoReflect.Descriptor instead.
func (*Dlb) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDescGZIP(), []int{0}
}

func (x *Dlb) GetId() uint32 {
	if x != nil {
		return x.Id
	}
	return 0
}

func (x *Dlb) GetMaxRetries() uint32 {
	if x != nil {
		return x.MaxRetries
	}
	return 0
}

func (x *Dlb) GetFallbackPolicy() Dlb_FallbackPolicy {
	if x != nil {
		return x.FallbackPolicy
	}
	return Dlb_None
}

var File_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDesc = "" +
	"\n" +
	"Icontrib/envoy/extensions/network/connection_balance/dlb/v3alpha/dlb.proto\x127envoy.extensions.network.connection_balance.dlb.v3alpha\x1a\x1dudpa/annotations/status.proto\"\xfe\x01\n" +
	"\x03Dlb\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\rR\x02id\x12\x1f\n" +
	"\vmax_retries\x18\x02 \x01(\rR\n" +
	"maxRetries\x12t\n" +
	"\x0ffallback_policy\x18\x03 \x01(\x0e2K.envoy.extensions.network.connection_balance.dlb.v3alpha.Dlb.FallbackPolicyR\x0efallbackPolicy\"P\n" +
	"\x0eFallbackPolicy\x12\b\n" +
	"\x04None\x10\x00\x12\x18\n" +
	"\x14NopConnectionBalance\x10\x01\x12\x1a\n" +
	"\x16ExactConnectionBalance\x10\x02B\xc3\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Eio.envoyproxy.envoy.extensions.network.connection_balance.dlb.v3alphaB\bDlbProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/network/connection_balance/dlb/v3alphab\x06proto3"

var (
	file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDesc), len(file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDescData
}

var file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_goTypes = []any{
	(Dlb_FallbackPolicy)(0), // 0: envoy.extensions.network.connection_balance.dlb.v3alpha.Dlb.FallbackPolicy
	(*Dlb)(nil),             // 1: envoy.extensions.network.connection_balance.dlb.v3alpha.Dlb
}
var file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_depIdxs = []int32{
	0, // 0: envoy.extensions.network.connection_balance.dlb.v3alpha.Dlb.fallback_policy:type_name -> envoy.extensions.network.connection_balance.dlb.v3alpha.Dlb.FallbackPolicy
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_init() }
func file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_init() {
	if File_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDesc), len(file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_depIdxs,
		EnumInfos:         file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_enumTypes,
		MessageInfos:      file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto = out.File
	file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_goTypes = nil
	file_contrib_envoy_extensions_network_connection_balance_dlb_v3alpha_dlb_proto_depIdxs = nil
}
