// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/private_key_providers/cryptomb/v3alpha/cryptomb.proto

package v3alpha

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// A CryptoMbPrivateKeyMethodConfig message specifies how the CryptoMb private
// key provider is configured. The private key provider provides “SIMD“
// processing for ECDSA sign operations and RSA sign and decrypt operations.
// The provider works by gathering the operations into a worker-thread specific
// queue, and processing the queue using “ipp-crypto“ library when the queue
// is full or when a timer expires.
// [#extension-category: envoy.tls.key_providers]
type CryptoMbPrivateKeyMethodConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Private key to use in the private key provider. If set to inline_bytes or
	// inline_string, the value needs to be the private key in PEM format.
	PrivateKey *v3.DataSource `protobuf:"bytes,1,opt,name=private_key,json=privateKey,proto3" json:"private_key,omitempty"`
	// How long to wait until the per-thread processing queue should be
	// processed. If the processing queue gets full (eight sign or decrypt
	// requests are received) it is processed immediately. However, if the
	// queue is not filled before the delay has expired, the requests
	// already in the queue are processed, even if the queue is not full.
	// In effect, this value controls the balance between latency and
	// throughput. The duration needs to be set to a value greater than or equal to 1 millisecond.
	PollDelay     *durationpb.Duration `protobuf:"bytes,2,opt,name=poll_delay,json=pollDelay,proto3" json:"poll_delay,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CryptoMbPrivateKeyMethodConfig) Reset() {
	*x = CryptoMbPrivateKeyMethodConfig{}
	mi := &file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CryptoMbPrivateKeyMethodConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CryptoMbPrivateKeyMethodConfig) ProtoMessage() {}

func (x *CryptoMbPrivateKeyMethodConfig) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CryptoMbPrivateKeyMethodConfig.ProtoReflect.Descriptor instead.
func (*CryptoMbPrivateKeyMethodConfig) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDescGZIP(), []int{0}
}

func (x *CryptoMbPrivateKeyMethodConfig) GetPrivateKey() *v3.DataSource {
	if x != nil {
		return x.PrivateKey
	}
	return nil
}

func (x *CryptoMbPrivateKeyMethodConfig) GetPollDelay() *durationpb.Duration {
	if x != nil {
		return x.PollDelay
	}
	return nil
}

var File_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDesc = "" +
	"\n" +
	"Ncontrib/envoy/extensions/private_key_providers/cryptomb/v3alpha/cryptomb.proto\x127envoy.extensions.private_key_providers.cryptomb.v3alpha\x1a\x1fenvoy/config/core/v3/base.proto\x1a\x1egoogle/protobuf/duration.proto\x1a udpa/annotations/sensitive.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xb5\x01\n" +
	"\x1eCryptoMbPrivateKeyMethodConfig\x12I\n" +
	"\vprivate_key\x18\x01 \x01(\v2 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01R\n" +
	"privateKey\x12H\n" +
	"\n" +
	"poll_delay\x18\x02 \x01(\v2\x19.google.protobuf.DurationB\x0e\xfaB\v\xaa\x01\b\b\x012\x04\x10\xc0\x84=R\tpollDelayB\xc8\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Eio.envoyproxy.envoy.extensions.private_key_providers.cryptomb.v3alphaB\rCryptombProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/private_key_providers/cryptomb/v3alphab\x06proto3"

var (
	file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDesc), len(file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDescData
}

var file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_goTypes = []any{
	(*CryptoMbPrivateKeyMethodConfig)(nil), // 0: envoy.extensions.private_key_providers.cryptomb.v3alpha.CryptoMbPrivateKeyMethodConfig
	(*v3.DataSource)(nil),                  // 1: envoy.config.core.v3.DataSource
	(*durationpb.Duration)(nil),            // 2: google.protobuf.Duration
}
var file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.private_key_providers.cryptomb.v3alpha.CryptoMbPrivateKeyMethodConfig.private_key:type_name -> envoy.config.core.v3.DataSource
	2, // 1: envoy.extensions.private_key_providers.cryptomb.v3alpha.CryptoMbPrivateKeyMethodConfig.poll_delay:type_name -> google.protobuf.Duration
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() {
	file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_init()
}
func file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_init() {
	if File_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDesc), len(file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_depIdxs,
		MessageInfos:      file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto = out.File
	file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_goTypes = nil
	file_contrib_envoy_extensions_private_key_providers_cryptomb_v3alpha_cryptomb_proto_depIdxs = nil
}
