// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/reverse_tunnel/v3/drain_aware_hcm.proto

package reverse_tunnelv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/http_connection_manager/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the drain-aware HTTP Connection Manager.
// All HCM fields are forwarded verbatim; the only difference is that this
// filter registers a listener-drain callback and emits a GOAWAY before the
// connection is closed.
type DrainAwareHttpConnectionManager struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The underlying HCM configuration to apply.
	HcmConfig     *v3.HttpConnectionManager `protobuf:"bytes,1,opt,name=hcm_config,json=hcmConfig,proto3" json:"hcm_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DrainAwareHttpConnectionManager) Reset() {
	*x = DrainAwareHttpConnectionManager{}
	mi := &file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DrainAwareHttpConnectionManager) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DrainAwareHttpConnectionManager) ProtoMessage() {}

func (x *DrainAwareHttpConnectionManager) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DrainAwareHttpConnectionManager.ProtoReflect.Descriptor instead.
func (*DrainAwareHttpConnectionManager) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDescGZIP(), []int{0}
}

func (x *DrainAwareHttpConnectionManager) GetHcmConfig() *v3.HttpConnectionManager {
	if x != nil {
		return x.HcmConfig
	}
	return nil
}

var File_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDesc = "" +
	"\n" +
	"Henvoy/extensions/filters/network/reverse_tunnel/v3/drain_aware_hcm.proto\x122envoy.extensions.filters.network.reverse_tunnel.v3\x1aYenvoy/extensions/filters/network/http_connection_manager/v3/http_connection_manager.proto\x1a\x1dudpa/annotations/status.proto\"\x94\x01\n" +
	"\x1fDrainAwareHttpConnectionManager\x12q\n" +
	"\n" +
	"hcm_config\x18\x01 \x01(\v2R.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManagerR\thcmConfigB\xcc\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"@io.envoyproxy.envoy.extensions.filters.network.reverse_tunnel.v3B\x12DrainAwareHcmProtoP\x01Zjgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/reverse_tunnel/v3;reverse_tunnelv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDesc), len(file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDescData
}

var file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_goTypes = []any{
	(*DrainAwareHttpConnectionManager)(nil), // 0: envoy.extensions.filters.network.reverse_tunnel.v3.DrainAwareHttpConnectionManager
	(*v3.HttpConnectionManager)(nil),        // 1: envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager
}
var file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.reverse_tunnel.v3.DrainAwareHttpConnectionManager.hcm_config:type_name -> envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_init() }
func file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_init() {
	if File_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDesc), len(file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto = out.File
	file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_goTypes = nil
	file_envoy_extensions_filters_network_reverse_tunnel_v3_drain_aware_hcm_proto_depIdxs = nil
}
