// Code generated by protoc-gen-jsonshim. DO NOT EDIT.
package v1alpha1

import (
	bytes "bytes"
	jsonpb "github.com/golang/protobuf/jsonpb"
)

// MarshalJSON is a custom marshaler for Metadata
func (this *Metadata) MarshalJSON() ([]byte, error) {
	str, err := MetadataMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for Metadata
func (this *Metadata) UnmarshalJSON(b []byte) error {
	return MetadataUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

var (
	MetadataMarshaler   = &jsonpb.Marshaler{}
	MetadataUnmarshaler = &jsonpb.Unmarshaler{AllowUnknownFields: true}
)
