// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/data/tap/v3/common.proto

package tapv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Wrapper for tapped body data. This includes HTTP request/response body, transport socket received
// and transmitted data, etc.
type Body struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to BodyType:
	//
	//	*Body_AsBytes
	//	*Body_AsString
	BodyType isBody_BodyType `protobuf_oneof:"body_type"`
	// Specifies whether body data has been truncated to fit within the specified
	// :ref:`max_buffered_rx_bytes
	// <envoy_v3_api_field_config.tap.v3.OutputConfig.max_buffered_rx_bytes>` and
	// :ref:`max_buffered_tx_bytes
	// <envoy_v3_api_field_config.tap.v3.OutputConfig.max_buffered_tx_bytes>` settings.
	Truncated     bool `protobuf:"varint,3,opt,name=truncated,proto3" json:"truncated,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Body) Reset() {
	*x = Body{}
	mi := &file_envoy_data_tap_v3_common_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Body) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Body) ProtoMessage() {}

func (x *Body) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_tap_v3_common_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Body.ProtoReflect.Descriptor instead.
func (*Body) Descriptor() ([]byte, []int) {
	return file_envoy_data_tap_v3_common_proto_rawDescGZIP(), []int{0}
}

func (x *Body) GetBodyType() isBody_BodyType {
	if x != nil {
		return x.BodyType
	}
	return nil
}

func (x *Body) GetAsBytes() []byte {
	if x != nil {
		if x, ok := x.BodyType.(*Body_AsBytes); ok {
			return x.AsBytes
		}
	}
	return nil
}

func (x *Body) GetAsString() string {
	if x != nil {
		if x, ok := x.BodyType.(*Body_AsString); ok {
			return x.AsString
		}
	}
	return ""
}

func (x *Body) GetTruncated() bool {
	if x != nil {
		return x.Truncated
	}
	return false
}

type isBody_BodyType interface {
	isBody_BodyType()
}

type Body_AsBytes struct {
	// Body data as bytes. By default, tap body data will be present in this field, as the proto
	// “bytes“ type can contain any valid byte.
	AsBytes []byte `protobuf:"bytes,1,opt,name=as_bytes,json=asBytes,proto3,oneof"`
}

type Body_AsString struct {
	// Body data as string. This field is only used when the :ref:`JSON_BODY_AS_STRING
	// <envoy_v3_api_enum_value_config.tap.v3.OutputSink.Format.JSON_BODY_AS_STRING>` sink
	// format type is selected. See the documentation for that option for why this is useful.
	AsString string `protobuf:"bytes,2,opt,name=as_string,json=asString,proto3,oneof"`
}

func (*Body_AsBytes) isBody_BodyType() {}

func (*Body_AsString) isBody_BodyType() {}

// Connection properties.
type Connection struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Local address.
	LocalAddress *v3.Address `protobuf:"bytes,1,opt,name=local_address,json=localAddress,proto3" json:"local_address,omitempty"`
	// Remote address.
	RemoteAddress *v3.Address `protobuf:"bytes,2,opt,name=remote_address,json=remoteAddress,proto3" json:"remote_address,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Connection) Reset() {
	*x = Connection{}
	mi := &file_envoy_data_tap_v3_common_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Connection) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Connection) ProtoMessage() {}

func (x *Connection) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_tap_v3_common_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Connection.ProtoReflect.Descriptor instead.
func (*Connection) Descriptor() ([]byte, []int) {
	return file_envoy_data_tap_v3_common_proto_rawDescGZIP(), []int{1}
}

func (x *Connection) GetLocalAddress() *v3.Address {
	if x != nil {
		return x.LocalAddress
	}
	return nil
}

func (x *Connection) GetRemoteAddress() *v3.Address {
	if x != nil {
		return x.RemoteAddress
	}
	return nil
}

var File_envoy_data_tap_v3_common_proto protoreflect.FileDescriptor

const file_envoy_data_tap_v3_common_proto_rawDesc = "" +
	"\n" +
	"\x1eenvoy/data/tap/v3/common.proto\x12\x11envoy.data.tap.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\x91\x01\n" +
	"\x04Body\x12\x1b\n" +
	"\bas_bytes\x18\x01 \x01(\fH\x00R\aasBytes\x12\x1d\n" +
	"\tas_string\x18\x02 \x01(\tH\x00R\basString\x12\x1c\n" +
	"\ttruncated\x18\x03 \x01(\bR\ttruncated:\"\x9aň\x1e\x1d\n" +
	"\x1benvoy.data.tap.v2alpha.BodyB\v\n" +
	"\tbody_type\"\xc0\x01\n" +
	"\n" +
	"Connection\x12B\n" +
	"\rlocal_address\x18\x01 \x01(\v2\x1d.envoy.config.core.v3.AddressR\flocalAddress\x12D\n" +
	"\x0eremote_address\x18\x02 \x01(\v2\x1d.envoy.config.core.v3.AddressR\rremoteAddress:(\x9aň\x1e#\n" +
	"!envoy.data.tap.v2alpha.ConnectionBx\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\x1fio.envoyproxy.envoy.data.tap.v3B\vCommonProtoP\x01Z>github.com/envoyproxy/go-control-plane/envoy/data/tap/v3;tapv3b\x06proto3"

var (
	file_envoy_data_tap_v3_common_proto_rawDescOnce sync.Once
	file_envoy_data_tap_v3_common_proto_rawDescData []byte
)

func file_envoy_data_tap_v3_common_proto_rawDescGZIP() []byte {
	file_envoy_data_tap_v3_common_proto_rawDescOnce.Do(func() {
		file_envoy_data_tap_v3_common_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_data_tap_v3_common_proto_rawDesc), len(file_envoy_data_tap_v3_common_proto_rawDesc)))
	})
	return file_envoy_data_tap_v3_common_proto_rawDescData
}

var file_envoy_data_tap_v3_common_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_data_tap_v3_common_proto_goTypes = []any{
	(*Body)(nil),       // 0: envoy.data.tap.v3.Body
	(*Connection)(nil), // 1: envoy.data.tap.v3.Connection
	(*v3.Address)(nil), // 2: envoy.config.core.v3.Address
}
var file_envoy_data_tap_v3_common_proto_depIdxs = []int32{
	2, // 0: envoy.data.tap.v3.Connection.local_address:type_name -> envoy.config.core.v3.Address
	2, // 1: envoy.data.tap.v3.Connection.remote_address:type_name -> envoy.config.core.v3.Address
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_data_tap_v3_common_proto_init() }
func file_envoy_data_tap_v3_common_proto_init() {
	if File_envoy_data_tap_v3_common_proto != nil {
		return
	}
	file_envoy_data_tap_v3_common_proto_msgTypes[0].OneofWrappers = []any{
		(*Body_AsBytes)(nil),
		(*Body_AsString)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_data_tap_v3_common_proto_rawDesc), len(file_envoy_data_tap_v3_common_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_data_tap_v3_common_proto_goTypes,
		DependencyIndexes: file_envoy_data_tap_v3_common_proto_depIdxs,
		MessageInfos:      file_envoy_data_tap_v3_common_proto_msgTypes,
	}.Build()
	File_envoy_data_tap_v3_common_proto = out.File
	file_envoy_data_tap_v3_common_proto_goTypes = nil
	file_envoy_data_tap_v3_common_proto_depIdxs = nil
}
