// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/dynamic_modules/v3/dynamic_modules.proto

package dynamic_modulesv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/dynamic_modules/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the Dynamic Modules network filter. This filter allows loading shared object
// files that can be loaded via “dlopen“ to extend the network filter chain.
//
// A module can be loaded by multiple network filters; the module is loaded only once and shared
// across multiple filters.
//
// Unlike HTTP filters which operate on structured headers, body, and trailers, network filters work
// with raw TCP byte streams. The filter can:
//
// * Inspect, modify, or inject data into the downstream connection.
// * Access connection-level information such as addresses and TLS status.
// * Control connection lifecycle (e.g., close the connection).
type DynamicModuleNetworkFilter struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies the shared-object level configuration.
	DynamicModuleConfig *v3.DynamicModuleConfig `protobuf:"bytes,1,opt,name=dynamic_module_config,json=dynamicModuleConfig,proto3" json:"dynamic_module_config,omitempty"`
	// The name for this filter configuration.
	//
	// This can be used to distinguish between different filter implementations inside a dynamic
	// module. For example, a module can have completely different filter implementations. When Envoy
	// receives this configuration, it passes the “filter_name“ to the dynamic module's network
	// filter config init function together with the “filter_config“. That way a module can decide
	// which in-module filter implementation to use based on the name at load time.
	FilterName string `protobuf:"bytes,2,opt,name=filter_name,json=filterName,proto3" json:"filter_name,omitempty"`
	// The configuration for the filter chosen by “filter_name“.
	//
	// This is passed to the module's network filter initialization function. Together with the
	// “filter_name“, the module can decide which in-module filter implementation to use and
	// fine-tune the behavior of the filter.
	//
	// For example, if a module has two filter implementations, one for echo and one for rate
	// limiting, “filter_name“ is used to choose either echo or rate limiting. The
	// “filter_config“ can be used to configure the echo behavior or the rate limiting parameters.
	//
	// “google.protobuf.Struct“ is serialized as JSON before passing it to the module.
	// “google.protobuf.BytesValue“ and “google.protobuf.StringValue“ are passed directly
	// without the wrapper.
	//
	// .. code-block:: yaml
	//
	//	# Passing a string value
	//	filter_config:
	//	  "@type": "type.googleapis.com/google.protobuf.StringValue"
	//	  value: hello
	//
	//	# Passing raw bytes
	//	filter_config:
	//	  "@type": "type.googleapis.com/google.protobuf.BytesValue"
	//	  value: aGVsbG8=  # echo -n "hello" | base64
	FilterConfig *anypb.Any `protobuf:"bytes,3,opt,name=filter_config,json=filterConfig,proto3" json:"filter_config,omitempty"`
	// If “true“, the dynamic module is a terminal filter to use without an upstream connection.
	//
	// The dynamic module is responsible for creating and sending the response to downstream.
	//
	// Defaults to “false“.
	TerminalFilter bool `protobuf:"varint,4,opt,name=terminal_filter,json=terminalFilter,proto3" json:"terminal_filter,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *DynamicModuleNetworkFilter) Reset() {
	*x = DynamicModuleNetworkFilter{}
	mi := &file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DynamicModuleNetworkFilter) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DynamicModuleNetworkFilter) ProtoMessage() {}

func (x *DynamicModuleNetworkFilter) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DynamicModuleNetworkFilter.ProtoReflect.Descriptor instead.
func (*DynamicModuleNetworkFilter) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDescGZIP(), []int{0}
}

func (x *DynamicModuleNetworkFilter) GetDynamicModuleConfig() *v3.DynamicModuleConfig {
	if x != nil {
		return x.DynamicModuleConfig
	}
	return nil
}

func (x *DynamicModuleNetworkFilter) GetFilterName() string {
	if x != nil {
		return x.FilterName
	}
	return ""
}

func (x *DynamicModuleNetworkFilter) GetFilterConfig() *anypb.Any {
	if x != nil {
		return x.FilterConfig
	}
	return nil
}

func (x *DynamicModuleNetworkFilter) GetTerminalFilter() bool {
	if x != nil {
		return x.TerminalFilter
	}
	return false
}

var File_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDesc = "" +
	"\n" +
	"Ienvoy/extensions/filters/network/dynamic_modules/v3/dynamic_modules.proto\x123envoy.extensions.filters.network.dynamic_modules.v3\x1a9envoy/extensions/dynamic_modules/v3/dynamic_modules.proto\x1a\x19google/protobuf/any.proto\x1a\x1dudpa/annotations/status.proto\"\x8f\x02\n" +
	"\x1aDynamicModuleNetworkFilter\x12l\n" +
	"\x15dynamic_module_config\x18\x01 \x01(\v28.envoy.extensions.dynamic_modules.v3.DynamicModuleConfigR\x13dynamicModuleConfig\x12\x1f\n" +
	"\vfilter_name\x18\x02 \x01(\tR\n" +
	"filterName\x129\n" +
	"\rfilter_config\x18\x03 \x01(\v2\x14.google.protobuf.AnyR\ffilterConfig\x12'\n" +
	"\x0fterminal_filter\x18\x04 \x01(\bR\x0eterminalFilterB\xd0\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Aio.envoyproxy.envoy.extensions.filters.network.dynamic_modules.v3B\x13DynamicModulesProtoP\x01Zlgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/dynamic_modules/v3;dynamic_modulesv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDesc), len(file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDescData
}

var file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_goTypes = []any{
	(*DynamicModuleNetworkFilter)(nil), // 0: envoy.extensions.filters.network.dynamic_modules.v3.DynamicModuleNetworkFilter
	(*v3.DynamicModuleConfig)(nil),     // 1: envoy.extensions.dynamic_modules.v3.DynamicModuleConfig
	(*anypb.Any)(nil),                  // 2: google.protobuf.Any
}
var file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.dynamic_modules.v3.DynamicModuleNetworkFilter.dynamic_module_config:type_name -> envoy.extensions.dynamic_modules.v3.DynamicModuleConfig
	2, // 1: envoy.extensions.filters.network.dynamic_modules.v3.DynamicModuleNetworkFilter.filter_config:type_name -> google.protobuf.Any
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_init() }
func file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_init() {
	if File_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDesc), len(file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto = out.File
	file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_goTypes = nil
	file_envoy_extensions_filters_network_dynamic_modules_v3_dynamic_modules_proto_depIdxs = nil
}
