// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/matching/common_inputs/transport_socket/v3/transport_socket_inputs.proto

package transport_socketv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Specifies that matching should be performed by the endpoint metadata.
// This input extracts metadata from the selected endpoint for transport socket selection.
// The metadata is extracted using a filter and path specification similar to
// :ref:`DynamicMetadataInput <envoy_v3_api_msg_extensions.matching.common_inputs.network.v3.DynamicMetadataInput>`.
//
// Example: Extract a metadata value for transport socket matching.
//
// .. code-block:: yaml
//
//	typed_config:
//	  "@type": type.googleapis.com/envoy.extensions.matching.common_inputs.transport_socket.v3.EndpointMetadataInput
//	  filter: "envoy.transport_socket_match"
//	  path:
//	    - key: "socket_type"
//
// This configuration extracts the value at path “["envoy.transport_socket_match"]["socket_type"]“
// from the endpoint metadata for use in transport socket selection.
//
// [#extension: envoy.matching.inputs.endpoint_metadata]
type EndpointMetadataInput struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The filter name to retrieve the Struct from the endpoint metadata.
	// If not specified, defaults to “envoy.lb“ which is commonly used for load balancing metadata.
	Filter string `protobuf:"bytes,1,opt,name=filter,proto3" json:"filter,omitempty"`
	// The path to retrieve the Value from the Struct.
	Path          []*EndpointMetadataInput_PathSegment `protobuf:"bytes,2,rep,name=path,proto3" json:"path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EndpointMetadataInput) Reset() {
	*x = EndpointMetadataInput{}
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EndpointMetadataInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EndpointMetadataInput) ProtoMessage() {}

func (x *EndpointMetadataInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EndpointMetadataInput.ProtoReflect.Descriptor instead.
func (*EndpointMetadataInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescGZIP(), []int{0}
}

func (x *EndpointMetadataInput) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

func (x *EndpointMetadataInput) GetPath() []*EndpointMetadataInput_PathSegment {
	if x != nil {
		return x.Path
	}
	return nil
}

// Specifies that matching should be performed by the locality metadata.
// This input extracts metadata from the endpoint's locality for transport socket selection.
// The metadata is extracted using a filter and path specification similar to
// :ref:`DynamicMetadataInput <envoy_v3_api_msg_extensions.matching.common_inputs.network.v3.DynamicMetadataInput>`.
//
// Example: Extract a metadata value from locality for transport socket matching.
//
// .. code-block:: yaml
//
//	typed_config:
//	  "@type": type.googleapis.com/envoy.extensions.matching.common_inputs.transport_socket.v3.LocalityMetadataInput
//	  filter: "envoy.transport_socket_match"
//	  path:
//	    - key: "region"
//
// This configuration extracts the value at path “["envoy.transport_socket_match"]["region"]“
// from the locality metadata for use in transport socket selection.
//
// [#extension: envoy.matching.inputs.locality_metadata]
type LocalityMetadataInput struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The filter name to retrieve the Struct from the locality metadata.
	// If not specified, defaults to “envoy.lb“ which is commonly used for load balancing metadata.
	Filter string `protobuf:"bytes,1,opt,name=filter,proto3" json:"filter,omitempty"`
	// The path to retrieve the Value from the Struct.
	Path          []*LocalityMetadataInput_PathSegment `protobuf:"bytes,2,rep,name=path,proto3" json:"path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LocalityMetadataInput) Reset() {
	*x = LocalityMetadataInput{}
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LocalityMetadataInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LocalityMetadataInput) ProtoMessage() {}

func (x *LocalityMetadataInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LocalityMetadataInput.ProtoReflect.Descriptor instead.
func (*LocalityMetadataInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescGZIP(), []int{1}
}

func (x *LocalityMetadataInput) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

func (x *LocalityMetadataInput) GetPath() []*LocalityMetadataInput_PathSegment {
	if x != nil {
		return x.Path
	}
	return nil
}

// Specifies that matching should be performed by filter state.
// This input extracts a value from filter state that was explicitly shared from the
// downstream connection to the upstream connection via “TransportSocketOptions“.
// This enables flexible downstream-connection-based transport socket selection,
// such as matching on network namespace or any custom filter state data.
//
// Example: Match on network namespace stored in filter state.
//
// .. code-block:: yaml
//
//	typed_config:
//	  "@type": type.googleapis.com/envoy.extensions.matching.common_inputs.transport_socket.v3.FilterStateInput
//	  key: "envoy.network.namespace"
//
// [#extension: envoy.matching.inputs.transport_socket_filter_state]
type FilterStateInput struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The key of the filter state object to retrieve.
	// The object must implement serializeAsString() to be used for matching.
	Key           string `protobuf:"bytes,1,opt,name=key,proto3" json:"key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FilterStateInput) Reset() {
	*x = FilterStateInput{}
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FilterStateInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FilterStateInput) ProtoMessage() {}

func (x *FilterStateInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FilterStateInput.ProtoReflect.Descriptor instead.
func (*FilterStateInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescGZIP(), []int{2}
}

func (x *FilterStateInput) GetKey() string {
	if x != nil {
		return x.Key
	}
	return ""
}

// Configuration for the transport socket name action.
// This action sets the name of the transport socket to use when the matcher matches.
// [#extension: envoy.matching.action.transport_socket.name]
type TransportSocketNameAction struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the transport socket to use.
	// This name must reference a named transport socket in the cluster's transport_socket_matches.
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TransportSocketNameAction) Reset() {
	*x = TransportSocketNameAction{}
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TransportSocketNameAction) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TransportSocketNameAction) ProtoMessage() {}

func (x *TransportSocketNameAction) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TransportSocketNameAction.ProtoReflect.Descriptor instead.
func (*TransportSocketNameAction) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescGZIP(), []int{3}
}

func (x *TransportSocketNameAction) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// Specifies the segment in a path to retrieve value from Metadata.
// Note: Currently it's not supported to retrieve a value from a list in Metadata. This means that
// if the segment key refers to a list, it has to be the last segment in a path.
type EndpointMetadataInput_PathSegment struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Segment:
	//
	//	*EndpointMetadataInput_PathSegment_Key
	Segment       isEndpointMetadataInput_PathSegment_Segment `protobuf_oneof:"segment"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EndpointMetadataInput_PathSegment) Reset() {
	*x = EndpointMetadataInput_PathSegment{}
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EndpointMetadataInput_PathSegment) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EndpointMetadataInput_PathSegment) ProtoMessage() {}

func (x *EndpointMetadataInput_PathSegment) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EndpointMetadataInput_PathSegment.ProtoReflect.Descriptor instead.
func (*EndpointMetadataInput_PathSegment) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescGZIP(), []int{0, 0}
}

func (x *EndpointMetadataInput_PathSegment) GetSegment() isEndpointMetadataInput_PathSegment_Segment {
	if x != nil {
		return x.Segment
	}
	return nil
}

func (x *EndpointMetadataInput_PathSegment) GetKey() string {
	if x != nil {
		if x, ok := x.Segment.(*EndpointMetadataInput_PathSegment_Key); ok {
			return x.Key
		}
	}
	return ""
}

type isEndpointMetadataInput_PathSegment_Segment interface {
	isEndpointMetadataInput_PathSegment_Segment()
}

type EndpointMetadataInput_PathSegment_Key struct {
	// If specified, use the key to retrieve the value in a Struct.
	Key string `protobuf:"bytes,1,opt,name=key,proto3,oneof"`
}

func (*EndpointMetadataInput_PathSegment_Key) isEndpointMetadataInput_PathSegment_Segment() {}

// Specifies the segment in a path to retrieve value from Metadata.
// Note: Currently it's not supported to retrieve a value from a list in Metadata. This means that
// if the segment key refers to a list, it has to be the last segment in a path.
type LocalityMetadataInput_PathSegment struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Segment:
	//
	//	*LocalityMetadataInput_PathSegment_Key
	Segment       isLocalityMetadataInput_PathSegment_Segment `protobuf_oneof:"segment"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LocalityMetadataInput_PathSegment) Reset() {
	*x = LocalityMetadataInput_PathSegment{}
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LocalityMetadataInput_PathSegment) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LocalityMetadataInput_PathSegment) ProtoMessage() {}

func (x *LocalityMetadataInput_PathSegment) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LocalityMetadataInput_PathSegment.ProtoReflect.Descriptor instead.
func (*LocalityMetadataInput_PathSegment) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescGZIP(), []int{1, 0}
}

func (x *LocalityMetadataInput_PathSegment) GetSegment() isLocalityMetadataInput_PathSegment_Segment {
	if x != nil {
		return x.Segment
	}
	return nil
}

func (x *LocalityMetadataInput_PathSegment) GetKey() string {
	if x != nil {
		if x, ok := x.Segment.(*LocalityMetadataInput_PathSegment_Key); ok {
			return x.Key
		}
	}
	return ""
}

type isLocalityMetadataInput_PathSegment_Segment interface {
	isLocalityMetadataInput_PathSegment_Segment()
}

type LocalityMetadataInput_PathSegment_Key struct {
	// If specified, use the key to retrieve the value in a Struct.
	Key string `protobuf:"bytes,1,opt,name=key,proto3,oneof"`
}

func (*LocalityMetadataInput_PathSegment_Key) isLocalityMetadataInput_PathSegment_Segment() {}

var File_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto protoreflect.FileDescriptor

const file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDesc = "" +
	"\n" +
	"Yenvoy/extensions/matching/common_inputs/transport_socket/v3/transport_socket_inputs.proto\x12;envoy.extensions.matching.common_inputs.transport_socket.v3\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe9\x01\n" +
	"\x15EndpointMetadataInput\x12\x16\n" +
	"\x06filter\x18\x01 \x01(\tR\x06filter\x12|\n" +
	"\x04path\x18\x02 \x03(\v2^.envoy.extensions.matching.common_inputs.transport_socket.v3.EndpointMetadataInput.PathSegmentB\b\xfaB\x05\x92\x01\x02\b\x01R\x04path\x1a:\n" +
	"\vPathSegment\x12\x1b\n" +
	"\x03key\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\x03keyB\x0e\n" +
	"\asegment\x12\x03\xf8B\x01\"\xe9\x01\n" +
	"\x15LocalityMetadataInput\x12\x16\n" +
	"\x06filter\x18\x01 \x01(\tR\x06filter\x12|\n" +
	"\x04path\x18\x02 \x03(\v2^.envoy.extensions.matching.common_inputs.transport_socket.v3.LocalityMetadataInput.PathSegmentB\b\xfaB\x05\x92\x01\x02\b\x01R\x04path\x1a:\n" +
	"\vPathSegment\x12\x1b\n" +
	"\x03key\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\x03keyB\x0e\n" +
	"\asegment\x12\x03\xf8B\x01\"-\n" +
	"\x10FilterStateInput\x12\x19\n" +
	"\x03key\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x03key\"8\n" +
	"\x19TransportSocketNameAction\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04nameB\xe8\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Iio.envoyproxy.envoy.extensions.matching.common_inputs.transport_socket.v3B\x1aTransportSocketInputsProtoP\x01Zugithub.com/envoyproxy/go-control-plane/envoy/extensions/matching/common_inputs/transport_socket/v3;transport_socketv3b\x06proto3"

var (
	file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescOnce sync.Once
	file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescData []byte
)

func file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescGZIP() []byte {
	file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDesc), len(file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDesc)))
	})
	return file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDescData
}

var file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes = make([]protoimpl.MessageInfo, 6)
var file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_goTypes = []any{
	(*EndpointMetadataInput)(nil),             // 0: envoy.extensions.matching.common_inputs.transport_socket.v3.EndpointMetadataInput
	(*LocalityMetadataInput)(nil),             // 1: envoy.extensions.matching.common_inputs.transport_socket.v3.LocalityMetadataInput
	(*FilterStateInput)(nil),                  // 2: envoy.extensions.matching.common_inputs.transport_socket.v3.FilterStateInput
	(*TransportSocketNameAction)(nil),         // 3: envoy.extensions.matching.common_inputs.transport_socket.v3.TransportSocketNameAction
	(*EndpointMetadataInput_PathSegment)(nil), // 4: envoy.extensions.matching.common_inputs.transport_socket.v3.EndpointMetadataInput.PathSegment
	(*LocalityMetadataInput_PathSegment)(nil), // 5: envoy.extensions.matching.common_inputs.transport_socket.v3.LocalityMetadataInput.PathSegment
}
var file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_depIdxs = []int32{
	4, // 0: envoy.extensions.matching.common_inputs.transport_socket.v3.EndpointMetadataInput.path:type_name -> envoy.extensions.matching.common_inputs.transport_socket.v3.EndpointMetadataInput.PathSegment
	5, // 1: envoy.extensions.matching.common_inputs.transport_socket.v3.LocalityMetadataInput.path:type_name -> envoy.extensions.matching.common_inputs.transport_socket.v3.LocalityMetadataInput.PathSegment
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_init()
}
func file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_init() {
	if File_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto != nil {
		return
	}
	file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[4].OneofWrappers = []any{
		(*EndpointMetadataInput_PathSegment_Key)(nil),
	}
	file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes[5].OneofWrappers = []any{
		(*LocalityMetadataInput_PathSegment_Key)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDesc), len(file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   6,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_msgTypes,
	}.Build()
	File_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto = out.File
	file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_goTypes = nil
	file_envoy_extensions_matching_common_inputs_transport_socket_v3_transport_socket_inputs_proto_depIdxs = nil
}
