// Code generated by protoc-gen-jsonshim. DO NOT EDIT.
package v1alpha1

import (
	bytes "bytes"
	jsonpb "github.com/golang/protobuf/jsonpb"
)

// MarshalJSON is a custom marshaler for Network
func (this *Network) MarshalJSON() ([]byte, error) {
	str, err := NetworkMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for Network
func (this *Network) UnmarshalJSON(b []byte) error {
	return NetworkUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for Network_NetworkEndpoints
func (this *Network_NetworkEndpoints) MarshalJSON() ([]byte, error) {
	str, err := NetworkMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for Network_NetworkEndpoints
func (this *Network_NetworkEndpoints) UnmarshalJSON(b []byte) error {
	return NetworkUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for Network_IstioNetworkGateway
func (this *Network_IstioNetworkGateway) MarshalJSON() ([]byte, error) {
	str, err := NetworkMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for Network_IstioNetworkGateway
func (this *Network_IstioNetworkGateway) UnmarshalJSON(b []byte) error {
	return NetworkUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for MeshNetworks
func (this *MeshNetworks) MarshalJSON() ([]byte, error) {
	str, err := NetworkMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for MeshNetworks
func (this *MeshNetworks) UnmarshalJSON(b []byte) error {
	return NetworkUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

var (
	NetworkMarshaler   = &jsonpb.Marshaler{}
	NetworkUnmarshaler = &jsonpb.Unmarshaler{AllowUnknownFields: true}
)
