/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1alpha2

// InferencePoolStatusApplyConfiguration represents a declarative configuration of the InferencePoolStatus type for use
// with apply.
//
// InferencePoolStatus defines the observed state of InferencePool.
type InferencePoolStatusApplyConfiguration struct {
	// Parents is a list of parent resources (usually Gateways) that are
	// associated with the InferencePool, and the status of the InferencePool with respect to
	// each parent.
	//
	// A maximum of 32 Gateways will be represented in this list. When the list contains
	// `kind: Status, name: default`, it indicates that the InferencePool is not
	// associated with any Gateway and a controller must perform the following:
	//
	// - Remove the parent when setting the "Accepted" condition.
	// - Add the parent when the controller will no longer manage the InferencePool
	// and no other parents exist.
	Parents []PoolStatusApplyConfiguration `json:"parent,omitempty"`
}

// InferencePoolStatusApplyConfiguration constructs a declarative configuration of the InferencePoolStatus type for use with
// apply.
func InferencePoolStatus() *InferencePoolStatusApplyConfiguration {
	return &InferencePoolStatusApplyConfiguration{}
}

// WithParents adds the given value to the Parents field in the declarative configuration
// and returns the receiver, so that objects can be build by chaining "With" function invocations.
// If called multiple times, values provided by each call will be appended to the Parents field.
func (b *InferencePoolStatusApplyConfiguration) WithParents(values ...*PoolStatusApplyConfiguration) *InferencePoolStatusApplyConfiguration {
	for i := range values {
		if values[i] == nil {
			panic("nil value passed to WithParents")
		}
		b.Parents = append(b.Parents, *values[i])
	}
	return b
}
