// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/data/core/v3/tlv_metadata.proto

package corev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type TlvsMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Typed metadata for :ref:`Proxy protocol filter <envoy_v3_api_msg_extensions.filters.listener.proxy_protocol.v3.ProxyProtocol>`, that represents a map of TLVs.
	// Each entry in the map consists of a key which corresponds to a configured
	// :ref:`rule key <envoy_v3_api_field_extensions.filters.listener.proxy_protocol.v3.ProxyProtocol.KeyValuePair.key>` and a value (TLV value in bytes).
	// :ref:`Proxy protocol filter <envoy_v3_api_msg_extensions.filters.listener.proxy_protocol.v3.ProxyProtocol>`
	// populates both typed and untyped metadata.
	TypedMetadata map[string][]byte `protobuf:"bytes,1,rep,name=typed_metadata,json=typedMetadata,proto3" json:"typed_metadata,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TlvsMetadata) Reset() {
	*x = TlvsMetadata{}
	mi := &file_envoy_data_core_v3_tlv_metadata_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TlvsMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TlvsMetadata) ProtoMessage() {}

func (x *TlvsMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_core_v3_tlv_metadata_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TlvsMetadata.ProtoReflect.Descriptor instead.
func (*TlvsMetadata) Descriptor() ([]byte, []int) {
	return file_envoy_data_core_v3_tlv_metadata_proto_rawDescGZIP(), []int{0}
}

func (x *TlvsMetadata) GetTypedMetadata() map[string][]byte {
	if x != nil {
		return x.TypedMetadata
	}
	return nil
}

var File_envoy_data_core_v3_tlv_metadata_proto protoreflect.FileDescriptor

const file_envoy_data_core_v3_tlv_metadata_proto_rawDesc = "" +
	"\n" +
	"%envoy/data/core/v3/tlv_metadata.proto\x12\x12envoy.data.core.v3\x1a\x1dudpa/annotations/status.proto\"\xac\x01\n" +
	"\fTlvsMetadata\x12Z\n" +
	"\x0etyped_metadata\x18\x01 \x03(\v23.envoy.data.core.v3.TlvsMetadata.TypedMetadataEntryR\rtypedMetadata\x1a@\n" +
	"\x12TypedMetadataEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\fR\x05value:\x028\x01B\x80\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	" io.envoyproxy.envoy.data.core.v3B\x10TlvMetadataProtoP\x01Z@github.com/envoyproxy/go-control-plane/envoy/data/core/v3;corev3b\x06proto3"

var (
	file_envoy_data_core_v3_tlv_metadata_proto_rawDescOnce sync.Once
	file_envoy_data_core_v3_tlv_metadata_proto_rawDescData []byte
)

func file_envoy_data_core_v3_tlv_metadata_proto_rawDescGZIP() []byte {
	file_envoy_data_core_v3_tlv_metadata_proto_rawDescOnce.Do(func() {
		file_envoy_data_core_v3_tlv_metadata_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_data_core_v3_tlv_metadata_proto_rawDesc), len(file_envoy_data_core_v3_tlv_metadata_proto_rawDesc)))
	})
	return file_envoy_data_core_v3_tlv_metadata_proto_rawDescData
}

var file_envoy_data_core_v3_tlv_metadata_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_data_core_v3_tlv_metadata_proto_goTypes = []any{
	(*TlvsMetadata)(nil), // 0: envoy.data.core.v3.TlvsMetadata
	nil,                  // 1: envoy.data.core.v3.TlvsMetadata.TypedMetadataEntry
}
var file_envoy_data_core_v3_tlv_metadata_proto_depIdxs = []int32{
	1, // 0: envoy.data.core.v3.TlvsMetadata.typed_metadata:type_name -> envoy.data.core.v3.TlvsMetadata.TypedMetadataEntry
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_data_core_v3_tlv_metadata_proto_init() }
func file_envoy_data_core_v3_tlv_metadata_proto_init() {
	if File_envoy_data_core_v3_tlv_metadata_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_data_core_v3_tlv_metadata_proto_rawDesc), len(file_envoy_data_core_v3_tlv_metadata_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_data_core_v3_tlv_metadata_proto_goTypes,
		DependencyIndexes: file_envoy_data_core_v3_tlv_metadata_proto_depIdxs,
		MessageInfos:      file_envoy_data_core_v3_tlv_metadata_proto_msgTypes,
	}.Build()
	File_envoy_data_core_v3_tlv_metadata_proto = out.File
	file_envoy_data_core_v3_tlv_metadata_proto_goTypes = nil
	file_envoy_data_core_v3_tlv_metadata_proto_depIdxs = nil
}
