// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/quic/server_preferred_address/v3/datasource.proto

package server_preferred_addressv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for DataSourceServerPreferredAddressConfig.
type DataSourceServerPreferredAddressConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The IPv4 address to advertise to clients for Server Preferred Address.
	Ipv4Config *DataSourceServerPreferredAddressConfig_AddressFamilyConfig `protobuf:"bytes,1,opt,name=ipv4_config,json=ipv4Config,proto3" json:"ipv4_config,omitempty"`
	// The IPv6 address to advertise to clients for Server Preferred Address.
	Ipv6Config    *DataSourceServerPreferredAddressConfig_AddressFamilyConfig `protobuf:"bytes,2,opt,name=ipv6_config,json=ipv6Config,proto3" json:"ipv6_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DataSourceServerPreferredAddressConfig) Reset() {
	*x = DataSourceServerPreferredAddressConfig{}
	mi := &file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DataSourceServerPreferredAddressConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DataSourceServerPreferredAddressConfig) ProtoMessage() {}

func (x *DataSourceServerPreferredAddressConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DataSourceServerPreferredAddressConfig.ProtoReflect.Descriptor instead.
func (*DataSourceServerPreferredAddressConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDescGZIP(), []int{0}
}

func (x *DataSourceServerPreferredAddressConfig) GetIpv4Config() *DataSourceServerPreferredAddressConfig_AddressFamilyConfig {
	if x != nil {
		return x.Ipv4Config
	}
	return nil
}

func (x *DataSourceServerPreferredAddressConfig) GetIpv6Config() *DataSourceServerPreferredAddressConfig_AddressFamilyConfig {
	if x != nil {
		return x.Ipv6Config
	}
	return nil
}

// Addresses for server preferred address for a single address family (IPv4 or IPv6).
type DataSourceServerPreferredAddressConfig_AddressFamilyConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The server preferred address sent to clients. The data must contain an IP address string.
	Address *v3.DataSource `protobuf:"bytes,1,opt,name=address,proto3" json:"address,omitempty"`
	// The server preferred address port sent to clients. The data must contain a integer port value.
	//
	// If this is not specified, the listener's port is used.
	//
	// Note: Envoy currently must receive all packets for a QUIC connection on the same port, so unless
	// :ref:`dnat_address <envoy_v3_api_field_extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfig.dnat_address>`
	// is configured, this must be left unset.
	Port *v3.DataSource `protobuf:"bytes,2,opt,name=port,proto3" json:"port,omitempty"`
	// If there is a DNAT between the client and Envoy, the address that Envoy will observe
	// server preferred address packets being sent to. If this is not specified, it is assumed
	// there is no DNAT and the server preferred address packets will be sent to the address advertised
	// to clients for server preferred address.
	DnatAddress   *v3.DataSource `protobuf:"bytes,3,opt,name=dnat_address,json=dnatAddress,proto3" json:"dnat_address,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DataSourceServerPreferredAddressConfig_AddressFamilyConfig) Reset() {
	*x = DataSourceServerPreferredAddressConfig_AddressFamilyConfig{}
	mi := &file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DataSourceServerPreferredAddressConfig_AddressFamilyConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DataSourceServerPreferredAddressConfig_AddressFamilyConfig) ProtoMessage() {}

func (x *DataSourceServerPreferredAddressConfig_AddressFamilyConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DataSourceServerPreferredAddressConfig_AddressFamilyConfig.ProtoReflect.Descriptor instead.
func (*DataSourceServerPreferredAddressConfig_AddressFamilyConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDescGZIP(), []int{0, 0}
}

func (x *DataSourceServerPreferredAddressConfig_AddressFamilyConfig) GetAddress() *v3.DataSource {
	if x != nil {
		return x.Address
	}
	return nil
}

func (x *DataSourceServerPreferredAddressConfig_AddressFamilyConfig) GetPort() *v3.DataSource {
	if x != nil {
		return x.Port
	}
	return nil
}

func (x *DataSourceServerPreferredAddressConfig_AddressFamilyConfig) GetDnatAddress() *v3.DataSource {
	if x != nil {
		return x.DnatAddress
	}
	return nil
}

var File_envoy_extensions_quic_server_preferred_address_v3_datasource_proto protoreflect.FileDescriptor

const file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDesc = "" +
	"\n" +
	"Benvoy/extensions/quic/server_preferred_address/v3/datasource.proto\x121envoy.extensions.quic.server_preferred_address.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xa3\x04\n" +
	"&DataSourceServerPreferredAddressConfig\x12\x8e\x01\n" +
	"\vipv4_config\x18\x01 \x01(\v2m.envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfigR\n" +
	"ipv4Config\x12\x8e\x01\n" +
	"\vipv6_config\x18\x02 \x01(\v2m.envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfigR\n" +
	"ipv6Config\x1a\xd6\x01\n" +
	"\x13AddressFamilyConfig\x12D\n" +
	"\aaddress\x18\x01 \x01(\v2 .envoy.config.core.v3.DataSourceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\aaddress\x124\n" +
	"\x04port\x18\x02 \x01(\v2 .envoy.config.core.v3.DataSourceR\x04port\x12C\n" +
	"\fdnat_address\x18\x03 \x01(\v2 .envoy.config.core.v3.DataSourceR\vdnatAddressB\xd1\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"?io.envoyproxy.envoy.extensions.quic.server_preferred_address.v3B\x0fDatasourceProtoP\x01Zsgithub.com/envoyproxy/go-control-plane/envoy/extensions/quic/server_preferred_address/v3;server_preferred_addressv3b\x06proto3"

var (
	file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDescOnce sync.Once
	file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDescData []byte
)

func file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDescGZIP() []byte {
	file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDesc), len(file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDesc)))
	})
	return file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDescData
}

var file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_goTypes = []any{
	(*DataSourceServerPreferredAddressConfig)(nil),                     // 0: envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig
	(*DataSourceServerPreferredAddressConfig_AddressFamilyConfig)(nil), // 1: envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfig
	(*v3.DataSource)(nil), // 2: envoy.config.core.v3.DataSource
}
var file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.ipv4_config:type_name -> envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfig
	1, // 1: envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.ipv6_config:type_name -> envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfig
	2, // 2: envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfig.address:type_name -> envoy.config.core.v3.DataSource
	2, // 3: envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfig.port:type_name -> envoy.config.core.v3.DataSource
	2, // 4: envoy.extensions.quic.server_preferred_address.v3.DataSourceServerPreferredAddressConfig.AddressFamilyConfig.dnat_address:type_name -> envoy.config.core.v3.DataSource
	5, // [5:5] is the sub-list for method output_type
	5, // [5:5] is the sub-list for method input_type
	5, // [5:5] is the sub-list for extension type_name
	5, // [5:5] is the sub-list for extension extendee
	0, // [0:5] is the sub-list for field type_name
}

func init() { file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_init() }
func file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_init() {
	if File_envoy_extensions_quic_server_preferred_address_v3_datasource_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDesc), len(file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_msgTypes,
	}.Build()
	File_envoy_extensions_quic_server_preferred_address_v3_datasource_proto = out.File
	file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_goTypes = nil
	file_envoy_extensions_quic_server_preferred_address_v3_datasource_proto_depIdxs = nil
}
