// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/quic/connection_id_generator/quic_lb/v3/quic_lb.proto

package quic_lbv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for a connection ID generator implementation for the QUIC-LB draft RFC for
// routable connection IDs.
//
// Connection IDs always have the length self encoded, as described in
// https://datatracker.ietf.org/doc/html/draft-ietf-quic-load-balancers#name-length-self-description.
//
// See https://datatracker.ietf.org/doc/html/draft-ietf-quic-load-balancers for details.
//
// .. warning::
//
//	This is still a work in progress. Interoperability testing has not yet been performed.
//
// [#next-free-field: 7]
type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Must be at least 1 octet.
	// The length of server_id and nonce_length_bytes must be 18 or less.
	// See https://datatracker.ietf.org/doc/html/draft-ietf-quic-load-balancers#name-server-id-allocation.
	ServerId *v3.DataSource `protobuf:"bytes,2,opt,name=server_id,json=serverId,proto3" json:"server_id,omitempty"`
	// If true, indicates that the :ref:`server_id
	// <envoy_v3_api_field_extensions.quic.connection_id_generator.quic_lb.v3.Config.server_id>` is base64 encoded.
	//
	// This can be useful if the ID may contain binary data and must be transmitted as a string, for example in
	// an environment variable.
	ServerIdBase64Encoded bool `protobuf:"varint,6,opt,name=server_id_base64_encoded,json=serverIdBase64Encoded,proto3" json:"server_id_base64_encoded,omitempty"`
	// Optional validation of the expected server ID length. If this is non-zero and the value in “server_id“
	// does not have a matching length, a configuration error is generated. This can be useful for validating
	// that the server ID is valid.
	ExpectedServerIdLength uint32 `protobuf:"varint,3,opt,name=expected_server_id_length,json=expectedServerIdLength,proto3" json:"expected_server_id_length,omitempty"`
	// The nonce length must be at least 4 bytes.
	// The length of server_id and nonce_length_bytes must be 18 bytes or less.
	NonceLengthBytes uint32 `protobuf:"varint,4,opt,name=nonce_length_bytes,json=nonceLengthBytes,proto3" json:"nonce_length_bytes,omitempty"`
	// Configuration to fetch the encryption key and configuration version.
	//
	// The SDS service is for a :ref:`GenericSecret <envoy_v3_api_msg_extensions.transport_sockets.tls.v3.GenericSecret>`.
	// The data should populate :ref:`secrets <envoy_v3_api_field_extensions.transport_sockets.tls.v3.GenericSecret.secrets>`:
	//
	// "encryption_key" must contain the 16 byte encryption key.
	//
	// "configuration_version" must contain a 1 byte unsigned integer of value less than 7.
	// See https://datatracker.ietf.org/doc/html/draft-ietf-quic-load-balancers#name-config-rotation.
	EncryptionParameters *v31.SdsSecretConfig `protobuf:"bytes,5,opt,name=encryption_parameters,json=encryptionParameters,proto3" json:"encryption_parameters,omitempty"`
	// Use the unencrypted mode. This is useful for testing or a simplified implementation of the
	// downstream load balancer, but allows for linking different CIDs for the same connection, and
	// leaks information about the valid server IDs in use. This mode does not comply with the RFC.
	//
	// Note that in this mode, :ref:`encryption_parameters
	// <envoy_v3_api_field_extensions.quic.connection_id_generator.quic_lb.v3.Config.encryption_parameters>`
	// is still required because it contains “configuration_version“, which is still
	// needed. “encryption_key“ can be set to “inline_string: '0000000000000000'“.
	UnencryptedMode bool `protobuf:"varint,1,opt,name=unencrypted_mode,json=unencryptedMode,proto3" json:"unencrypted_mode,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetServerId() *v3.DataSource {
	if x != nil {
		return x.ServerId
	}
	return nil
}

func (x *Config) GetServerIdBase64Encoded() bool {
	if x != nil {
		return x.ServerIdBase64Encoded
	}
	return false
}

func (x *Config) GetExpectedServerIdLength() uint32 {
	if x != nil {
		return x.ExpectedServerIdLength
	}
	return 0
}

func (x *Config) GetNonceLengthBytes() uint32 {
	if x != nil {
		return x.NonceLengthBytes
	}
	return 0
}

func (x *Config) GetEncryptionParameters() *v31.SdsSecretConfig {
	if x != nil {
		return x.EncryptionParameters
	}
	return nil
}

func (x *Config) GetUnencryptedMode() bool {
	if x != nil {
		return x.UnencryptedMode
	}
	return false
}

var File_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto protoreflect.FileDescriptor

const file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDesc = "" +
	"\n" +
	"Fenvoy/extensions/quic/connection_id_generator/quic_lb/v3/quic_lb.proto\x128envoy.extensions.quic.connection_id_generator.quic_lb.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a6envoy/extensions/transport_sockets/tls/v3/secret.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xb7\x03\n" +
	"\x06Config\x12G\n" +
	"\tserver_id\x18\x02 \x01(\v2 .envoy.config.core.v3.DataSourceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\bserverId\x127\n" +
	"\x18server_id_base64_encoded\x18\x06 \x01(\bR\x15serverIdBase64Encoded\x12B\n" +
	"\x19expected_server_id_length\x18\x03 \x01(\rB\a\xfaB\x04*\x02\x18\x12R\x16expectedServerIdLength\x127\n" +
	"\x12nonce_length_bytes\x18\x04 \x01(\rB\t\xfaB\x06*\x04\x18\x10(\x04R\x10nonceLengthBytes\x12y\n" +
	"\x15encryption_parameters\x18\x05 \x01(\v2:.envoy.extensions.transport_sockets.tls.v3.SdsSecretConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x14encryptionParameters\x12)\n" +
	"\x10unencrypted_mode\x18\x01 \x01(\bR\x0funencryptedMode:\b\xd2Ƥ\xe1\x06\x02\b\x01B\xca\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Fio.envoyproxy.envoy.extensions.quic.connection_id_generator.quic_lb.v3B\vQuicLbProtoP\x01Zigithub.com/envoyproxy/go-control-plane/envoy/extensions/quic/connection_id_generator/quic_lb/v3;quic_lbv3b\x06proto3"

var (
	file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDescOnce sync.Once
	file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDescData []byte
)

func file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDescGZIP() []byte {
	file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDesc), len(file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDesc)))
	})
	return file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDescData
}

var file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_goTypes = []any{
	(*Config)(nil),              // 0: envoy.extensions.quic.connection_id_generator.quic_lb.v3.Config
	(*v3.DataSource)(nil),       // 1: envoy.config.core.v3.DataSource
	(*v31.SdsSecretConfig)(nil), // 2: envoy.extensions.transport_sockets.tls.v3.SdsSecretConfig
}
var file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.quic.connection_id_generator.quic_lb.v3.Config.server_id:type_name -> envoy.config.core.v3.DataSource
	2, // 1: envoy.extensions.quic.connection_id_generator.quic_lb.v3.Config.encryption_parameters:type_name -> envoy.extensions.transport_sockets.tls.v3.SdsSecretConfig
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_init() }
func file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_init() {
	if File_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDesc), len(file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_msgTypes,
	}.Build()
	File_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto = out.File
	file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_goTypes = nil
	file_envoy_extensions_quic_connection_id_generator_quic_lb_v3_quic_lb_proto_depIdxs = nil
}
