// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/router/cluster_specifier/golang/v3alpha/golang.proto

package v3alpha

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#extension-category: envoy.router.cluster_specifier_plugin]
type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Globally unique ID for a dynamic library file.
	LibraryId string `protobuf:"bytes,1,opt,name=library_id,json=libraryId,proto3" json:"library_id,omitempty"`
	// Path to a dynamic library implementing the
	// :repo:`ClusterSpecifier API <contrib/golang/router/cluster_specifier/source/go/pkg/api.ClusterSpecifier>`
	// interface.
	// [#comment:TODO(wangfakang): Support for downloading libraries from remote repositories.]
	LibraryPath string `protobuf:"bytes,2,opt,name=library_path,json=libraryPath,proto3" json:"library_path,omitempty"`
	// Default cluster.
	//
	// It will be used when the specifier interface return empty string or panic.
	DefaultCluster string `protobuf:"bytes,3,opt,name=default_cluster,json=defaultCluster,proto3" json:"default_cluster,omitempty"`
	// Configuration for the Go cluster specifier plugin.
	//
	// .. note::
	//
	//	This configuration is only parsed in the go cluster specifier, and is therefore not validated
	//	by Envoy.
	//
	//	See the :repo:`StreamFilter API <contrib/golang/router/cluster_specifier/source/go/pkg/cluster_specifier/config.go>`
	//	for more information about how the plugin's configuration data can be accessed.
	Config        *anypb.Any `protobuf:"bytes,4,opt,name=config,proto3" json:"config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetLibraryId() string {
	if x != nil {
		return x.LibraryId
	}
	return ""
}

func (x *Config) GetLibraryPath() string {
	if x != nil {
		return x.LibraryPath
	}
	return ""
}

func (x *Config) GetDefaultCluster() string {
	if x != nil {
		return x.DefaultCluster
	}
	return ""
}

func (x *Config) GetConfig() *anypb.Any {
	if x != nil {
		return x.Config
	}
	return nil
}

var File_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDesc = "" +
	"\n" +
	"Mcontrib/envoy/extensions/router/cluster_specifier/golang/v3alpha/golang.proto\x128envoy.extensions.router.cluster_specifier.golang.v3alpha\x1a\x19google/protobuf/any.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xbc\x01\n" +
	"\x06Config\x12&\n" +
	"\n" +
	"library_id\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\tlibraryId\x12*\n" +
	"\flibrary_path\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vlibraryPath\x120\n" +
	"\x0fdefault_cluster\x18\x03 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x0edefaultCluster\x12,\n" +
	"\x06config\x18\x04 \x01(\v2\x14.google.protobuf.AnyR\x06configB\xd0\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"Fio.envoyproxy.envoy.extensions.router.cluster_specifier.golang.v3alphaB\vGolangProtoP\x01Zggithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/router/cluster_specifier/golang/v3alphab\x06proto3"

var (
	file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDesc), len(file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDescData
}

var file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_goTypes = []any{
	(*Config)(nil),    // 0: envoy.extensions.router.cluster_specifier.golang.v3alpha.Config
	(*anypb.Any)(nil), // 1: google.protobuf.Any
}
var file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.router.cluster_specifier.golang.v3alpha.Config.config:type_name -> google.protobuf.Any
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() {
	file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_init()
}
func file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_init() {
	if File_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDesc), len(file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_depIdxs,
		MessageInfos:      file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto = out.File
	file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_goTypes = nil
	file_contrib_envoy_extensions_router_cluster_specifier_golang_v3alpha_golang_proto_depIdxs = nil
}
