// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/bootstrap/reverse_tunnel/upstream_socket_interface/v3/upstream_reverse_connection_socket_interface.proto

package upstream_socket_interfacev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the upstream reverse connection socket interface.
// [#next-free-field: 6]
type UpstreamReverseConnectionSocketInterface struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Stat prefix for upstream reverse connection socket interface stats.
	StatPrefix string `protobuf:"bytes,1,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// Number of consecutive ping failures before an idle reverse connection socket is marked dead.
	// Defaults to 3 if unset. Must be at least 1.
	PingFailureThreshold *wrapperspb.UInt32Value `protobuf:"bytes,2,opt,name=ping_failure_threshold,json=pingFailureThreshold,proto3" json:"ping_failure_threshold,omitempty"`
	// Enable detailed per-node and per-cluster statistics.
	// When enabled, emits hidden statistics for individual nodes and clusters.
	// Defaults to false.
	EnableDetailedStats bool `protobuf:"varint,3,opt,name=enable_detailed_stats,json=enableDetailedStats,proto3" json:"enable_detailed_stats,omitempty"`
	// Optional configuration for a tunnel reporting extension. When provided,
	// the socket interface instantiates a reporter via the configured factory.
	// If unset, no reporting is done.
	ReporterConfig *v3.TypedExtensionConfig `protobuf:"bytes,4,opt,name=reporter_config,json=reporterConfig,proto3" json:"reporter_config,omitempty"`
	// Enables tenant-aware isolation for reverse connections. When set to “true“, the socket
	// interface requires tenant identifiers in addition to node and cluster identifiers and derives
	// composite “tenant:node“ and “tenant:cluster“ keys for socket tracking. Identifiers
	// containing the “:“ delimiter are rejected to avoid ambiguity.
	// Defaults to “false“ for backwards compatibility.
	EnableTenantIsolation *wrapperspb.BoolValue `protobuf:"bytes,5,opt,name=enable_tenant_isolation,json=enableTenantIsolation,proto3" json:"enable_tenant_isolation,omitempty"`
	unknownFields         protoimpl.UnknownFields
	sizeCache             protoimpl.SizeCache
}

func (x *UpstreamReverseConnectionSocketInterface) Reset() {
	*x = UpstreamReverseConnectionSocketInterface{}
	mi := &file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpstreamReverseConnectionSocketInterface) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpstreamReverseConnectionSocketInterface) ProtoMessage() {}

func (x *UpstreamReverseConnectionSocketInterface) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpstreamReverseConnectionSocketInterface.ProtoReflect.Descriptor instead.
func (*UpstreamReverseConnectionSocketInterface) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDescGZIP(), []int{0}
}

func (x *UpstreamReverseConnectionSocketInterface) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *UpstreamReverseConnectionSocketInterface) GetPingFailureThreshold() *wrapperspb.UInt32Value {
	if x != nil {
		return x.PingFailureThreshold
	}
	return nil
}

func (x *UpstreamReverseConnectionSocketInterface) GetEnableDetailedStats() bool {
	if x != nil {
		return x.EnableDetailedStats
	}
	return false
}

func (x *UpstreamReverseConnectionSocketInterface) GetReporterConfig() *v3.TypedExtensionConfig {
	if x != nil {
		return x.ReporterConfig
	}
	return nil
}

func (x *UpstreamReverseConnectionSocketInterface) GetEnableTenantIsolation() *wrapperspb.BoolValue {
	if x != nil {
		return x.EnableTenantIsolation
	}
	return nil
}

var File_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto protoreflect.FileDescriptor

const file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDesc = "" +
	"\n" +
	"yenvoy/extensions/bootstrap/reverse_tunnel/upstream_socket_interface/v3/upstream_reverse_connection_socket_interface.proto\x12Fenvoy.extensions.bootstrap.reverse_tunnel.upstream_socket_interface.v3\x1a$envoy/config/core/v3/extension.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x85\x03\n" +
	"(UpstreamReverseConnectionSocketInterface\x12\x1f\n" +
	"\vstat_prefix\x18\x01 \x01(\tR\n" +
	"statPrefix\x12[\n" +
	"\x16ping_failure_threshold\x18\x02 \x01(\v2\x1c.google.protobuf.UInt32ValueB\a\xfaB\x04*\x02(\x01R\x14pingFailureThreshold\x122\n" +
	"\x15enable_detailed_stats\x18\x03 \x01(\bR\x13enableDetailedStats\x12S\n" +
	"\x0freporter_config\x18\x04 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigR\x0ereporterConfig\x12R\n" +
	"\x17enable_tenant_isolation\x18\x05 \x01(\v2\x1a.google.protobuf.BoolValueR\x15enableTenantIsolationB\x9b\x02\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Tio.envoyproxy.envoy.extensions.bootstrap.reverse_tunnel.upstream_socket_interface.v3B-UpstreamReverseConnectionSocketInterfaceProtoP\x01Z\x89\x01github.com/envoyproxy/go-control-plane/envoy/extensions/bootstrap/reverse_tunnel/upstream_socket_interface/v3;upstream_socket_interfacev3b\x06proto3"

var (
	file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDescOnce sync.Once
	file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDescData []byte
)

func file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDescGZIP() []byte {
	file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDesc), len(file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDesc)))
	})
	return file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDescData
}

var file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_goTypes = []any{
	(*UpstreamReverseConnectionSocketInterface)(nil), // 0: envoy.extensions.bootstrap.reverse_tunnel.upstream_socket_interface.v3.UpstreamReverseConnectionSocketInterface
	(*wrapperspb.UInt32Value)(nil),                   // 1: google.protobuf.UInt32Value
	(*v3.TypedExtensionConfig)(nil),                  // 2: envoy.config.core.v3.TypedExtensionConfig
	(*wrapperspb.BoolValue)(nil),                     // 3: google.protobuf.BoolValue
}
var file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.bootstrap.reverse_tunnel.upstream_socket_interface.v3.UpstreamReverseConnectionSocketInterface.ping_failure_threshold:type_name -> google.protobuf.UInt32Value
	2, // 1: envoy.extensions.bootstrap.reverse_tunnel.upstream_socket_interface.v3.UpstreamReverseConnectionSocketInterface.reporter_config:type_name -> envoy.config.core.v3.TypedExtensionConfig
	3, // 2: envoy.extensions.bootstrap.reverse_tunnel.upstream_socket_interface.v3.UpstreamReverseConnectionSocketInterface.enable_tenant_isolation:type_name -> google.protobuf.BoolValue
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_init()
}
func file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_init() {
	if File_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDesc), len(file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_msgTypes,
	}.Build()
	File_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto = out.File
	file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_goTypes = nil
	file_envoy_extensions_bootstrap_reverse_tunnel_upstream_socket_interface_v3_upstream_reverse_connection_socket_interface_proto_depIdxs = nil
}
