// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/mcp_router/v3/mcp_router.proto

package mcp_routerv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/metadata/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ValidationPolicy_Mode int32

const (
	// Not specified. Defaults to DISABLED behavior.
	ValidationPolicy_MODE_UNSPECIFIED ValidationPolicy_Mode = 0
	// Bind identity on Initialize if present, but do not validate subsequent requests.
	// If extraction fails, the session proceeds anonymously.
	ValidationPolicy_DISABLED ValidationPolicy_Mode = 1
	// Reject the request (403) if the identity cannot be extracted
	// or if the session identity does not match the request identity.
	ValidationPolicy_ENFORCE ValidationPolicy_Mode = 2
)

// Enum value maps for ValidationPolicy_Mode.
var (
	ValidationPolicy_Mode_name = map[int32]string{
		0: "MODE_UNSPECIFIED",
		1: "DISABLED",
		2: "ENFORCE",
	}
	ValidationPolicy_Mode_value = map[string]int32{
		"MODE_UNSPECIFIED": 0,
		"DISABLED":         1,
		"ENFORCE":          2,
	}
)

func (x ValidationPolicy_Mode) Enum() *ValidationPolicy_Mode {
	p := new(ValidationPolicy_Mode)
	*p = x
	return p
}

func (x ValidationPolicy_Mode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ValidationPolicy_Mode) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_enumTypes[0].Descriptor()
}

func (ValidationPolicy_Mode) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_enumTypes[0]
}

func (x ValidationPolicy_Mode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ValidationPolicy_Mode.Descriptor instead.
func (ValidationPolicy_Mode) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{3, 0}
}

// Extract identity from a request header.
type HeaderSource struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Header name to extract (e.g., "x-user-identity").
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HeaderSource) Reset() {
	*x = HeaderSource{}
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HeaderSource) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HeaderSource) ProtoMessage() {}

func (x *HeaderSource) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HeaderSource.ProtoReflect.Descriptor instead.
func (*HeaderSource) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{0}
}

func (x *HeaderSource) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// Extract identity from dynamic metadata (e.g., populated by JWT or ext_authz filter).
type DynamicMetadataSource struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The metadata key to retrieve the value from.
	Key           *v3.MetadataKey `protobuf:"bytes,1,opt,name=key,proto3" json:"key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DynamicMetadataSource) Reset() {
	*x = DynamicMetadataSource{}
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DynamicMetadataSource) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DynamicMetadataSource) ProtoMessage() {}

func (x *DynamicMetadataSource) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DynamicMetadataSource.ProtoReflect.Descriptor instead.
func (*DynamicMetadataSource) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{1}
}

func (x *DynamicMetadataSource) GetKey() *v3.MetadataKey {
	if x != nil {
		return x.Key
	}
	return nil
}

// Defines how the identity (user/principal) is extracted from the request.
// Exactly one of “header“ or “dynamic_metadata“ must be set.
type IdentityExtractor struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Extract identity from a request header.
	Header *HeaderSource `protobuf:"bytes,1,opt,name=header,proto3" json:"header,omitempty"`
	// Extract identity from dynamic metadata.
	DynamicMetadata *DynamicMetadataSource `protobuf:"bytes,2,opt,name=dynamic_metadata,json=dynamicMetadata,proto3" json:"dynamic_metadata,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *IdentityExtractor) Reset() {
	*x = IdentityExtractor{}
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *IdentityExtractor) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*IdentityExtractor) ProtoMessage() {}

func (x *IdentityExtractor) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use IdentityExtractor.ProtoReflect.Descriptor instead.
func (*IdentityExtractor) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{2}
}

func (x *IdentityExtractor) GetHeader() *HeaderSource {
	if x != nil {
		return x.Header
	}
	return nil
}

func (x *IdentityExtractor) GetDynamicMetadata() *DynamicMetadataSource {
	if x != nil {
		return x.DynamicMetadata
	}
	return nil
}

// Specifies how to handle requests where the identity is missing or mismatched.
type ValidationPolicy struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Mode          ValidationPolicy_Mode  `protobuf:"varint,1,opt,name=mode,proto3,enum=envoy.extensions.filters.http.mcp_router.v3.ValidationPolicy_Mode" json:"mode,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ValidationPolicy) Reset() {
	*x = ValidationPolicy{}
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ValidationPolicy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ValidationPolicy) ProtoMessage() {}

func (x *ValidationPolicy) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ValidationPolicy.ProtoReflect.Descriptor instead.
func (*ValidationPolicy) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{3}
}

func (x *ValidationPolicy) GetMode() ValidationPolicy_Mode {
	if x != nil {
		return x.Mode
	}
	return ValidationPolicy_MODE_UNSPECIFIED
}

// Session identity configuration.
type SessionIdentity struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Defines how the identity (user/principal) is extracted from the request.
	Identity *IdentityExtractor `protobuf:"bytes,1,opt,name=identity,proto3" json:"identity,omitempty"`
	// Specifies how to handle requests where the subject is missing or invalid.
	// Defaults to DISABLED.
	Validation    *ValidationPolicy `protobuf:"bytes,2,opt,name=validation,proto3" json:"validation,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SessionIdentity) Reset() {
	*x = SessionIdentity{}
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SessionIdentity) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SessionIdentity) ProtoMessage() {}

func (x *SessionIdentity) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SessionIdentity.ProtoReflect.Descriptor instead.
func (*SessionIdentity) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{4}
}

func (x *SessionIdentity) GetIdentity() *IdentityExtractor {
	if x != nil {
		return x.Identity
	}
	return nil
}

func (x *SessionIdentity) GetValidation() *ValidationPolicy {
	if x != nil {
		return x.Validation
	}
	return nil
}

type McpRouter struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// A list of remote MCP servers. MCP router aggregates capabilities, tools and resources from remote MCP servers
	// and presents itself as single MCP server to the client. All remote MCP servers are sent the same capabilities
	// that the client presented to Envoy.
	Servers []*McpRouter_McpBackend `protobuf:"bytes,1,rep,name=servers,proto3" json:"servers,omitempty"`
	// If set, extracts a request "subject" and binds it into the MCP session.
	// If not set, sessions are created without identity binding.
	SessionIdentity *SessionIdentity `protobuf:"bytes,2,opt,name=session_identity,json=sessionIdentity,proto3" json:"session_identity,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *McpRouter) Reset() {
	*x = McpRouter{}
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *McpRouter) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*McpRouter) ProtoMessage() {}

func (x *McpRouter) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use McpRouter.ProtoReflect.Descriptor instead.
func (*McpRouter) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{5}
}

func (x *McpRouter) GetServers() []*McpRouter_McpBackend {
	if x != nil {
		return x.Servers
	}
	return nil
}

func (x *McpRouter) GetSessionIdentity() *SessionIdentity {
	if x != nil {
		return x.SessionIdentity
	}
	return nil
}

// Specification of the MCP server.
type McpRouter_McpBackend struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Unique name for this backend. Used for:
	// - Tool name prefixing (e.g., "time__get_current_time")
	// - Session ID composition
	// - Logging and error messages.
	// Default will be the cluster name if not specified.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Backend target specification.
	McpCluster    *McpRouter_McpCluster `protobuf:"bytes,2,opt,name=mcp_cluster,json=mcpCluster,proto3" json:"mcp_cluster,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *McpRouter_McpBackend) Reset() {
	*x = McpRouter_McpBackend{}
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *McpRouter_McpBackend) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*McpRouter_McpBackend) ProtoMessage() {}

func (x *McpRouter_McpBackend) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use McpRouter_McpBackend.ProtoReflect.Descriptor instead.
func (*McpRouter_McpBackend) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{5, 0}
}

func (x *McpRouter_McpBackend) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *McpRouter_McpBackend) GetMcpCluster() *McpRouter_McpCluster {
	if x != nil {
		return x.McpCluster
	}
	return nil
}

// Cluster-based backend configuration.
type McpRouter_McpCluster struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Cluster name to route requests to.
	Cluster string `protobuf:"bytes,1,opt,name=cluster,proto3" json:"cluster,omitempty"`
	// Path to use for MCP requests. Defaults to "/mcp".
	Path string `protobuf:"bytes,2,opt,name=path,proto3" json:"path,omitempty"`
	// Request timeout.
	// If not set, uses cluster's timeout configuration.
	Timeout *durationpb.Duration `protobuf:"bytes,3,opt,name=timeout,proto3" json:"timeout,omitempty"`
	// Indicates that during forwarding, the host header will be swapped with
	// this value.
	HostRewriteLiteral string `protobuf:"bytes,4,opt,name=host_rewrite_literal,json=hostRewriteLiteral,proto3" json:"host_rewrite_literal,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *McpRouter_McpCluster) Reset() {
	*x = McpRouter_McpCluster{}
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *McpRouter_McpCluster) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*McpRouter_McpCluster) ProtoMessage() {}

func (x *McpRouter_McpCluster) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use McpRouter_McpCluster.ProtoReflect.Descriptor instead.
func (*McpRouter_McpCluster) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP(), []int{5, 1}
}

func (x *McpRouter_McpCluster) GetCluster() string {
	if x != nil {
		return x.Cluster
	}
	return ""
}

func (x *McpRouter_McpCluster) GetPath() string {
	if x != nil {
		return x.Path
	}
	return ""
}

func (x *McpRouter_McpCluster) GetTimeout() *durationpb.Duration {
	if x != nil {
		return x.Timeout
	}
	return nil
}

func (x *McpRouter_McpCluster) GetHostRewriteLiteral() string {
	if x != nil {
		return x.HostRewriteLiteral
	}
	return ""
}

var File_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDesc = "" +
	"\n" +
	"<envoy/extensions/filters/http/mcp_router/v3/mcp_router.proto\x12+envoy.extensions.filters.http.mcp_router.v3\x1a%envoy/type/metadata/v3/metadata.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\".\n" +
	"\fHeaderSource\x12\x1e\n" +
	"\x04name\x18\x01 \x01(\tB\n" +
	"\xfaB\ar\x05\x10\x01\xc0\x01\x01R\x04name\"X\n" +
	"\x15DynamicMetadataSource\x12?\n" +
	"\x03key\x18\x01 \x01(\v2#.envoy.type.metadata.v3.MetadataKeyB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x03key\"\xd5\x01\n" +
	"\x11IdentityExtractor\x12Q\n" +
	"\x06header\x18\x01 \x01(\v29.envoy.extensions.filters.http.mcp_router.v3.HeaderSourceR\x06header\x12m\n" +
	"\x10dynamic_metadata\x18\x02 \x01(\v2B.envoy.extensions.filters.http.mcp_router.v3.DynamicMetadataSourceR\x0fdynamicMetadata\"\xad\x01\n" +
	"\x10ValidationPolicy\x12`\n" +
	"\x04mode\x18\x01 \x01(\x0e2B.envoy.extensions.filters.http.mcp_router.v3.ValidationPolicy.ModeB\b\xfaB\x05\x82\x01\x02\x10\x01R\x04mode\"7\n" +
	"\x04Mode\x12\x14\n" +
	"\x10MODE_UNSPECIFIED\x10\x00\x12\f\n" +
	"\bDISABLED\x10\x01\x12\v\n" +
	"\aENFORCE\x10\x02\"\xd6\x01\n" +
	"\x0fSessionIdentity\x12d\n" +
	"\bidentity\x18\x01 \x01(\v2>.envoy.extensions.filters.http.mcp_router.v3.IdentityExtractorB\b\xfaB\x05\x8a\x01\x02\x10\x01R\bidentity\x12]\n" +
	"\n" +
	"validation\x18\x02 \x01(\v2=.envoy.extensions.filters.http.mcp_router.v3.ValidationPolicyR\n" +
	"validation\"\x85\x04\n" +
	"\tMcpRouter\x12[\n" +
	"\aservers\x18\x01 \x03(\v2A.envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpBackendR\aservers\x12g\n" +
	"\x10session_identity\x18\x02 \x01(\v2<.envoy.extensions.filters.http.mcp_router.v3.SessionIdentityR\x0fsessionIdentity\x1a\x84\x01\n" +
	"\n" +
	"McpBackend\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12b\n" +
	"\vmcp_cluster\x18\x02 \x01(\v2A.envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpClusterR\n" +
	"mcpCluster\x1a\xaa\x01\n" +
	"\n" +
	"McpCluster\x12!\n" +
	"\acluster\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\acluster\x12\x12\n" +
	"\x04path\x18\x02 \x01(\tR\x04path\x123\n" +
	"\atimeout\x18\x03 \x01(\v2\x19.google.protobuf.DurationR\atimeout\x120\n" +
	"\x14host_rewrite_literal\x18\x04 \x01(\tR\x12hostRewriteLiteralB\xbe\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"9io.envoyproxy.envoy.extensions.filters.http.mcp_router.v3B\x0eMcpRouterProtoP\x01Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/mcp_router/v3;mcp_routerv3b\x06proto3"

var (
	file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDesc), len(file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDescData
}

var file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes = make([]protoimpl.MessageInfo, 8)
var file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_goTypes = []any{
	(ValidationPolicy_Mode)(0),    // 0: envoy.extensions.filters.http.mcp_router.v3.ValidationPolicy.Mode
	(*HeaderSource)(nil),          // 1: envoy.extensions.filters.http.mcp_router.v3.HeaderSource
	(*DynamicMetadataSource)(nil), // 2: envoy.extensions.filters.http.mcp_router.v3.DynamicMetadataSource
	(*IdentityExtractor)(nil),     // 3: envoy.extensions.filters.http.mcp_router.v3.IdentityExtractor
	(*ValidationPolicy)(nil),      // 4: envoy.extensions.filters.http.mcp_router.v3.ValidationPolicy
	(*SessionIdentity)(nil),       // 5: envoy.extensions.filters.http.mcp_router.v3.SessionIdentity
	(*McpRouter)(nil),             // 6: envoy.extensions.filters.http.mcp_router.v3.McpRouter
	(*McpRouter_McpBackend)(nil),  // 7: envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpBackend
	(*McpRouter_McpCluster)(nil),  // 8: envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpCluster
	(*v3.MetadataKey)(nil),        // 9: envoy.type.metadata.v3.MetadataKey
	(*durationpb.Duration)(nil),   // 10: google.protobuf.Duration
}
var file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_depIdxs = []int32{
	9,  // 0: envoy.extensions.filters.http.mcp_router.v3.DynamicMetadataSource.key:type_name -> envoy.type.metadata.v3.MetadataKey
	1,  // 1: envoy.extensions.filters.http.mcp_router.v3.IdentityExtractor.header:type_name -> envoy.extensions.filters.http.mcp_router.v3.HeaderSource
	2,  // 2: envoy.extensions.filters.http.mcp_router.v3.IdentityExtractor.dynamic_metadata:type_name -> envoy.extensions.filters.http.mcp_router.v3.DynamicMetadataSource
	0,  // 3: envoy.extensions.filters.http.mcp_router.v3.ValidationPolicy.mode:type_name -> envoy.extensions.filters.http.mcp_router.v3.ValidationPolicy.Mode
	3,  // 4: envoy.extensions.filters.http.mcp_router.v3.SessionIdentity.identity:type_name -> envoy.extensions.filters.http.mcp_router.v3.IdentityExtractor
	4,  // 5: envoy.extensions.filters.http.mcp_router.v3.SessionIdentity.validation:type_name -> envoy.extensions.filters.http.mcp_router.v3.ValidationPolicy
	7,  // 6: envoy.extensions.filters.http.mcp_router.v3.McpRouter.servers:type_name -> envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpBackend
	5,  // 7: envoy.extensions.filters.http.mcp_router.v3.McpRouter.session_identity:type_name -> envoy.extensions.filters.http.mcp_router.v3.SessionIdentity
	8,  // 8: envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpBackend.mcp_cluster:type_name -> envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpCluster
	10, // 9: envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpCluster.timeout:type_name -> google.protobuf.Duration
	10, // [10:10] is the sub-list for method output_type
	10, // [10:10] is the sub-list for method input_type
	10, // [10:10] is the sub-list for extension type_name
	10, // [10:10] is the sub-list for extension extendee
	0,  // [0:10] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_init() }
func file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_init() {
	if File_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDesc), len(file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   8,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto = out.File
	file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_goTypes = nil
	file_envoy_extensions_filters_http_mcp_router_v3_mcp_router_proto_depIdxs = nil
}
