/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	apiv1 "sigs.k8s.io/gateway-api-inference-extension/api/v1"
)

// PortApplyConfiguration represents a declarative configuration of the Port type for use
// with apply.
//
// Port defines the network port that will be exposed by this InferencePool.
type PortApplyConfiguration struct {
	// Number defines the port number to access the selected model server Pods.
	// The number must be in the range 1 to 65535.
	Number *apiv1.PortNumber `json:"number,omitempty"`
}

// PortApplyConfiguration constructs a declarative configuration of the Port type for use with
// apply.
func Port() *PortApplyConfiguration {
	return &PortApplyConfiguration{}
}

// WithNumber sets the Number field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Number field is set to the value of the last call.
func (b *PortApplyConfiguration) WithNumber(value apiv1.PortNumber) *PortApplyConfiguration {
	b.Number = &value
	return b
}
