// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/filters/http/language/v3alpha/language.proto

package v3alpha

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Language detection filter config.
type Language struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The default language to be used as a fallback.
	// The value will be included in the list of the supported languages.
	//
	// See https://unicode-org.github.io/icu/userguide/locale/
	DefaultLanguage string `protobuf:"bytes,1,opt,name=default_language,json=defaultLanguage,proto3" json:"default_language,omitempty"`
	// The set of supported languages. There is no order priority.
	// The order will be determined by the Accept-Language header priority list
	// of the client.
	//
	// See https://unicode-org.github.io/icu/userguide/locale/
	SupportedLanguages []string `protobuf:"bytes,2,rep,name=supported_languages,json=supportedLanguages,proto3" json:"supported_languages,omitempty"`
	// If the x-language header is altered, clear the route cache for the current request.
	// This should be set if the route configuration may depend on the x-language header.
	// Otherwise it should be unset to avoid the performance cost of route recalculation.
	ClearRouteCache bool `protobuf:"varint,3,opt,name=clear_route_cache,json=clearRouteCache,proto3" json:"clear_route_cache,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *Language) Reset() {
	*x = Language{}
	mi := &file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Language) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Language) ProtoMessage() {}

func (x *Language) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Language.ProtoReflect.Descriptor instead.
func (*Language) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDescGZIP(), []int{0}
}

func (x *Language) GetDefaultLanguage() string {
	if x != nil {
		return x.DefaultLanguage
	}
	return ""
}

func (x *Language) GetSupportedLanguages() []string {
	if x != nil {
		return x.SupportedLanguages
	}
	return nil
}

func (x *Language) GetClearRouteCache() bool {
	if x != nil {
		return x.ClearRouteCache
	}
	return false
}

var File_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDesc = "" +
	"\n" +
	"Econtrib/envoy/extensions/filters/http/language/v3alpha/language.proto\x12.envoy.extensions.filters.http.language.v3alpha\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xad\x01\n" +
	"\bLanguage\x122\n" +
	"\x10default_language\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x02R\x0fdefaultLanguage\x12A\n" +
	"\x13supported_languages\x18\x02 \x03(\tB\x10\xfaB\r\x92\x01\n" +
	"\b\x01\x18\x01\"\x04r\x02\x10\x02R\x12supportedLanguages\x12*\n" +
	"\x11clear_route_cache\x18\x03 \x01(\bR\x0fclearRouteCacheB\xb6\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"<io.envoyproxy.envoy.extensions.filters.http.language.v3alphaB\rLanguageProtoP\x01Z]github.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/language/v3alphab\x06proto3"

var (
	file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDesc), len(file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDescData
}

var file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_goTypes = []any{
	(*Language)(nil), // 0: envoy.extensions.filters.http.language.v3alpha.Language
}
var file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_init() }
func file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_init() {
	if File_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDesc), len(file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_depIdxs,
		MessageInfos:      file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto = out.File
	file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_goTypes = nil
	file_contrib_envoy_extensions_filters_http_language_v3alpha_language_proto_depIdxs = nil
}
