// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/tls/cert_validator/dynamic_modules/v3/dynamic_modules.proto

package dynamic_modulesv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/dynamic_modules/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the dynamic module certificate validator.
//
// Example:
//
// .. validated-code-block:: yaml
//
//	:type-name: envoy.extensions.transport_sockets.tls.v3.CertificateValidationContext
//
//	custom_validator_config:
//	  name: envoy.tls.cert_validator.dynamic_modules
//	  typed_config:
//	    "@type": type.googleapis.com/envoy.extensions.transport_sockets.tls.cert_validator.dynamic_modules.v3.DynamicModuleCertValidatorConfig
//	    dynamic_module_config:
//	      name: my_module
//	    validator_name: my_validator
type DynamicModuleCertValidatorConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Dynamic module configuration. See :ref:`dynamic module configuration
	// <envoy_v3_api_msg_extensions.dynamic_modules.v3.DynamicModuleConfig>` for details.
	DynamicModuleConfig *v3.DynamicModuleConfig `protobuf:"bytes,1,opt,name=dynamic_module_config,json=dynamicModuleConfig,proto3" json:"dynamic_module_config,omitempty"`
	// The name of the cert validator implementation in the dynamic module.
	// This is passed to the module's “envoy_dynamic_module_on_cert_validator_config_new“
	// function.
	ValidatorName string `protobuf:"bytes,2,opt,name=validator_name,json=validatorName,proto3" json:"validator_name,omitempty"`
	// Optional configuration for the cert validator. This is passed as bytes to the dynamic module.
	ValidatorConfig *anypb.Any `protobuf:"bytes,3,opt,name=validator_config,json=validatorConfig,proto3" json:"validator_config,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *DynamicModuleCertValidatorConfig) Reset() {
	*x = DynamicModuleCertValidatorConfig{}
	mi := &file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DynamicModuleCertValidatorConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DynamicModuleCertValidatorConfig) ProtoMessage() {}

func (x *DynamicModuleCertValidatorConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DynamicModuleCertValidatorConfig.ProtoReflect.Descriptor instead.
func (*DynamicModuleCertValidatorConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDescGZIP(), []int{0}
}

func (x *DynamicModuleCertValidatorConfig) GetDynamicModuleConfig() *v3.DynamicModuleConfig {
	if x != nil {
		return x.DynamicModuleConfig
	}
	return nil
}

func (x *DynamicModuleCertValidatorConfig) GetValidatorName() string {
	if x != nil {
		return x.ValidatorName
	}
	return ""
}

func (x *DynamicModuleCertValidatorConfig) GetValidatorConfig() *anypb.Any {
	if x != nil {
		return x.ValidatorConfig
	}
	return nil
}

var File_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDesc = "" +
	"\n" +
	"^envoy/extensions/transport_sockets/tls/cert_validator/dynamic_modules/v3/dynamic_modules.proto\x12Henvoy.extensions.transport_sockets.tls.cert_validator.dynamic_modules.v3\x1a9envoy/extensions/dynamic_modules/v3/dynamic_modules.proto\x1a\x19google/protobuf/any.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x8b\x02\n" +
	" DynamicModuleCertValidatorConfig\x12v\n" +
	"\x15dynamic_module_config\x18\x01 \x01(\v28.envoy.extensions.dynamic_modules.v3.DynamicModuleConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x13dynamicModuleConfig\x12.\n" +
	"\x0evalidator_name\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\rvalidatorName\x12?\n" +
	"\x10validator_config\x18\x03 \x01(\v2\x14.google.protobuf.AnyR\x0fvalidatorConfigB\xfb\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Vio.envoyproxy.envoy.extensions.transport_sockets.tls.cert_validator.dynamic_modules.v3B\x13DynamicModulesProtoP\x01Z\x81\x01github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/cert_validator/dynamic_modules/v3;dynamic_modulesv3b\x06proto3"

var (
	file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDescOnce sync.Once
	file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDescData []byte
)

func file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDescGZIP() []byte {
	file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDesc), len(file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDesc)))
	})
	return file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDescData
}

var file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_goTypes = []any{
	(*DynamicModuleCertValidatorConfig)(nil), // 0: envoy.extensions.transport_sockets.tls.cert_validator.dynamic_modules.v3.DynamicModuleCertValidatorConfig
	(*v3.DynamicModuleConfig)(nil),           // 1: envoy.extensions.dynamic_modules.v3.DynamicModuleConfig
	(*anypb.Any)(nil),                        // 2: google.protobuf.Any
}
var file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.transport_sockets.tls.cert_validator.dynamic_modules.v3.DynamicModuleCertValidatorConfig.dynamic_module_config:type_name -> envoy.extensions.dynamic_modules.v3.DynamicModuleConfig
	2, // 1: envoy.extensions.transport_sockets.tls.cert_validator.dynamic_modules.v3.DynamicModuleCertValidatorConfig.validator_config:type_name -> google.protobuf.Any
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_init()
}
func file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_init() {
	if File_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDesc), len(file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_msgTypes,
	}.Build()
	File_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto = out.File
	file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_tls_cert_validator_dynamic_modules_v3_dynamic_modules_proto_depIdxs = nil
}
