// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/tracers/opentelemetry/resource_detectors/v3/static_config_resource_detector.proto

package resource_detectorsv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the Static Resource detector extension.
// The resource detector uses static config for resource attribute,
// as per the OpenTelemetry specification.
//
// [#extension: envoy.tracers.opentelemetry.resource_detectors.static_config]
type StaticConfigResourceDetectorConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Custom Resource attributes to be included.
	Attributes    map[string]string `protobuf:"bytes,1,rep,name=attributes,proto3" json:"attributes,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StaticConfigResourceDetectorConfig) Reset() {
	*x = StaticConfigResourceDetectorConfig{}
	mi := &file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StaticConfigResourceDetectorConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StaticConfigResourceDetectorConfig) ProtoMessage() {}

func (x *StaticConfigResourceDetectorConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StaticConfigResourceDetectorConfig.ProtoReflect.Descriptor instead.
func (*StaticConfigResourceDetectorConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDescGZIP(), []int{0}
}

func (x *StaticConfigResourceDetectorConfig) GetAttributes() map[string]string {
	if x != nil {
		return x.Attributes
	}
	return nil
}

var File_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto protoreflect.FileDescriptor

const file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDesc = "" +
	"\n" +
	"benvoy/extensions/tracers/opentelemetry/resource_detectors/v3/static_config_resource_detector.proto\x12<envoy.extensions.tracers.opentelemetry.resource_detectors.v3\x1a\x1dudpa/annotations/status.proto\"\xf6\x01\n" +
	"\"StaticConfigResourceDetectorConfig\x12\x90\x01\n" +
	"\n" +
	"attributes\x18\x01 \x03(\v2p.envoy.extensions.tracers.opentelemetry.resource_detectors.v3.StaticConfigResourceDetectorConfig.AttributesEntryR\n" +
	"attributes\x1a=\n" +
	"\x0fAttributesEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01B\xf3\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Jio.envoyproxy.envoy.extensions.tracers.opentelemetry.resource_detectors.v3B!StaticConfigResourceDetectorProtoP\x01Zxgithub.com/envoyproxy/go-control-plane/envoy/extensions/tracers/opentelemetry/resource_detectors/v3;resource_detectorsv3b\x06proto3"

var (
	file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDescOnce sync.Once
	file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDescData []byte
)

func file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDescGZIP() []byte {
	file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDesc), len(file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDesc)))
	})
	return file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDescData
}

var file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_goTypes = []any{
	(*StaticConfigResourceDetectorConfig)(nil), // 0: envoy.extensions.tracers.opentelemetry.resource_detectors.v3.StaticConfigResourceDetectorConfig
	nil, // 1: envoy.extensions.tracers.opentelemetry.resource_detectors.v3.StaticConfigResourceDetectorConfig.AttributesEntry
}
var file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.tracers.opentelemetry.resource_detectors.v3.StaticConfigResourceDetectorConfig.attributes:type_name -> envoy.extensions.tracers.opentelemetry.resource_detectors.v3.StaticConfigResourceDetectorConfig.AttributesEntry
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_init()
}
func file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_init() {
	if File_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDesc), len(file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_msgTypes,
	}.Build()
	File_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto = out.File
	file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_goTypes = nil
	file_envoy_extensions_tracers_opentelemetry_resource_detectors_v3_static_config_resource_detector_proto_depIdxs = nil
}
