// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/filters/network/rocketmq_proxy/v3/route.proto

package rocketmq_proxyv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v32 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/route/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/matcher/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type RouteConfiguration struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the route configuration.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The list of routes that will be matched, in order, against incoming requests. The first route
	// that matches will be used.
	Routes        []*Route `protobuf:"bytes,2,rep,name=routes,proto3" json:"routes,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RouteConfiguration) Reset() {
	*x = RouteConfiguration{}
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RouteConfiguration) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RouteConfiguration) ProtoMessage() {}

func (x *RouteConfiguration) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RouteConfiguration.ProtoReflect.Descriptor instead.
func (*RouteConfiguration) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescGZIP(), []int{0}
}

func (x *RouteConfiguration) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *RouteConfiguration) GetRoutes() []*Route {
	if x != nil {
		return x.Routes
	}
	return nil
}

type Route struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Route matching parameters.
	Match *RouteMatch `protobuf:"bytes,1,opt,name=match,proto3" json:"match,omitempty"`
	// Route request to some upstream cluster.
	Route         *RouteAction `protobuf:"bytes,2,opt,name=route,proto3" json:"route,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Route) Reset() {
	*x = Route{}
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Route) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Route) ProtoMessage() {}

func (x *Route) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Route.ProtoReflect.Descriptor instead.
func (*Route) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescGZIP(), []int{1}
}

func (x *Route) GetMatch() *RouteMatch {
	if x != nil {
		return x.Match
	}
	return nil
}

func (x *Route) GetRoute() *RouteAction {
	if x != nil {
		return x.Route
	}
	return nil
}

type RouteMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the topic.
	Topic *v3.StringMatcher `protobuf:"bytes,1,opt,name=topic,proto3" json:"topic,omitempty"`
	// Specifies a set of headers that the route should match on. The router will check the request’s
	// headers against all the specified headers in the route config. A match will happen if all the
	// headers in the route are present in the request with the same values (or based on presence if
	// the value field is not in the config).
	Headers       []*v31.HeaderMatcher `protobuf:"bytes,2,rep,name=headers,proto3" json:"headers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RouteMatch) Reset() {
	*x = RouteMatch{}
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RouteMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RouteMatch) ProtoMessage() {}

func (x *RouteMatch) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RouteMatch.ProtoReflect.Descriptor instead.
func (*RouteMatch) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescGZIP(), []int{2}
}

func (x *RouteMatch) GetTopic() *v3.StringMatcher {
	if x != nil {
		return x.Topic
	}
	return nil
}

func (x *RouteMatch) GetHeaders() []*v31.HeaderMatcher {
	if x != nil {
		return x.Headers
	}
	return nil
}

type RouteAction struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Indicates the upstream cluster to which the request should be routed.
	Cluster string `protobuf:"bytes,1,opt,name=cluster,proto3" json:"cluster,omitempty"`
	// Optional endpoint metadata match criteria used by the subset load balancer.
	MetadataMatch *v32.Metadata `protobuf:"bytes,2,opt,name=metadata_match,json=metadataMatch,proto3" json:"metadata_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RouteAction) Reset() {
	*x = RouteAction{}
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RouteAction) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RouteAction) ProtoMessage() {}

func (x *RouteAction) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RouteAction.ProtoReflect.Descriptor instead.
func (*RouteAction) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescGZIP(), []int{3}
}

func (x *RouteAction) GetCluster() string {
	if x != nil {
		return x.Cluster
	}
	return ""
}

func (x *RouteAction) GetMetadataMatch() *v32.Metadata {
	if x != nil {
		return x.MetadataMatch
	}
	return nil
}

var File_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDesc = "" +
	"\n" +
	"Fcontrib/envoy/extensions/filters/network/rocketmq_proxy/v3/route.proto\x122envoy.extensions.filters.network.rocketmq_proxy.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a,envoy/config/route/v3/route_components.proto\x1a\"envoy/type/matcher/v3/string.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"{\n" +
	"\x12RouteConfiguration\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12Q\n" +
	"\x06routes\x18\x02 \x03(\v29.envoy.extensions.filters.network.rocketmq_proxy.v3.RouteR\x06routes\"\xc8\x01\n" +
	"\x05Route\x12^\n" +
	"\x05match\x18\x01 \x01(\v2>.envoy.extensions.filters.network.rocketmq_proxy.v3.RouteMatchB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05match\x12_\n" +
	"\x05route\x18\x02 \x01(\v2?.envoy.extensions.filters.network.rocketmq_proxy.v3.RouteActionB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05route\"\x92\x01\n" +
	"\n" +
	"RouteMatch\x12D\n" +
	"\x05topic\x18\x01 \x01(\v2$.envoy.type.matcher.v3.StringMatcherB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05topic\x12>\n" +
	"\aheaders\x18\x02 \x03(\v2$.envoy.config.route.v3.HeaderMatcherR\aheaders\"w\n" +
	"\vRouteAction\x12!\n" +
	"\acluster\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\acluster\x12E\n" +
	"\x0emetadata_match\x18\x02 \x01(\v2\x1e.envoy.config.core.v3.MetadataR\rmetadataMatchB\xcc\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"@io.envoyproxy.envoy.extensions.filters.network.rocketmq_proxy.v3B\n" +
	"RouteProtoP\x01Zrgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/rocketmq_proxy/v3;rocketmq_proxyv3b\x06proto3"

var (
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDescData
}

var file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_goTypes = []any{
	(*RouteConfiguration)(nil), // 0: envoy.extensions.filters.network.rocketmq_proxy.v3.RouteConfiguration
	(*Route)(nil),              // 1: envoy.extensions.filters.network.rocketmq_proxy.v3.Route
	(*RouteMatch)(nil),         // 2: envoy.extensions.filters.network.rocketmq_proxy.v3.RouteMatch
	(*RouteAction)(nil),        // 3: envoy.extensions.filters.network.rocketmq_proxy.v3.RouteAction
	(*v3.StringMatcher)(nil),   // 4: envoy.type.matcher.v3.StringMatcher
	(*v31.HeaderMatcher)(nil),  // 5: envoy.config.route.v3.HeaderMatcher
	(*v32.Metadata)(nil),       // 6: envoy.config.core.v3.Metadata
}
var file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.rocketmq_proxy.v3.RouteConfiguration.routes:type_name -> envoy.extensions.filters.network.rocketmq_proxy.v3.Route
	2, // 1: envoy.extensions.filters.network.rocketmq_proxy.v3.Route.match:type_name -> envoy.extensions.filters.network.rocketmq_proxy.v3.RouteMatch
	3, // 2: envoy.extensions.filters.network.rocketmq_proxy.v3.Route.route:type_name -> envoy.extensions.filters.network.rocketmq_proxy.v3.RouteAction
	4, // 3: envoy.extensions.filters.network.rocketmq_proxy.v3.RouteMatch.topic:type_name -> envoy.type.matcher.v3.StringMatcher
	5, // 4: envoy.extensions.filters.network.rocketmq_proxy.v3.RouteMatch.headers:type_name -> envoy.config.route.v3.HeaderMatcher
	6, // 5: envoy.extensions.filters.network.rocketmq_proxy.v3.RouteAction.metadata_match:type_name -> envoy.config.core.v3.Metadata
	6, // [6:6] is the sub-list for method output_type
	6, // [6:6] is the sub-list for method input_type
	6, // [6:6] is the sub-list for extension type_name
	6, // [6:6] is the sub-list for extension extendee
	0, // [0:6] is the sub-list for field type_name
}

func init() { file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_init() }
func file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_init() {
	if File_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_depIdxs,
		MessageInfos:      file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto = out.File
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_goTypes = nil
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_depIdxs = nil
}
