// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/set_filter_state/v3/set_filter_state.proto

package set_filter_statev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/common/set_filter_state/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// A sequence of the filter state values to apply in the specified order
	// when a new connection is received.
	OnNewConnection []*v3.FilterStateValue `protobuf:"bytes,1,rep,name=on_new_connection,json=onNewConnection,proto3" json:"on_new_connection,omitempty"`
	// A sequence of the filter state values to apply in the specified order
	// when the downstream TLS handshake is complete.
	//
	// For non-TLS downstream connections (where there is no TLS handshake), this
	// list is applied when a new connection is received.
	OnDownstreamTlsHandshake []*v3.FilterStateValue `protobuf:"bytes,2,rep,name=on_downstream_tls_handshake,json=onDownstreamTlsHandshake,proto3" json:"on_downstream_tls_handshake,omitempty"`
	// A sequence of the filter state values to apply in the specified order
	// when data is first received from the downstream connection.
	OnDownstreamData []*v3.FilterStateValue `protobuf:"bytes,3,rep,name=on_downstream_data,json=onDownstreamData,proto3" json:"on_downstream_data,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetOnNewConnection() []*v3.FilterStateValue {
	if x != nil {
		return x.OnNewConnection
	}
	return nil
}

func (x *Config) GetOnDownstreamTlsHandshake() []*v3.FilterStateValue {
	if x != nil {
		return x.OnDownstreamTlsHandshake
	}
	return nil
}

func (x *Config) GetOnDownstreamData() []*v3.FilterStateValue {
	if x != nil {
		return x.OnDownstreamData
	}
	return nil
}

var File_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDesc = "" +
	"\n" +
	"Kenvoy/extensions/filters/network/set_filter_state/v3/set_filter_state.proto\x124envoy.extensions.filters.network.set_filter_state.v3\x1a?envoy/extensions/filters/common/set_filter_state/v3/value.proto\x1a\x1dudpa/annotations/status.proto\"\xf7\x02\n" +
	"\x06Config\x12q\n" +
	"\x11on_new_connection\x18\x01 \x03(\v2E.envoy.extensions.filters.common.set_filter_state.v3.FilterStateValueR\x0fonNewConnection\x12\x84\x01\n" +
	"\x1bon_downstream_tls_handshake\x18\x02 \x03(\v2E.envoy.extensions.filters.common.set_filter_state.v3.FilterStateValueR\x18onDownstreamTlsHandshake\x12s\n" +
	"\x12on_downstream_data\x18\x03 \x03(\v2E.envoy.extensions.filters.common.set_filter_state.v3.FilterStateValueR\x10onDownstreamDataB\xd3\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Bio.envoyproxy.envoy.extensions.filters.network.set_filter_state.v3B\x13SetFilterStateProtoP\x01Zngithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/set_filter_state/v3;set_filter_statev3b\x06proto3"

var (
	file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDesc), len(file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDescData
}

var file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_goTypes = []any{
	(*Config)(nil),              // 0: envoy.extensions.filters.network.set_filter_state.v3.Config
	(*v3.FilterStateValue)(nil), // 1: envoy.extensions.filters.common.set_filter_state.v3.FilterStateValue
}
var file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.set_filter_state.v3.Config.on_new_connection:type_name -> envoy.extensions.filters.common.set_filter_state.v3.FilterStateValue
	1, // 1: envoy.extensions.filters.network.set_filter_state.v3.Config.on_downstream_tls_handshake:type_name -> envoy.extensions.filters.common.set_filter_state.v3.FilterStateValue
	1, // 2: envoy.extensions.filters.network.set_filter_state.v3.Config.on_downstream_data:type_name -> envoy.extensions.filters.common.set_filter_state.v3.FilterStateValue
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_init() }
func file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_init() {
	if File_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDesc), len(file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto = out.File
	file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_goTypes = nil
	file_envoy_extensions_filters_network_set_filter_state_v3_set_filter_state_proto_depIdxs = nil
}
