// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/router/cluster_specifiers/matcher/v3/matcher.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/cncf/xds/go/xds/type/matcher/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ClusterAction struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Indicates the upstream cluster to which the request should be routed
	// to.
	Cluster       string `protobuf:"bytes,1,opt,name=cluster,proto3" json:"cluster,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClusterAction) Reset() {
	*x = ClusterAction{}
	mi := &file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterAction) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterAction) ProtoMessage() {}

func (x *ClusterAction) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterAction.ProtoReflect.Descriptor instead.
func (*ClusterAction) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDescGZIP(), []int{0}
}

func (x *ClusterAction) GetCluster() string {
	if x != nil {
		return x.Cluster
	}
	return ""
}

type MatcherClusterSpecifier struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The matcher for cluster selection after the route has been selected. This is used when the
	// route has multiple clusters (like multiple clusters for different users) and the matcher
	// is used to select the cluster to use for the request.
	//
	// The match tree to use for grouping incoming requests into buckets.
	//
	// Example:
	//
	// .. validated-code-block:: yaml
	//
	//	:type-name: xds.type.matcher.v3.Matcher
	//
	//	matcher_list:
	//	  matchers:
	//	  - predicate:
	//	      single_predicate:
	//	        input:
	//	          typed_config:
	//	            '@type': type.googleapis.com/envoy.type.matcher.v3.HttpRequestHeaderMatchInput
	//	            header_name: env
	//	        value_match:
	//	          exact: staging
	//	    on_match:
	//	      action:
	//	        typed_config:
	//	          '@type': type.googleapis.com/envoy.extensions.router.cluster_specifiers.matcher.v3.ClusterAction
	//	          cluster: "staging-cluster"
	//
	//	  - predicate:
	//	      single_predicate:
	//	        input:
	//	          typed_config:
	//	            '@type': type.googleapis.com/envoy.type.matcher.v3.HttpRequestHeaderMatchInput
	//	            header_name: env
	//	        value_match:
	//	          exact: prod
	//	    on_match:
	//	      action:
	//	        typed_config:
	//	          '@type': type.googleapis.com/envoy.extensions.router.cluster_specifiers.matcher.v3.ClusterAction
	//	          cluster: "prod-cluster"
	//
	//	# Catch-all with a default cluster.
	//	on_no_match:
	//	  action:
	//	    typed_config:
	//	      '@type': type.googleapis.com/envoy.extensions.router.cluster_specifiers.matcher.v3.ClusterAction
	//	      cluster: "default-cluster"
	ClusterMatcher *v3.Matcher `protobuf:"bytes,1,opt,name=cluster_matcher,json=clusterMatcher,proto3" json:"cluster_matcher,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *MatcherClusterSpecifier) Reset() {
	*x = MatcherClusterSpecifier{}
	mi := &file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MatcherClusterSpecifier) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MatcherClusterSpecifier) ProtoMessage() {}

func (x *MatcherClusterSpecifier) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MatcherClusterSpecifier.ProtoReflect.Descriptor instead.
func (*MatcherClusterSpecifier) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDescGZIP(), []int{1}
}

func (x *MatcherClusterSpecifier) GetClusterMatcher() *v3.Matcher {
	if x != nil {
		return x.ClusterMatcher
	}
	return nil
}

var File_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto protoreflect.FileDescriptor

const file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDesc = "" +
	"\n" +
	"Cenvoy/extensions/router/cluster_specifiers/matcher/v3/matcher.proto\x125envoy.extensions.router.cluster_specifiers.matcher.v3\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"2\n" +
	"\rClusterAction\x12!\n" +
	"\acluster\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\acluster\"j\n" +
	"\x17MatcherClusterSpecifier\x12O\n" +
	"\x0fcluster_matcher\x18\x01 \x01(\v2\x1c.xds.type.matcher.v3.MatcherB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x0eclusterMatcherB\xc5\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Cio.envoyproxy.envoy.extensions.router.cluster_specifiers.matcher.v3B\fMatcherProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/envoy/extensions/router/cluster_specifiers/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDescOnce sync.Once
	file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDescData []byte
)

func file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDescGZIP() []byte {
	file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDesc), len(file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDesc)))
	})
	return file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDescData
}

var file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_goTypes = []any{
	(*ClusterAction)(nil),           // 0: envoy.extensions.router.cluster_specifiers.matcher.v3.ClusterAction
	(*MatcherClusterSpecifier)(nil), // 1: envoy.extensions.router.cluster_specifiers.matcher.v3.MatcherClusterSpecifier
	(*v3.Matcher)(nil),              // 2: xds.type.matcher.v3.Matcher
}
var file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.router.cluster_specifiers.matcher.v3.MatcherClusterSpecifier.cluster_matcher:type_name -> xds.type.matcher.v3.Matcher
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_init() }
func file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_init() {
	if File_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDesc), len(file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_msgTypes,
	}.Build()
	File_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto = out.File
	file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_goTypes = nil
	file_envoy_extensions_router_cluster_specifiers_matcher_v3_matcher_proto_depIdxs = nil
}
