// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/outlier_detection_monitors/consecutive_errors/v3/consecutive_errors.proto

package consecutive_errorsv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/outlier_detection_monitors/common/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Monitor which counts consecutive errors.
// If number of consecutive errors exceeds the threshold, monitor will report that the host
// is unhealthy.
// [#not-implemented-hide:]
type ConsecutiveErrors struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Monitor name.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The number of consecutive errors before ejection occurs.
	Threshold *wrapperspb.UInt32Value `protobuf:"bytes,2,opt,name=threshold,proto3" json:"threshold,omitempty"`
	// The % chance that a host is actually ejected. Defaults to 100.
	Enforcing *wrapperspb.UInt32Value `protobuf:"bytes,3,opt,name=enforcing,proto3" json:"enforcing,omitempty"`
	// Error buckets.
	ErrorBuckets  *v3.ErrorBuckets `protobuf:"bytes,4,opt,name=error_buckets,json=errorBuckets,proto3" json:"error_buckets,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ConsecutiveErrors) Reset() {
	*x = ConsecutiveErrors{}
	mi := &file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ConsecutiveErrors) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ConsecutiveErrors) ProtoMessage() {}

func (x *ConsecutiveErrors) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ConsecutiveErrors.ProtoReflect.Descriptor instead.
func (*ConsecutiveErrors) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDescGZIP(), []int{0}
}

func (x *ConsecutiveErrors) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ConsecutiveErrors) GetThreshold() *wrapperspb.UInt32Value {
	if x != nil {
		return x.Threshold
	}
	return nil
}

func (x *ConsecutiveErrors) GetEnforcing() *wrapperspb.UInt32Value {
	if x != nil {
		return x.Enforcing
	}
	return nil
}

func (x *ConsecutiveErrors) GetErrorBuckets() *v3.ErrorBuckets {
	if x != nil {
		return x.ErrorBuckets
	}
	return nil
}

var File_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto protoreflect.FileDescriptor

const file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDesc = "" +
	"\n" +
	"Zenvoy/extensions/outlier_detection_monitors/consecutive_errors/v3/consecutive_errors.proto\x12Aenvoy.extensions.outlier_detection_monitors.consecutive_errors.v3\x1aGenvoy/extensions/outlier_detection_monitors/common/v3/error_types.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x9b\x02\n" +
	"\x11ConsecutiveErrors\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12C\n" +
	"\tthreshold\x18\x02 \x01(\v2\x1c.google.protobuf.UInt32ValueB\a\xfaB\x04*\x02\x18dR\tthreshold\x12C\n" +
	"\tenforcing\x18\x03 \x01(\v2\x1c.google.protobuf.UInt32ValueB\a\xfaB\x04*\x02\x18dR\tenforcing\x12h\n" +
	"\rerror_buckets\x18\x04 \x01(\v2C.envoy.extensions.outlier_detection_monitors.common.v3.ErrorBucketsR\ferrorBucketsB\xf2\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Oio.envoyproxy.envoy.extensions.outlier_detection_monitors.consecutive_errors.v3B\x16ConsecutiveErrorsProtoP\x01Z}github.com/envoyproxy/go-control-plane/envoy/extensions/outlier_detection_monitors/consecutive_errors/v3;consecutive_errorsv3b\x06proto3"

var (
	file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDescOnce sync.Once
	file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDescData []byte
)

func file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDescGZIP() []byte {
	file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDesc), len(file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDesc)))
	})
	return file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDescData
}

var file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_goTypes = []any{
	(*ConsecutiveErrors)(nil),      // 0: envoy.extensions.outlier_detection_monitors.consecutive_errors.v3.ConsecutiveErrors
	(*wrapperspb.UInt32Value)(nil), // 1: google.protobuf.UInt32Value
	(*v3.ErrorBuckets)(nil),        // 2: envoy.extensions.outlier_detection_monitors.common.v3.ErrorBuckets
}
var file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.outlier_detection_monitors.consecutive_errors.v3.ConsecutiveErrors.threshold:type_name -> google.protobuf.UInt32Value
	1, // 1: envoy.extensions.outlier_detection_monitors.consecutive_errors.v3.ConsecutiveErrors.enforcing:type_name -> google.protobuf.UInt32Value
	2, // 2: envoy.extensions.outlier_detection_monitors.consecutive_errors.v3.ConsecutiveErrors.error_buckets:type_name -> envoy.extensions.outlier_detection_monitors.common.v3.ErrorBuckets
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_init()
}
func file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_init() {
	if File_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDesc), len(file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_msgTypes,
	}.Build()
	File_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto = out.File
	file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_goTypes = nil
	file_envoy_extensions_outlier_detection_monitors_consecutive_errors_v3_consecutive_errors_proto_depIdxs = nil
}
