// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/alternate_protocols_cache/v3/alternate_protocols_cache.proto

package alternate_protocols_cachev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the alternate protocols cache HTTP filter.
// [#extension: envoy.filters.http.alternate_protocols_cache]
type FilterConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This field is ignored: the alternate protocols cache filter will use the
	// cache for the cluster the request is routed to.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/filters/http/alternate_protocols_cache/v3/alternate_protocols_cache.proto.
	AlternateProtocolsCacheOptions *v3.AlternateProtocolsCacheOptions `protobuf:"bytes,1,opt,name=alternate_protocols_cache_options,json=alternateProtocolsCacheOptions,proto3" json:"alternate_protocols_cache_options,omitempty"`
	unknownFields                  protoimpl.UnknownFields
	sizeCache                      protoimpl.SizeCache
}

func (x *FilterConfig) Reset() {
	*x = FilterConfig{}
	mi := &file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FilterConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FilterConfig) ProtoMessage() {}

func (x *FilterConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FilterConfig.ProtoReflect.Descriptor instead.
func (*FilterConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDescGZIP(), []int{0}
}

// Deprecated: Marked as deprecated in envoy/extensions/filters/http/alternate_protocols_cache/v3/alternate_protocols_cache.proto.
func (x *FilterConfig) GetAlternateProtocolsCacheOptions() *v3.AlternateProtocolsCacheOptions {
	if x != nil {
		return x.AlternateProtocolsCacheOptions
	}
	return nil
}

var File_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDesc = "" +
	"\n" +
	"Zenvoy/extensions/filters/http/alternate_protocols_cache/v3/alternate_protocols_cache.proto\x12:envoy.extensions.filters.http.alternate_protocols_cache.v3\x1a#envoy/config/core/v3/protocol.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\"\x9d\x01\n" +
	"\fFilterConfig\x12\x8c\x01\n" +
	"!alternate_protocols_cache_options\x18\x01 \x01(\v24.envoy.config.core.v3.AlternateProtocolsCacheOptionsB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\x1ealternateProtocolsCacheOptionsB\xf1\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Hio.envoyproxy.envoy.extensions.filters.http.alternate_protocols_cache.v3B\x1cAlternateProtocolsCacheProtoP\x01Z}github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/alternate_protocols_cache/v3;alternate_protocols_cachev3b\x06proto3"

var (
	file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDesc), len(file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDescData
}

var file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_goTypes = []any{
	(*FilterConfig)(nil),                      // 0: envoy.extensions.filters.http.alternate_protocols_cache.v3.FilterConfig
	(*v3.AlternateProtocolsCacheOptions)(nil), // 1: envoy.config.core.v3.AlternateProtocolsCacheOptions
}
var file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.http.alternate_protocols_cache.v3.FilterConfig.alternate_protocols_cache_options:type_name -> envoy.config.core.v3.AlternateProtocolsCacheOptions
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_init()
}
func file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_init() {
	if File_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDesc), len(file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto = out.File
	file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_goTypes = nil
	file_envoy_extensions_filters_http_alternate_protocols_cache_v3_alternate_protocols_cache_proto_depIdxs = nil
}
