// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/grpc_credential/v3/file_based_metadata.proto

package grpc_credentialv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type FileBasedMetadataConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Location or inline data of secret to use for authentication of the Google gRPC connection
	// this secret will be attached to a header of the gRPC connection
	SecretData *v3.DataSource `protobuf:"bytes,1,opt,name=secret_data,json=secretData,proto3" json:"secret_data,omitempty"`
	// Metadata header key to use for sending the secret data
	// if no header key is set, "authorization" header will be used
	HeaderKey string `protobuf:"bytes,2,opt,name=header_key,json=headerKey,proto3" json:"header_key,omitempty"`
	// Prefix to prepend to the secret in the metadata header
	// if no prefix is set, the default is to use no prefix
	HeaderPrefix  string `protobuf:"bytes,3,opt,name=header_prefix,json=headerPrefix,proto3" json:"header_prefix,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FileBasedMetadataConfig) Reset() {
	*x = FileBasedMetadataConfig{}
	mi := &file_envoy_config_grpc_credential_v3_file_based_metadata_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FileBasedMetadataConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FileBasedMetadataConfig) ProtoMessage() {}

func (x *FileBasedMetadataConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_grpc_credential_v3_file_based_metadata_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FileBasedMetadataConfig.ProtoReflect.Descriptor instead.
func (*FileBasedMetadataConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDescGZIP(), []int{0}
}

func (x *FileBasedMetadataConfig) GetSecretData() *v3.DataSource {
	if x != nil {
		return x.SecretData
	}
	return nil
}

func (x *FileBasedMetadataConfig) GetHeaderKey() string {
	if x != nil {
		return x.HeaderKey
	}
	return ""
}

func (x *FileBasedMetadataConfig) GetHeaderPrefix() string {
	if x != nil {
		return x.HeaderPrefix
	}
	return ""
}

var File_envoy_config_grpc_credential_v3_file_based_metadata_proto protoreflect.FileDescriptor

const file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDesc = "" +
	"\n" +
	"9envoy/config/grpc_credential/v3/file_based_metadata.proto\x12\x1fenvoy.config.grpc_credential.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a udpa/annotations/sensitive.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xed\x01\n" +
	"\x17FileBasedMetadataConfig\x12I\n" +
	"\vsecret_data\x18\x01 \x01(\v2 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01R\n" +
	"secretData\x12\x1d\n" +
	"\n" +
	"header_key\x18\x02 \x01(\tR\theaderKey\x12#\n" +
	"\rheader_prefix\x18\x03 \x01(\tR\fheaderPrefix:C\x9aň\x1e>\n" +
	"<envoy.config.grpc_credential.v2alpha.FileBasedMetadataConfigB\xab\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"-io.envoyproxy.envoy.config.grpc_credential.v3B\x16FileBasedMetadataProtoP\x01ZXgithub.com/envoyproxy/go-control-plane/envoy/config/grpc_credential/v3;grpc_credentialv3b\x06proto3"

var (
	file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDescOnce sync.Once
	file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDescData []byte
)

func file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDescGZIP() []byte {
	file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDescOnce.Do(func() {
		file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDesc), len(file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDesc)))
	})
	return file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDescData
}

var file_envoy_config_grpc_credential_v3_file_based_metadata_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_grpc_credential_v3_file_based_metadata_proto_goTypes = []any{
	(*FileBasedMetadataConfig)(nil), // 0: envoy.config.grpc_credential.v3.FileBasedMetadataConfig
	(*v3.DataSource)(nil),           // 1: envoy.config.core.v3.DataSource
}
var file_envoy_config_grpc_credential_v3_file_based_metadata_proto_depIdxs = []int32{
	1, // 0: envoy.config.grpc_credential.v3.FileBasedMetadataConfig.secret_data:type_name -> envoy.config.core.v3.DataSource
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_config_grpc_credential_v3_file_based_metadata_proto_init() }
func file_envoy_config_grpc_credential_v3_file_based_metadata_proto_init() {
	if File_envoy_config_grpc_credential_v3_file_based_metadata_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDesc), len(file_envoy_config_grpc_credential_v3_file_based_metadata_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_grpc_credential_v3_file_based_metadata_proto_goTypes,
		DependencyIndexes: file_envoy_config_grpc_credential_v3_file_based_metadata_proto_depIdxs,
		MessageInfos:      file_envoy_config_grpc_credential_v3_file_based_metadata_proto_msgTypes,
	}.Build()
	File_envoy_config_grpc_credential_v3_file_based_metadata_proto = out.File
	file_envoy_config_grpc_credential_v3_file_based_metadata_proto_goTypes = nil
	file_envoy_config_grpc_credential_v3_file_based_metadata_proto_depIdxs = nil
}
