// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/http/header_formatters/preserve_case/v3/preserve_case.proto

package preserve_casev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders int32

const (
	// Use LowerCase on Envoy added headers.
	PreserveCaseFormatterConfig_DEFAULT PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders = 0
	// Use ProperCaseHeaderKeyFormatter on Envoy added headers that upper cases the first character
	// in each word. The first character as well as any alpha character following a special
	// character is upper cased.
	PreserveCaseFormatterConfig_PROPER_CASE PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders = 1
)

// Enum value maps for PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders.
var (
	PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders_name = map[int32]string{
		0: "DEFAULT",
		1: "PROPER_CASE",
	}
	PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders_value = map[string]int32{
		"DEFAULT":     0,
		"PROPER_CASE": 1,
	}
)

func (x PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders) Enum() *PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders {
	p := new(PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders)
	*p = x
	return p
}

func (x PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_enumTypes[0].Descriptor()
}

func (PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders) Type() protoreflect.EnumType {
	return &file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_enumTypes[0]
}

func (x PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders.Descriptor instead.
func (PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDescGZIP(), []int{0, 0}
}

// Configuration for the preserve case header formatter.
// See the :ref:`header casing <config_http_conn_man_header_casing>` configuration guide for more
// information.
type PreserveCaseFormatterConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Allows forwarding reason phrase text.
	// This is off by default, and a standard reason phrase is used for a corresponding HTTP response code.
	ForwardReasonPhrase bool `protobuf:"varint,1,opt,name=forward_reason_phrase,json=forwardReasonPhrase,proto3" json:"forward_reason_phrase,omitempty"`
	// Type of formatter to use on headers which are added by Envoy (which are lower case by default).
	// The default type is DEFAULT, use LowerCase on Envoy headers.
	FormatterTypeOnEnvoyHeaders PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders `protobuf:"varint,2,opt,name=formatter_type_on_envoy_headers,json=formatterTypeOnEnvoyHeaders,proto3,enum=envoy.extensions.http.header_formatters.preserve_case.v3.PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders" json:"formatter_type_on_envoy_headers,omitempty"`
	unknownFields               protoimpl.UnknownFields
	sizeCache                   protoimpl.SizeCache
}

func (x *PreserveCaseFormatterConfig) Reset() {
	*x = PreserveCaseFormatterConfig{}
	mi := &file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PreserveCaseFormatterConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PreserveCaseFormatterConfig) ProtoMessage() {}

func (x *PreserveCaseFormatterConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PreserveCaseFormatterConfig.ProtoReflect.Descriptor instead.
func (*PreserveCaseFormatterConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDescGZIP(), []int{0}
}

func (x *PreserveCaseFormatterConfig) GetForwardReasonPhrase() bool {
	if x != nil {
		return x.ForwardReasonPhrase
	}
	return false
}

func (x *PreserveCaseFormatterConfig) GetFormatterTypeOnEnvoyHeaders() PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders {
	if x != nil {
		return x.FormatterTypeOnEnvoyHeaders
	}
	return PreserveCaseFormatterConfig_DEFAULT
}

var File_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto protoreflect.FileDescriptor

const file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDesc = "" +
	"\n" +
	"Lenvoy/extensions/http/header_formatters/preserve_case/v3/preserve_case.proto\x128envoy.extensions.http.header_formatters.preserve_case.v3\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd2\x02\n" +
	"\x1bPreserveCaseFormatterConfig\x122\n" +
	"\x15forward_reason_phrase\x18\x01 \x01(\bR\x13forwardReasonPhrase\x12\xc1\x01\n" +
	"\x1fformatter_type_on_envoy_headers\x18\x02 \x01(\x0e2q.envoy.extensions.http.header_formatters.preserve_case.v3.PreserveCaseFormatterConfig.FormatterTypeOnEnvoyHeadersB\b\xfaB\x05\x82\x01\x02\x10\x01R\x1bformatterTypeOnEnvoyHeaders\";\n" +
	"\x1bFormatterTypeOnEnvoyHeaders\x12\v\n" +
	"\aDEFAULT\x10\x00\x12\x0f\n" +
	"\vPROPER_CASE\x10\x01B\xd6\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Fio.envoyproxy.envoy.extensions.http.header_formatters.preserve_case.v3B\x11PreserveCaseProtoP\x01Zogithub.com/envoyproxy/go-control-plane/envoy/extensions/http/header_formatters/preserve_case/v3;preserve_casev3b\x06proto3"

var (
	file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDescOnce sync.Once
	file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDescData []byte
)

func file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDescGZIP() []byte {
	file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDesc), len(file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDesc)))
	})
	return file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDescData
}

var file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_goTypes = []any{
	(PreserveCaseFormatterConfig_FormatterTypeOnEnvoyHeaders)(0), // 0: envoy.extensions.http.header_formatters.preserve_case.v3.PreserveCaseFormatterConfig.FormatterTypeOnEnvoyHeaders
	(*PreserveCaseFormatterConfig)(nil),                          // 1: envoy.extensions.http.header_formatters.preserve_case.v3.PreserveCaseFormatterConfig
}
var file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_depIdxs = []int32{
	0, // 0: envoy.extensions.http.header_formatters.preserve_case.v3.PreserveCaseFormatterConfig.formatter_type_on_envoy_headers:type_name -> envoy.extensions.http.header_formatters.preserve_case.v3.PreserveCaseFormatterConfig.FormatterTypeOnEnvoyHeaders
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_init() }
func file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_init() {
	if File_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDesc), len(file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_msgTypes,
	}.Build()
	File_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto = out.File
	file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_goTypes = nil
	file_envoy_extensions_http_header_formatters_preserve_case_v3_preserve_case_proto_depIdxs = nil
}
