// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/composite/v3/composite.proto

package compositev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/cncf/xds/go/xds/type/matcher/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// :ref:`Composite filter <config_http_filters_composite>` config. The composite filter config
// allows delegating filter handling to another filter as determined by matching on the request
// headers. This makes it possible to use different filters or filter configurations based on the
// incoming request.
//
// This is intended to be used with
// :ref:`ExtensionWithMatcher <envoy_v3_api_msg_extensions.common.matching.v3.ExtensionWithMatcher>`
// where a match tree is specified that indicates (via
// :ref:`ExecuteFilterAction <envoy_v3_api_msg_extensions.filters.http.composite.v3.ExecuteFilterAction>`)
// which filter configuration to create and delegate to.
type Composite struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Named filter chain definitions that can be referenced from
	// :ref:`ExecuteFilterAction.filter_chain_name
	// <envoy_v3_api_field_extensions.filters.http.composite.v3.ExecuteFilterAction.filter_chain_name>`.
	// The filter chains are compiled at configuration time and can be referenced by name.
	// This is useful when the same filter chain needs to be applied across many routes,
	// as it avoids duplicating the filter chain configuration.
	NamedFilterChains map[string]*FilterChainConfiguration `protobuf:"bytes,1,rep,name=named_filter_chains,json=namedFilterChains,proto3" json:"named_filter_chains,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	// [#not-implemented-hide:]
	// The match tree that will be used to select an action to execute. The action type should be
	// :ref:`ExecuteFilterAction
	// <envoy_v3_api_msg_extensions.filters.http.composite.v3.ExecuteFilterAction>`.
	Matcher       *v3.Matcher `protobuf:"bytes,2,opt,name=matcher,proto3" json:"matcher,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Composite) Reset() {
	*x = Composite{}
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Composite) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Composite) ProtoMessage() {}

func (x *Composite) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Composite.ProtoReflect.Descriptor instead.
func (*Composite) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescGZIP(), []int{0}
}

func (x *Composite) GetNamedFilterChains() map[string]*FilterChainConfiguration {
	if x != nil {
		return x.NamedFilterChains
	}
	return nil
}

func (x *Composite) GetMatcher() *v3.Matcher {
	if x != nil {
		return x.Matcher
	}
	return nil
}

// Per-route configuration for the Composite filter.
// [#not-implemented-hide:]
type CompositePerRoute struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Override of the match tree for this route.
	Matcher       *v3.Matcher `protobuf:"bytes,1,opt,name=matcher,proto3" json:"matcher,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CompositePerRoute) Reset() {
	*x = CompositePerRoute{}
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CompositePerRoute) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CompositePerRoute) ProtoMessage() {}

func (x *CompositePerRoute) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CompositePerRoute.ProtoReflect.Descriptor instead.
func (*CompositePerRoute) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescGZIP(), []int{1}
}

func (x *CompositePerRoute) GetMatcher() *v3.Matcher {
	if x != nil {
		return x.Matcher
	}
	return nil
}

// A list of filter configurations to be called in order. Note that this can be used as the type
// inside of an ECDS :ref:`TypedExtensionConfig
// <envoy_v3_api_msg_config.core.v3.TypedExtensionConfig>` extension, which allows a chain of
// filters to be configured dynamically. In that case, the types of all filters in the chain must
// be present in the :ref:`ExtensionConfigSource.type_urls
// <envoy_v3_api_field_config.core.v3.ExtensionConfigSource.type_urls>` field.
type FilterChainConfiguration struct {
	state         protoimpl.MessageState      `protogen:"open.v1"`
	TypedConfig   []*v31.TypedExtensionConfig `protobuf:"bytes,1,rep,name=typed_config,json=typedConfig,proto3" json:"typed_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FilterChainConfiguration) Reset() {
	*x = FilterChainConfiguration{}
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FilterChainConfiguration) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FilterChainConfiguration) ProtoMessage() {}

func (x *FilterChainConfiguration) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FilterChainConfiguration.ProtoReflect.Descriptor instead.
func (*FilterChainConfiguration) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescGZIP(), []int{2}
}

func (x *FilterChainConfiguration) GetTypedConfig() []*v31.TypedExtensionConfig {
	if x != nil {
		return x.TypedConfig
	}
	return nil
}

// Configuration for an extension configuration discovery service with name.
type DynamicConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the extension configuration. It also serves as a resource name in ExtensionConfigDS.
	// The resource type in the “DiscoveryRequest“ will be :ref:`TypedExtensionConfig
	// <envoy_v3_api_msg_config.core.v3.TypedExtensionConfig>`.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Configuration source specifier for an extension configuration discovery
	// service. In case of a failure and without the default configuration,
	// 500(Internal Server Error) will be returned.
	ConfigDiscovery *v31.ExtensionConfigSource `protobuf:"bytes,2,opt,name=config_discovery,json=configDiscovery,proto3" json:"config_discovery,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *DynamicConfig) Reset() {
	*x = DynamicConfig{}
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DynamicConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DynamicConfig) ProtoMessage() {}

func (x *DynamicConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DynamicConfig.ProtoReflect.Descriptor instead.
func (*DynamicConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescGZIP(), []int{3}
}

func (x *DynamicConfig) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *DynamicConfig) GetConfigDiscovery() *v31.ExtensionConfigSource {
	if x != nil {
		return x.ConfigDiscovery
	}
	return nil
}

// Composite match action (see :ref:`matching docs <arch_overview_matching_api>` for more info on match actions).
// This specifies the filter configuration of the filter that the composite filter should delegate filter interactions to.
// [#next-free-field: 6]
type ExecuteFilterAction struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Filter specific configuration which depends on the filter being
	// instantiated. See the supported filters for further documentation.
	// Only one of “typed_config“, “dynamic_config“, “filter_chain“, or “filter_chain_name“
	// can be set.
	// [#extension-category: envoy.filters.http]
	TypedConfig *v31.TypedExtensionConfig `protobuf:"bytes,1,opt,name=typed_config,json=typedConfig,proto3" json:"typed_config,omitempty"`
	// Dynamic configuration of filter obtained via extension configuration discovery service.
	// Only one of “typed_config“, “dynamic_config“, “filter_chain“, or “filter_chain_name“
	// can be set.
	DynamicConfig *DynamicConfig `protobuf:"bytes,2,opt,name=dynamic_config,json=dynamicConfig,proto3" json:"dynamic_config,omitempty"`
	// An inlined list of filter configurations. The specified filters will be executed in order.
	// Only one of “typed_config“, “dynamic_config“, “filter_chain“, or “filter_chain_name“
	// can be set.
	FilterChain *FilterChainConfiguration `protobuf:"bytes,4,opt,name=filter_chain,json=filterChain,proto3" json:"filter_chain,omitempty"`
	// The name of a filter chain defined in
	// :ref:`Composite.named_filter_chains
	// <envoy_v3_api_field_extensions.filters.http.composite.v3.Composite.named_filter_chains>`.
	// At runtime, if the named filter chain is not found in the Composite filter's configuration,
	// no filter will be applied for this match (the action is silently skipped).
	// Only one of “typed_config“, “dynamic_config“, “filter_chain“, or “filter_chain_name“
	// can be set.
	FilterChainName string `protobuf:"bytes,5,opt,name=filter_chain_name,json=filterChainName,proto3" json:"filter_chain_name,omitempty"`
	// Probability of the action execution. If not specified, this is 100%.
	// This allows sampling behavior for the configured actions.
	// For example, if
	// :ref:`default_value <envoy_v3_api_field_config.core.v3.RuntimeFractionalPercent.default_value>`
	// under the “sample_percent“ is configured with 30%, a dice roll with that
	// probability is done. The underline action will only be executed if the
	// dice roll returns positive. Otherwise, the action is skipped.
	SamplePercent *v31.RuntimeFractionalPercent `protobuf:"bytes,3,opt,name=sample_percent,json=samplePercent,proto3" json:"sample_percent,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ExecuteFilterAction) Reset() {
	*x = ExecuteFilterAction{}
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ExecuteFilterAction) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ExecuteFilterAction) ProtoMessage() {}

func (x *ExecuteFilterAction) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ExecuteFilterAction.ProtoReflect.Descriptor instead.
func (*ExecuteFilterAction) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescGZIP(), []int{4}
}

func (x *ExecuteFilterAction) GetTypedConfig() *v31.TypedExtensionConfig {
	if x != nil {
		return x.TypedConfig
	}
	return nil
}

func (x *ExecuteFilterAction) GetDynamicConfig() *DynamicConfig {
	if x != nil {
		return x.DynamicConfig
	}
	return nil
}

func (x *ExecuteFilterAction) GetFilterChain() *FilterChainConfiguration {
	if x != nil {
		return x.FilterChain
	}
	return nil
}

func (x *ExecuteFilterAction) GetFilterChainName() string {
	if x != nil {
		return x.FilterChainName
	}
	return ""
}

func (x *ExecuteFilterAction) GetSamplePercent() *v31.RuntimeFractionalPercent {
	if x != nil {
		return x.SamplePercent
	}
	return nil
}

var File_envoy_extensions_filters_http_composite_v3_composite_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDesc = "" +
	"\n" +
	":envoy/extensions/filters/http/composite/v3/composite.proto\x12*envoy.extensions.filters.http.composite.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a(envoy/config/core/v3/config_source.proto\x1a$envoy/config/core/v3/extension.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xce\x02\n" +
	"\tComposite\x12|\n" +
	"\x13named_filter_chains\x18\x01 \x03(\v2L.envoy.extensions.filters.http.composite.v3.Composite.NamedFilterChainsEntryR\x11namedFilterChains\x126\n" +
	"\amatcher\x18\x02 \x01(\v2\x1c.xds.type.matcher.v3.MatcherR\amatcher\x1a\x8a\x01\n" +
	"\x16NamedFilterChainsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12Z\n" +
	"\x05value\x18\x02 \x01(\v2D.envoy.extensions.filters.http.composite.v3.FilterChainConfigurationR\x05value:\x028\x01\"U\n" +
	"\x11CompositePerRoute\x12@\n" +
	"\amatcher\x18\x01 \x01(\v2\x1c.xds.type.matcher.v3.MatcherB\b\xfaB\x05\x8a\x01\x02\x10\x01R\amatcher\"i\n" +
	"\x18FilterChainConfiguration\x12M\n" +
	"\ftyped_config\x18\x01 \x03(\v2*.envoy.config.core.v3.TypedExtensionConfigR\vtypedConfig\"\x84\x01\n" +
	"\rDynamicConfig\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12V\n" +
	"\x10config_discovery\x18\x02 \x01(\v2+.envoy.config.core.v3.ExtensionConfigSourceR\x0fconfigDiscovery\"\xdc\x03\n" +
	"\x13ExecuteFilterAction\x12b\n" +
	"\ftyped_config\x18\x01 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigB\x13\xf2\x98\xfe\x8f\x05\r\x12\vconfig_typeR\vtypedConfig\x12u\n" +
	"\x0edynamic_config\x18\x02 \x01(\v29.envoy.extensions.filters.http.composite.v3.DynamicConfigB\x13\xf2\x98\xfe\x8f\x05\r\x12\vconfig_typeR\rdynamicConfig\x12g\n" +
	"\ffilter_chain\x18\x04 \x01(\v2D.envoy.extensions.filters.http.composite.v3.FilterChainConfigurationR\vfilterChain\x12*\n" +
	"\x11filter_chain_name\x18\x05 \x01(\tR\x0ffilterChainName\x12U\n" +
	"\x0esample_percent\x18\x03 \x01(\v2..envoy.config.core.v3.RuntimeFractionalPercentR\rsamplePercentB\xb3\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"8io.envoyproxy.envoy.extensions.filters.http.composite.v3B\x0eCompositeProtoP\x01Z]github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/composite/v3;compositev3b\x06proto3"

var (
	file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDesc), len(file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDescData
}

var file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes = make([]protoimpl.MessageInfo, 6)
var file_envoy_extensions_filters_http_composite_v3_composite_proto_goTypes = []any{
	(*Composite)(nil),                    // 0: envoy.extensions.filters.http.composite.v3.Composite
	(*CompositePerRoute)(nil),            // 1: envoy.extensions.filters.http.composite.v3.CompositePerRoute
	(*FilterChainConfiguration)(nil),     // 2: envoy.extensions.filters.http.composite.v3.FilterChainConfiguration
	(*DynamicConfig)(nil),                // 3: envoy.extensions.filters.http.composite.v3.DynamicConfig
	(*ExecuteFilterAction)(nil),          // 4: envoy.extensions.filters.http.composite.v3.ExecuteFilterAction
	nil,                                  // 5: envoy.extensions.filters.http.composite.v3.Composite.NamedFilterChainsEntry
	(*v3.Matcher)(nil),                   // 6: xds.type.matcher.v3.Matcher
	(*v31.TypedExtensionConfig)(nil),     // 7: envoy.config.core.v3.TypedExtensionConfig
	(*v31.ExtensionConfigSource)(nil),    // 8: envoy.config.core.v3.ExtensionConfigSource
	(*v31.RuntimeFractionalPercent)(nil), // 9: envoy.config.core.v3.RuntimeFractionalPercent
}
var file_envoy_extensions_filters_http_composite_v3_composite_proto_depIdxs = []int32{
	5,  // 0: envoy.extensions.filters.http.composite.v3.Composite.named_filter_chains:type_name -> envoy.extensions.filters.http.composite.v3.Composite.NamedFilterChainsEntry
	6,  // 1: envoy.extensions.filters.http.composite.v3.Composite.matcher:type_name -> xds.type.matcher.v3.Matcher
	6,  // 2: envoy.extensions.filters.http.composite.v3.CompositePerRoute.matcher:type_name -> xds.type.matcher.v3.Matcher
	7,  // 3: envoy.extensions.filters.http.composite.v3.FilterChainConfiguration.typed_config:type_name -> envoy.config.core.v3.TypedExtensionConfig
	8,  // 4: envoy.extensions.filters.http.composite.v3.DynamicConfig.config_discovery:type_name -> envoy.config.core.v3.ExtensionConfigSource
	7,  // 5: envoy.extensions.filters.http.composite.v3.ExecuteFilterAction.typed_config:type_name -> envoy.config.core.v3.TypedExtensionConfig
	3,  // 6: envoy.extensions.filters.http.composite.v3.ExecuteFilterAction.dynamic_config:type_name -> envoy.extensions.filters.http.composite.v3.DynamicConfig
	2,  // 7: envoy.extensions.filters.http.composite.v3.ExecuteFilterAction.filter_chain:type_name -> envoy.extensions.filters.http.composite.v3.FilterChainConfiguration
	9,  // 8: envoy.extensions.filters.http.composite.v3.ExecuteFilterAction.sample_percent:type_name -> envoy.config.core.v3.RuntimeFractionalPercent
	2,  // 9: envoy.extensions.filters.http.composite.v3.Composite.NamedFilterChainsEntry.value:type_name -> envoy.extensions.filters.http.composite.v3.FilterChainConfiguration
	10, // [10:10] is the sub-list for method output_type
	10, // [10:10] is the sub-list for method input_type
	10, // [10:10] is the sub-list for extension type_name
	10, // [10:10] is the sub-list for extension extendee
	0,  // [0:10] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_http_composite_v3_composite_proto_init() }
func file_envoy_extensions_filters_http_composite_v3_composite_proto_init() {
	if File_envoy_extensions_filters_http_composite_v3_composite_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDesc), len(file_envoy_extensions_filters_http_composite_v3_composite_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   6,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_composite_v3_composite_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_composite_v3_composite_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_composite_v3_composite_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_composite_v3_composite_proto = out.File
	file_envoy_extensions_filters_http_composite_v3_composite_proto_goTypes = nil
	file_envoy_extensions_filters_http_composite_v3_composite_proto_depIdxs = nil
}
