// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/clusters/dynamic_modules/v3/cluster.proto

package dynamic_modulesv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/dynamic_modules/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the dynamic modules cluster.
//
// This cluster type delegates host discovery and load balancing to a dynamic module. The module
// manages hosts via callbacks such as “envoy_dynamic_module_callback_cluster_add_host“ and
// “envoy_dynamic_module_callback_cluster_remove_host“. The cluster must use
// “lb_policy: CLUSTER_PROVIDED“ since the module provides its own load balancer.
//
// [#extension: envoy.clusters.dynamic_modules]
type ClusterConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The dynamic module configuration for the cluster.
	DynamicModuleConfig *v3.DynamicModuleConfig `protobuf:"bytes,1,opt,name=dynamic_module_config,json=dynamicModuleConfig,proto3" json:"dynamic_module_config,omitempty"`
	// The name to identify the cluster implementation within the module.
	// This is passed to the module's “envoy_dynamic_module_on_cluster_config_new“ function.
	ClusterName string `protobuf:"bytes,2,opt,name=cluster_name,json=clusterName,proto3" json:"cluster_name,omitempty"`
	// The configuration for the module's cluster implementation.
	// This is passed to the module's “envoy_dynamic_module_on_cluster_config_new“ function.
	// The configuration can be any protobuf message. However, it is recommended to use
	// “google.protobuf.Struct“, “google.protobuf.StringValue“, or “google.protobuf.BytesValue“.
	// These types are passed directly as bytes to the module, so the module does not need to have
	// knowledge of protobuf encoding. Otherwise, the serialized bytes of the type are passed.
	// If not specified, an empty configuration is passed.
	ClusterConfig *anypb.Any `protobuf:"bytes,3,opt,name=cluster_config,json=clusterConfig,proto3" json:"cluster_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClusterConfig) Reset() {
	*x = ClusterConfig{}
	mi := &file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterConfig) ProtoMessage() {}

func (x *ClusterConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterConfig.ProtoReflect.Descriptor instead.
func (*ClusterConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDescGZIP(), []int{0}
}

func (x *ClusterConfig) GetDynamicModuleConfig() *v3.DynamicModuleConfig {
	if x != nil {
		return x.DynamicModuleConfig
	}
	return nil
}

func (x *ClusterConfig) GetClusterName() string {
	if x != nil {
		return x.ClusterName
	}
	return ""
}

func (x *ClusterConfig) GetClusterConfig() *anypb.Any {
	if x != nil {
		return x.ClusterConfig
	}
	return nil
}

var File_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto protoreflect.FileDescriptor

const file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDesc = "" +
	"\n" +
	":envoy/extensions/clusters/dynamic_modules/v3/cluster.proto\x12,envoy.extensions.clusters.dynamic_modules.v3\x1a9envoy/extensions/dynamic_modules/v3/dynamic_modules.proto\x1a\x19google/protobuf/any.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xf0\x01\n" +
	"\rClusterConfig\x12v\n" +
	"\x15dynamic_module_config\x18\x01 \x01(\v28.envoy.extensions.dynamic_modules.v3.DynamicModuleConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x13dynamicModuleConfig\x12*\n" +
	"\fcluster_name\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vclusterName\x12;\n" +
	"\x0ecluster_config\x18\x03 \x01(\v2\x14.google.protobuf.AnyR\rclusterConfigB\xbb\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	":io.envoyproxy.envoy.extensions.clusters.dynamic_modules.v3B\fClusterProtoP\x01Zegithub.com/envoyproxy/go-control-plane/envoy/extensions/clusters/dynamic_modules/v3;dynamic_modulesv3b\x06proto3"

var (
	file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDescOnce sync.Once
	file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDescData []byte
)

func file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDescGZIP() []byte {
	file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDesc), len(file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDesc)))
	})
	return file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDescData
}

var file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_goTypes = []any{
	(*ClusterConfig)(nil),          // 0: envoy.extensions.clusters.dynamic_modules.v3.ClusterConfig
	(*v3.DynamicModuleConfig)(nil), // 1: envoy.extensions.dynamic_modules.v3.DynamicModuleConfig
	(*anypb.Any)(nil),              // 2: google.protobuf.Any
}
var file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.clusters.dynamic_modules.v3.ClusterConfig.dynamic_module_config:type_name -> envoy.extensions.dynamic_modules.v3.DynamicModuleConfig
	2, // 1: envoy.extensions.clusters.dynamic_modules.v3.ClusterConfig.cluster_config:type_name -> google.protobuf.Any
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_init() }
func file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_init() {
	if File_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDesc), len(file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_msgTypes,
	}.Build()
	File_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto = out.File
	file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_goTypes = nil
	file_envoy_extensions_clusters_dynamic_modules_v3_cluster_proto_depIdxs = nil
}
