// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/on_demand/v3/on_demand.proto

package on_demandv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration of on-demand CDS.
type OnDemandCds struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// A configuration source for the service that will be used for
	// on-demand cluster discovery.
	Source *v3.ConfigSource `protobuf:"bytes,1,opt,name=source,proto3" json:"source,omitempty"`
	// xdstp:// resource locator for on-demand cluster collection.
	ResourcesLocator string `protobuf:"bytes,2,opt,name=resources_locator,json=resourcesLocator,proto3" json:"resources_locator,omitempty"`
	// The timeout for on demand cluster lookup. If not set, defaults to 5 seconds.
	Timeout       *durationpb.Duration `protobuf:"bytes,3,opt,name=timeout,proto3" json:"timeout,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OnDemandCds) Reset() {
	*x = OnDemandCds{}
	mi := &file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OnDemandCds) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OnDemandCds) ProtoMessage() {}

func (x *OnDemandCds) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OnDemandCds.ProtoReflect.Descriptor instead.
func (*OnDemandCds) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescGZIP(), []int{0}
}

func (x *OnDemandCds) GetSource() *v3.ConfigSource {
	if x != nil {
		return x.Source
	}
	return nil
}

func (x *OnDemandCds) GetResourcesLocator() string {
	if x != nil {
		return x.ResourcesLocator
	}
	return ""
}

func (x *OnDemandCds) GetTimeout() *durationpb.Duration {
	if x != nil {
		return x.Timeout
	}
	return nil
}

// On Demand Discovery filter config.
type OnDemand struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// An optional configuration for on-demand cluster discovery
	// service. If not specified, the on-demand cluster discovery will
	// be disabled. When it's specified, the filter will pause the
	// request to an unknown cluster and will begin a cluster discovery
	// process. When the discovery is finished (successfully or not), the
	// request will be resumed for further processing.
	Odcds         *OnDemandCds `protobuf:"bytes,1,opt,name=odcds,proto3" json:"odcds,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OnDemand) Reset() {
	*x = OnDemand{}
	mi := &file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OnDemand) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OnDemand) ProtoMessage() {}

func (x *OnDemand) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OnDemand.ProtoReflect.Descriptor instead.
func (*OnDemand) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescGZIP(), []int{1}
}

func (x *OnDemand) GetOdcds() *OnDemandCds {
	if x != nil {
		return x.Odcds
	}
	return nil
}

// Per-route configuration for On Demand Discovery.
type PerRouteConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// An optional configuration for on-demand cluster discovery
	// service. If not specified, the on-demand cluster discovery will
	// be disabled. When it's specified, the filter will pause the
	// request to an unknown cluster and will begin a cluster discovery
	// process. When the discovery is finished (successfully or not), the
	// request will be resumed for further processing.
	Odcds         *OnDemandCds `protobuf:"bytes,1,opt,name=odcds,proto3" json:"odcds,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PerRouteConfig) Reset() {
	*x = PerRouteConfig{}
	mi := &file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PerRouteConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PerRouteConfig) ProtoMessage() {}

func (x *PerRouteConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PerRouteConfig.ProtoReflect.Descriptor instead.
func (*PerRouteConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescGZIP(), []int{2}
}

func (x *PerRouteConfig) GetOdcds() *OnDemandCds {
	if x != nil {
		return x.Odcds
	}
	return nil
}

var File_envoy_extensions_filters_http_on_demand_v3_on_demand_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDesc = "" +
	"\n" +
	":envoy/extensions/filters/http/on_demand/v3/on_demand.proto\x12*envoy.extensions.filters.http.on_demand.v3\x1a(envoy/config/core/v3/config_source.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xab\x01\n" +
	"\vOnDemandCds\x12:\n" +
	"\x06source\x18\x01 \x01(\v2\".envoy.config.core.v3.ConfigSourceR\x06source\x12+\n" +
	"\x11resources_locator\x18\x02 \x01(\tR\x10resourcesLocator\x123\n" +
	"\atimeout\x18\x03 \x01(\v2\x19.google.protobuf.DurationR\atimeout\"\x90\x01\n" +
	"\bOnDemand\x12M\n" +
	"\x05odcds\x18\x01 \x01(\v27.envoy.extensions.filters.http.on_demand.v3.OnDemandCdsR\x05odcds:5\x9aň\x1e0\n" +
	".envoy.config.filter.http.on_demand.v2.OnDemand\"_\n" +
	"\x0ePerRouteConfig\x12M\n" +
	"\x05odcds\x18\x01 \x01(\v27.envoy.extensions.filters.http.on_demand.v3.OnDemandCdsR\x05odcdsB\xb2\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"8io.envoyproxy.envoy.extensions.filters.http.on_demand.v3B\rOnDemandProtoP\x01Z]github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/on_demand/v3;on_demandv3b\x06proto3"

var (
	file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDesc), len(file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDescData
}

var file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_goTypes = []any{
	(*OnDemandCds)(nil),         // 0: envoy.extensions.filters.http.on_demand.v3.OnDemandCds
	(*OnDemand)(nil),            // 1: envoy.extensions.filters.http.on_demand.v3.OnDemand
	(*PerRouteConfig)(nil),      // 2: envoy.extensions.filters.http.on_demand.v3.PerRouteConfig
	(*v3.ConfigSource)(nil),     // 3: envoy.config.core.v3.ConfigSource
	(*durationpb.Duration)(nil), // 4: google.protobuf.Duration
}
var file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_depIdxs = []int32{
	3, // 0: envoy.extensions.filters.http.on_demand.v3.OnDemandCds.source:type_name -> envoy.config.core.v3.ConfigSource
	4, // 1: envoy.extensions.filters.http.on_demand.v3.OnDemandCds.timeout:type_name -> google.protobuf.Duration
	0, // 2: envoy.extensions.filters.http.on_demand.v3.OnDemand.odcds:type_name -> envoy.extensions.filters.http.on_demand.v3.OnDemandCds
	0, // 3: envoy.extensions.filters.http.on_demand.v3.PerRouteConfig.odcds:type_name -> envoy.extensions.filters.http.on_demand.v3.OnDemandCds
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_init() }
func file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_init() {
	if File_envoy_extensions_filters_http_on_demand_v3_on_demand_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDesc), len(file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_on_demand_v3_on_demand_proto = out.File
	file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_goTypes = nil
	file_envoy_extensions_filters_http_on_demand_v3_on_demand_proto_depIdxs = nil
}
