// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/data/cluster/v3/outlier_detection_event.proto

package clusterv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Type of ejection that took place
type OutlierEjectionType int32

const (
	// In case upstream host returns certain number of consecutive 5xx.
	// If
	// :ref:`outlier_detection.split_external_local_origin_errors<envoy_v3_api_field_config.cluster.v3.OutlierDetection.split_external_local_origin_errors>`
	// is “false“, all type of errors are treated as HTTP 5xx errors.
	// See :ref:`Cluster outlier detection <arch_overview_outlier_detection>` documentation for
	// details.
	OutlierEjectionType_CONSECUTIVE_5XX OutlierEjectionType = 0
	// In case upstream host returns certain number of consecutive gateway errors
	OutlierEjectionType_CONSECUTIVE_GATEWAY_FAILURE OutlierEjectionType = 1
	// Runs over aggregated success rate statistics from every host in cluster
	// and selects hosts for which ratio of successful replies deviates from other hosts
	// in the cluster.
	// If
	// :ref:`outlier_detection.split_external_local_origin_errors<envoy_v3_api_field_config.cluster.v3.OutlierDetection.split_external_local_origin_errors>`
	// is “false“, all errors (externally and locally generated) are used to calculate success rate
	// statistics. See :ref:`Cluster outlier detection <arch_overview_outlier_detection>`
	// documentation for details.
	OutlierEjectionType_SUCCESS_RATE OutlierEjectionType = 2
	// Consecutive local origin failures: Connection failures, resets, timeouts, etc
	// This type of ejection happens only when
	// :ref:`outlier_detection.split_external_local_origin_errors<envoy_v3_api_field_config.cluster.v3.OutlierDetection.split_external_local_origin_errors>`
	// is set to “true“.
	// See :ref:`Cluster outlier detection <arch_overview_outlier_detection>` documentation for
	OutlierEjectionType_CONSECUTIVE_LOCAL_ORIGIN_FAILURE OutlierEjectionType = 3
	// Runs over aggregated success rate statistics for local origin failures
	// for all hosts in the cluster and selects hosts for which success rate deviates from other
	// hosts in the cluster. This type of ejection happens only when
	// :ref:`outlier_detection.split_external_local_origin_errors<envoy_v3_api_field_config.cluster.v3.OutlierDetection.split_external_local_origin_errors>`
	// is set to “true“.
	// See :ref:`Cluster outlier detection <arch_overview_outlier_detection>` documentation for
	OutlierEjectionType_SUCCESS_RATE_LOCAL_ORIGIN OutlierEjectionType = 4
	// Runs over aggregated success rate statistics from every host in cluster and selects hosts for
	// which ratio of failed replies is above configured value.
	OutlierEjectionType_FAILURE_PERCENTAGE OutlierEjectionType = 5
	// Runs over aggregated success rate statistics for local origin failures from every host in
	// cluster and selects hosts for which ratio of failed replies is above configured value.
	OutlierEjectionType_FAILURE_PERCENTAGE_LOCAL_ORIGIN OutlierEjectionType = 6
	// Host is detected as degraded via passive health checking (outlier detection).
	// The host returns responses with the x-envoy-degraded header, indicating it is under stress
	// but still able to serve traffic. Degraded hosts are deprioritized in load balancing but not
	// fully ejected.
	OutlierEjectionType_DEGRADED OutlierEjectionType = 7
)

// Enum value maps for OutlierEjectionType.
var (
	OutlierEjectionType_name = map[int32]string{
		0: "CONSECUTIVE_5XX",
		1: "CONSECUTIVE_GATEWAY_FAILURE",
		2: "SUCCESS_RATE",
		3: "CONSECUTIVE_LOCAL_ORIGIN_FAILURE",
		4: "SUCCESS_RATE_LOCAL_ORIGIN",
		5: "FAILURE_PERCENTAGE",
		6: "FAILURE_PERCENTAGE_LOCAL_ORIGIN",
		7: "DEGRADED",
	}
	OutlierEjectionType_value = map[string]int32{
		"CONSECUTIVE_5XX":                  0,
		"CONSECUTIVE_GATEWAY_FAILURE":      1,
		"SUCCESS_RATE":                     2,
		"CONSECUTIVE_LOCAL_ORIGIN_FAILURE": 3,
		"SUCCESS_RATE_LOCAL_ORIGIN":        4,
		"FAILURE_PERCENTAGE":               5,
		"FAILURE_PERCENTAGE_LOCAL_ORIGIN":  6,
		"DEGRADED":                         7,
	}
)

func (x OutlierEjectionType) Enum() *OutlierEjectionType {
	p := new(OutlierEjectionType)
	*p = x
	return p
}

func (x OutlierEjectionType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (OutlierEjectionType) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_enumTypes[0].Descriptor()
}

func (OutlierEjectionType) Type() protoreflect.EnumType {
	return &file_envoy_data_cluster_v3_outlier_detection_event_proto_enumTypes[0]
}

func (x OutlierEjectionType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use OutlierEjectionType.Descriptor instead.
func (OutlierEjectionType) EnumDescriptor() ([]byte, []int) {
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescGZIP(), []int{0}
}

// Represents possible action applied to upstream host
type Action int32

const (
	// In case host was excluded from service
	Action_EJECT Action = 0
	// In case host was brought back into service
	Action_UNEJECT Action = 1
)

// Enum value maps for Action.
var (
	Action_name = map[int32]string{
		0: "EJECT",
		1: "UNEJECT",
	}
	Action_value = map[string]int32{
		"EJECT":   0,
		"UNEJECT": 1,
	}
)

func (x Action) Enum() *Action {
	p := new(Action)
	*p = x
	return p
}

func (x Action) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Action) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_enumTypes[1].Descriptor()
}

func (Action) Type() protoreflect.EnumType {
	return &file_envoy_data_cluster_v3_outlier_detection_event_proto_enumTypes[1]
}

func (x Action) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use Action.Descriptor instead.
func (Action) EnumDescriptor() ([]byte, []int) {
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescGZIP(), []int{1}
}

// [#next-free-field: 12]
type OutlierDetectionEvent struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// In case of eject represents type of ejection that took place.
	Type OutlierEjectionType `protobuf:"varint,1,opt,name=type,proto3,enum=envoy.data.cluster.v3.OutlierEjectionType" json:"type,omitempty"`
	// Timestamp for event.
	Timestamp *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=timestamp,proto3" json:"timestamp,omitempty"`
	// The time in seconds since the last action (either an ejection or unejection) took place.
	SecsSinceLastAction *wrapperspb.UInt64Value `protobuf:"bytes,3,opt,name=secs_since_last_action,json=secsSinceLastAction,proto3" json:"secs_since_last_action,omitempty"`
	// The :ref:`cluster <envoy_v3_api_msg_config.cluster.v3.Cluster>` that owns the ejected host.
	ClusterName string `protobuf:"bytes,4,opt,name=cluster_name,json=clusterName,proto3" json:"cluster_name,omitempty"`
	// The URL of the ejected host. E.g., “tcp://1.2.3.4:80“.
	UpstreamUrl string `protobuf:"bytes,5,opt,name=upstream_url,json=upstreamUrl,proto3" json:"upstream_url,omitempty"`
	// The action that took place.
	Action Action `protobuf:"varint,6,opt,name=action,proto3,enum=envoy.data.cluster.v3.Action" json:"action,omitempty"`
	// If “action“ is “eject“, specifies the number of times the host has been ejected (local to
	// that Envoy and gets reset if the host gets removed from the upstream cluster for any reason and
	// then re-added).
	NumEjections uint32 `protobuf:"varint,7,opt,name=num_ejections,json=numEjections,proto3" json:"num_ejections,omitempty"`
	// If “action“ is “eject“, specifies if the ejection was enforced. “true“ means the host was
	// ejected. “false“ means the event was logged but the host was not actually ejected.
	Enforced bool `protobuf:"varint,8,opt,name=enforced,proto3" json:"enforced,omitempty"`
	// Types that are valid to be assigned to Event:
	//
	//	*OutlierDetectionEvent_EjectSuccessRateEvent
	//	*OutlierDetectionEvent_EjectConsecutiveEvent
	//	*OutlierDetectionEvent_EjectFailurePercentageEvent
	Event         isOutlierDetectionEvent_Event `protobuf_oneof:"event"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OutlierDetectionEvent) Reset() {
	*x = OutlierDetectionEvent{}
	mi := &file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OutlierDetectionEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OutlierDetectionEvent) ProtoMessage() {}

func (x *OutlierDetectionEvent) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OutlierDetectionEvent.ProtoReflect.Descriptor instead.
func (*OutlierDetectionEvent) Descriptor() ([]byte, []int) {
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescGZIP(), []int{0}
}

func (x *OutlierDetectionEvent) GetType() OutlierEjectionType {
	if x != nil {
		return x.Type
	}
	return OutlierEjectionType_CONSECUTIVE_5XX
}

func (x *OutlierDetectionEvent) GetTimestamp() *timestamppb.Timestamp {
	if x != nil {
		return x.Timestamp
	}
	return nil
}

func (x *OutlierDetectionEvent) GetSecsSinceLastAction() *wrapperspb.UInt64Value {
	if x != nil {
		return x.SecsSinceLastAction
	}
	return nil
}

func (x *OutlierDetectionEvent) GetClusterName() string {
	if x != nil {
		return x.ClusterName
	}
	return ""
}

func (x *OutlierDetectionEvent) GetUpstreamUrl() string {
	if x != nil {
		return x.UpstreamUrl
	}
	return ""
}

func (x *OutlierDetectionEvent) GetAction() Action {
	if x != nil {
		return x.Action
	}
	return Action_EJECT
}

func (x *OutlierDetectionEvent) GetNumEjections() uint32 {
	if x != nil {
		return x.NumEjections
	}
	return 0
}

func (x *OutlierDetectionEvent) GetEnforced() bool {
	if x != nil {
		return x.Enforced
	}
	return false
}

func (x *OutlierDetectionEvent) GetEvent() isOutlierDetectionEvent_Event {
	if x != nil {
		return x.Event
	}
	return nil
}

func (x *OutlierDetectionEvent) GetEjectSuccessRateEvent() *OutlierEjectSuccessRate {
	if x != nil {
		if x, ok := x.Event.(*OutlierDetectionEvent_EjectSuccessRateEvent); ok {
			return x.EjectSuccessRateEvent
		}
	}
	return nil
}

func (x *OutlierDetectionEvent) GetEjectConsecutiveEvent() *OutlierEjectConsecutive {
	if x != nil {
		if x, ok := x.Event.(*OutlierDetectionEvent_EjectConsecutiveEvent); ok {
			return x.EjectConsecutiveEvent
		}
	}
	return nil
}

func (x *OutlierDetectionEvent) GetEjectFailurePercentageEvent() *OutlierEjectFailurePercentage {
	if x != nil {
		if x, ok := x.Event.(*OutlierDetectionEvent_EjectFailurePercentageEvent); ok {
			return x.EjectFailurePercentageEvent
		}
	}
	return nil
}

type isOutlierDetectionEvent_Event interface {
	isOutlierDetectionEvent_Event()
}

type OutlierDetectionEvent_EjectSuccessRateEvent struct {
	EjectSuccessRateEvent *OutlierEjectSuccessRate `protobuf:"bytes,9,opt,name=eject_success_rate_event,json=ejectSuccessRateEvent,proto3,oneof"`
}

type OutlierDetectionEvent_EjectConsecutiveEvent struct {
	EjectConsecutiveEvent *OutlierEjectConsecutive `protobuf:"bytes,10,opt,name=eject_consecutive_event,json=ejectConsecutiveEvent,proto3,oneof"`
}

type OutlierDetectionEvent_EjectFailurePercentageEvent struct {
	EjectFailurePercentageEvent *OutlierEjectFailurePercentage `protobuf:"bytes,11,opt,name=eject_failure_percentage_event,json=ejectFailurePercentageEvent,proto3,oneof"`
}

func (*OutlierDetectionEvent_EjectSuccessRateEvent) isOutlierDetectionEvent_Event() {}

func (*OutlierDetectionEvent_EjectConsecutiveEvent) isOutlierDetectionEvent_Event() {}

func (*OutlierDetectionEvent_EjectFailurePercentageEvent) isOutlierDetectionEvent_Event() {}

type OutlierEjectSuccessRate struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Host’s success rate at the time of the ejection event on a 0-100 range.
	HostSuccessRate uint32 `protobuf:"varint,1,opt,name=host_success_rate,json=hostSuccessRate,proto3" json:"host_success_rate,omitempty"`
	// Average success rate of the hosts in the cluster at the time of the ejection event on a 0-100
	// range.
	ClusterAverageSuccessRate uint32 `protobuf:"varint,2,opt,name=cluster_average_success_rate,json=clusterAverageSuccessRate,proto3" json:"cluster_average_success_rate,omitempty"`
	// Success rate ejection threshold at the time of the ejection event.
	ClusterSuccessRateEjectionThreshold uint32 `protobuf:"varint,3,opt,name=cluster_success_rate_ejection_threshold,json=clusterSuccessRateEjectionThreshold,proto3" json:"cluster_success_rate_ejection_threshold,omitempty"`
	unknownFields                       protoimpl.UnknownFields
	sizeCache                           protoimpl.SizeCache
}

func (x *OutlierEjectSuccessRate) Reset() {
	*x = OutlierEjectSuccessRate{}
	mi := &file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OutlierEjectSuccessRate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OutlierEjectSuccessRate) ProtoMessage() {}

func (x *OutlierEjectSuccessRate) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OutlierEjectSuccessRate.ProtoReflect.Descriptor instead.
func (*OutlierEjectSuccessRate) Descriptor() ([]byte, []int) {
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescGZIP(), []int{1}
}

func (x *OutlierEjectSuccessRate) GetHostSuccessRate() uint32 {
	if x != nil {
		return x.HostSuccessRate
	}
	return 0
}

func (x *OutlierEjectSuccessRate) GetClusterAverageSuccessRate() uint32 {
	if x != nil {
		return x.ClusterAverageSuccessRate
	}
	return 0
}

func (x *OutlierEjectSuccessRate) GetClusterSuccessRateEjectionThreshold() uint32 {
	if x != nil {
		return x.ClusterSuccessRateEjectionThreshold
	}
	return 0
}

type OutlierEjectConsecutive struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OutlierEjectConsecutive) Reset() {
	*x = OutlierEjectConsecutive{}
	mi := &file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OutlierEjectConsecutive) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OutlierEjectConsecutive) ProtoMessage() {}

func (x *OutlierEjectConsecutive) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OutlierEjectConsecutive.ProtoReflect.Descriptor instead.
func (*OutlierEjectConsecutive) Descriptor() ([]byte, []int) {
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescGZIP(), []int{2}
}

type OutlierEjectFailurePercentage struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Host's success rate at the time of the ejection event on a 0-100 range.
	HostSuccessRate uint32 `protobuf:"varint,1,opt,name=host_success_rate,json=hostSuccessRate,proto3" json:"host_success_rate,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *OutlierEjectFailurePercentage) Reset() {
	*x = OutlierEjectFailurePercentage{}
	mi := &file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OutlierEjectFailurePercentage) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OutlierEjectFailurePercentage) ProtoMessage() {}

func (x *OutlierEjectFailurePercentage) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OutlierEjectFailurePercentage.ProtoReflect.Descriptor instead.
func (*OutlierEjectFailurePercentage) Descriptor() ([]byte, []int) {
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescGZIP(), []int{3}
}

func (x *OutlierEjectFailurePercentage) GetHostSuccessRate() uint32 {
	if x != nil {
		return x.HostSuccessRate
	}
	return 0
}

var File_envoy_data_cluster_v3_outlier_detection_event_proto protoreflect.FileDescriptor

const file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDesc = "" +
	"\n" +
	"3envoy/data/cluster/v3/outlier_detection_event.proto\x12\x15envoy.data.cluster.v3\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xe1\x06\n" +
	"\x15OutlierDetectionEvent\x12H\n" +
	"\x04type\x18\x01 \x01(\x0e2*.envoy.data.cluster.v3.OutlierEjectionTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\x04type\x128\n" +
	"\ttimestamp\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\ttimestamp\x12Q\n" +
	"\x16secs_since_last_action\x18\x03 \x01(\v2\x1c.google.protobuf.UInt64ValueR\x13secsSinceLastAction\x12*\n" +
	"\fcluster_name\x18\x04 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vclusterName\x12*\n" +
	"\fupstream_url\x18\x05 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vupstreamUrl\x12?\n" +
	"\x06action\x18\x06 \x01(\x0e2\x1d.envoy.data.cluster.v3.ActionB\b\xfaB\x05\x82\x01\x02\x10\x01R\x06action\x12#\n" +
	"\rnum_ejections\x18\a \x01(\rR\fnumEjections\x12\x1a\n" +
	"\benforced\x18\b \x01(\bR\benforced\x12i\n" +
	"\x18eject_success_rate_event\x18\t \x01(\v2..envoy.data.cluster.v3.OutlierEjectSuccessRateH\x00R\x15ejectSuccessRateEvent\x12h\n" +
	"\x17eject_consecutive_event\x18\n" +
	" \x01(\v2..envoy.data.cluster.v3.OutlierEjectConsecutiveH\x00R\x15ejectConsecutiveEvent\x12{\n" +
	"\x1eeject_failure_percentage_event\x18\v \x01(\v24.envoy.data.cluster.v3.OutlierEjectFailurePercentageH\x00R\x1bejectFailurePercentageEvent:7\x9aň\x1e2\n" +
	"0envoy.data.cluster.v2alpha.OutlierDetectionEventB\f\n" +
	"\x05event\x12\x03\xf8B\x01\"\xb2\x02\n" +
	"\x17OutlierEjectSuccessRate\x123\n" +
	"\x11host_success_rate\x18\x01 \x01(\rB\a\xfaB\x04*\x02\x18dR\x0fhostSuccessRate\x12H\n" +
	"\x1ccluster_average_success_rate\x18\x02 \x01(\rB\a\xfaB\x04*\x02\x18dR\x19clusterAverageSuccessRate\x12]\n" +
	"'cluster_success_rate_ejection_threshold\x18\x03 \x01(\rB\a\xfaB\x04*\x02\x18dR#clusterSuccessRateEjectionThreshold:9\x9aň\x1e4\n" +
	"2envoy.data.cluster.v2alpha.OutlierEjectSuccessRate\"T\n" +
	"\x17OutlierEjectConsecutive:9\x9aň\x1e4\n" +
	"2envoy.data.cluster.v2alpha.OutlierEjectConsecutive\"\x95\x01\n" +
	"\x1dOutlierEjectFailurePercentage\x123\n" +
	"\x11host_success_rate\x18\x01 \x01(\rB\a\xfaB\x04*\x02\x18dR\x0fhostSuccessRate:?\x9aň\x1e:\n" +
	"8envoy.data.cluster.v2alpha.OutlierEjectFailurePercentage*\xed\x01\n" +
	"\x13OutlierEjectionType\x12\x13\n" +
	"\x0fCONSECUTIVE_5XX\x10\x00\x12\x1f\n" +
	"\x1bCONSECUTIVE_GATEWAY_FAILURE\x10\x01\x12\x10\n" +
	"\fSUCCESS_RATE\x10\x02\x12$\n" +
	" CONSECUTIVE_LOCAL_ORIGIN_FAILURE\x10\x03\x12\x1d\n" +
	"\x19SUCCESS_RATE_LOCAL_ORIGIN\x10\x04\x12\x16\n" +
	"\x12FAILURE_PERCENTAGE\x10\x05\x12#\n" +
	"\x1fFAILURE_PERCENTAGE_LOCAL_ORIGIN\x10\x06\x12\f\n" +
	"\bDEGRADED\x10\a* \n" +
	"\x06Action\x12\t\n" +
	"\x05EJECT\x10\x00\x12\v\n" +
	"\aUNEJECT\x10\x01B\x93\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.data.cluster.v3B\x1aOutlierDetectionEventProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/data/cluster/v3;clusterv3b\x06proto3"

var (
	file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescOnce sync.Once
	file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescData []byte
)

func file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescGZIP() []byte {
	file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescOnce.Do(func() {
		file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDesc), len(file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDesc)))
	})
	return file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDescData
}

var file_envoy_data_cluster_v3_outlier_detection_event_proto_enumTypes = make([]protoimpl.EnumInfo, 2)
var file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_envoy_data_cluster_v3_outlier_detection_event_proto_goTypes = []any{
	(OutlierEjectionType)(0),              // 0: envoy.data.cluster.v3.OutlierEjectionType
	(Action)(0),                           // 1: envoy.data.cluster.v3.Action
	(*OutlierDetectionEvent)(nil),         // 2: envoy.data.cluster.v3.OutlierDetectionEvent
	(*OutlierEjectSuccessRate)(nil),       // 3: envoy.data.cluster.v3.OutlierEjectSuccessRate
	(*OutlierEjectConsecutive)(nil),       // 4: envoy.data.cluster.v3.OutlierEjectConsecutive
	(*OutlierEjectFailurePercentage)(nil), // 5: envoy.data.cluster.v3.OutlierEjectFailurePercentage
	(*timestamppb.Timestamp)(nil),         // 6: google.protobuf.Timestamp
	(*wrapperspb.UInt64Value)(nil),        // 7: google.protobuf.UInt64Value
}
var file_envoy_data_cluster_v3_outlier_detection_event_proto_depIdxs = []int32{
	0, // 0: envoy.data.cluster.v3.OutlierDetectionEvent.type:type_name -> envoy.data.cluster.v3.OutlierEjectionType
	6, // 1: envoy.data.cluster.v3.OutlierDetectionEvent.timestamp:type_name -> google.protobuf.Timestamp
	7, // 2: envoy.data.cluster.v3.OutlierDetectionEvent.secs_since_last_action:type_name -> google.protobuf.UInt64Value
	1, // 3: envoy.data.cluster.v3.OutlierDetectionEvent.action:type_name -> envoy.data.cluster.v3.Action
	3, // 4: envoy.data.cluster.v3.OutlierDetectionEvent.eject_success_rate_event:type_name -> envoy.data.cluster.v3.OutlierEjectSuccessRate
	4, // 5: envoy.data.cluster.v3.OutlierDetectionEvent.eject_consecutive_event:type_name -> envoy.data.cluster.v3.OutlierEjectConsecutive
	5, // 6: envoy.data.cluster.v3.OutlierDetectionEvent.eject_failure_percentage_event:type_name -> envoy.data.cluster.v3.OutlierEjectFailurePercentage
	7, // [7:7] is the sub-list for method output_type
	7, // [7:7] is the sub-list for method input_type
	7, // [7:7] is the sub-list for extension type_name
	7, // [7:7] is the sub-list for extension extendee
	0, // [0:7] is the sub-list for field type_name
}

func init() { file_envoy_data_cluster_v3_outlier_detection_event_proto_init() }
func file_envoy_data_cluster_v3_outlier_detection_event_proto_init() {
	if File_envoy_data_cluster_v3_outlier_detection_event_proto != nil {
		return
	}
	file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes[0].OneofWrappers = []any{
		(*OutlierDetectionEvent_EjectSuccessRateEvent)(nil),
		(*OutlierDetectionEvent_EjectConsecutiveEvent)(nil),
		(*OutlierDetectionEvent_EjectFailurePercentageEvent)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDesc), len(file_envoy_data_cluster_v3_outlier_detection_event_proto_rawDesc)),
			NumEnums:      2,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_data_cluster_v3_outlier_detection_event_proto_goTypes,
		DependencyIndexes: file_envoy_data_cluster_v3_outlier_detection_event_proto_depIdxs,
		EnumInfos:         file_envoy_data_cluster_v3_outlier_detection_event_proto_enumTypes,
		MessageInfos:      file_envoy_data_cluster_v3_outlier_detection_event_proto_msgTypes,
	}.Build()
	File_envoy_data_cluster_v3_outlier_detection_event_proto = out.File
	file_envoy_data_cluster_v3_outlier_detection_event_proto_goTypes = nil
	file_envoy_data_cluster_v3_outlier_detection_event_proto_depIdxs = nil
}
