// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/data/tap/v3/http.proto

package tapv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// A fully buffered HTTP trace message.
type HttpBufferedTrace struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Request message.
	Request *HttpBufferedTrace_Message `protobuf:"bytes,1,opt,name=request,proto3" json:"request,omitempty"`
	// Response message.
	Response *HttpBufferedTrace_Message `protobuf:"bytes,2,opt,name=response,proto3" json:"response,omitempty"`
	// downstream connection
	DownstreamConnection *Connection `protobuf:"bytes,3,opt,name=downstream_connection,json=downstreamConnection,proto3" json:"downstream_connection,omitempty"`
	// upstream connection
	UpstreamConnection *Connection `protobuf:"bytes,4,opt,name=upstream_connection,json=upstreamConnection,proto3" json:"upstream_connection,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *HttpBufferedTrace) Reset() {
	*x = HttpBufferedTrace{}
	mi := &file_envoy_data_tap_v3_http_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpBufferedTrace) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpBufferedTrace) ProtoMessage() {}

func (x *HttpBufferedTrace) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_tap_v3_http_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpBufferedTrace.ProtoReflect.Descriptor instead.
func (*HttpBufferedTrace) Descriptor() ([]byte, []int) {
	return file_envoy_data_tap_v3_http_proto_rawDescGZIP(), []int{0}
}

func (x *HttpBufferedTrace) GetRequest() *HttpBufferedTrace_Message {
	if x != nil {
		return x.Request
	}
	return nil
}

func (x *HttpBufferedTrace) GetResponse() *HttpBufferedTrace_Message {
	if x != nil {
		return x.Response
	}
	return nil
}

func (x *HttpBufferedTrace) GetDownstreamConnection() *Connection {
	if x != nil {
		return x.DownstreamConnection
	}
	return nil
}

func (x *HttpBufferedTrace) GetUpstreamConnection() *Connection {
	if x != nil {
		return x.UpstreamConnection
	}
	return nil
}

// A streamed HTTP trace segment. Multiple segments make up a full trace.
// [#next-free-field: 8]
type HttpStreamedTraceSegment struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Trace ID unique to the originating Envoy only. Trace IDs can repeat and should not be used
	// for long term stable uniqueness.
	TraceId uint64 `protobuf:"varint,1,opt,name=trace_id,json=traceId,proto3" json:"trace_id,omitempty"`
	// Types that are valid to be assigned to MessagePiece:
	//
	//	*HttpStreamedTraceSegment_RequestHeaders
	//	*HttpStreamedTraceSegment_RequestBodyChunk
	//	*HttpStreamedTraceSegment_RequestTrailers
	//	*HttpStreamedTraceSegment_ResponseHeaders
	//	*HttpStreamedTraceSegment_ResponseBodyChunk
	//	*HttpStreamedTraceSegment_ResponseTrailers
	MessagePiece  isHttpStreamedTraceSegment_MessagePiece `protobuf_oneof:"message_piece"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpStreamedTraceSegment) Reset() {
	*x = HttpStreamedTraceSegment{}
	mi := &file_envoy_data_tap_v3_http_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpStreamedTraceSegment) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpStreamedTraceSegment) ProtoMessage() {}

func (x *HttpStreamedTraceSegment) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_tap_v3_http_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpStreamedTraceSegment.ProtoReflect.Descriptor instead.
func (*HttpStreamedTraceSegment) Descriptor() ([]byte, []int) {
	return file_envoy_data_tap_v3_http_proto_rawDescGZIP(), []int{1}
}

func (x *HttpStreamedTraceSegment) GetTraceId() uint64 {
	if x != nil {
		return x.TraceId
	}
	return 0
}

func (x *HttpStreamedTraceSegment) GetMessagePiece() isHttpStreamedTraceSegment_MessagePiece {
	if x != nil {
		return x.MessagePiece
	}
	return nil
}

func (x *HttpStreamedTraceSegment) GetRequestHeaders() *v3.HeaderMap {
	if x != nil {
		if x, ok := x.MessagePiece.(*HttpStreamedTraceSegment_RequestHeaders); ok {
			return x.RequestHeaders
		}
	}
	return nil
}

func (x *HttpStreamedTraceSegment) GetRequestBodyChunk() *Body {
	if x != nil {
		if x, ok := x.MessagePiece.(*HttpStreamedTraceSegment_RequestBodyChunk); ok {
			return x.RequestBodyChunk
		}
	}
	return nil
}

func (x *HttpStreamedTraceSegment) GetRequestTrailers() *v3.HeaderMap {
	if x != nil {
		if x, ok := x.MessagePiece.(*HttpStreamedTraceSegment_RequestTrailers); ok {
			return x.RequestTrailers
		}
	}
	return nil
}

func (x *HttpStreamedTraceSegment) GetResponseHeaders() *v3.HeaderMap {
	if x != nil {
		if x, ok := x.MessagePiece.(*HttpStreamedTraceSegment_ResponseHeaders); ok {
			return x.ResponseHeaders
		}
	}
	return nil
}

func (x *HttpStreamedTraceSegment) GetResponseBodyChunk() *Body {
	if x != nil {
		if x, ok := x.MessagePiece.(*HttpStreamedTraceSegment_ResponseBodyChunk); ok {
			return x.ResponseBodyChunk
		}
	}
	return nil
}

func (x *HttpStreamedTraceSegment) GetResponseTrailers() *v3.HeaderMap {
	if x != nil {
		if x, ok := x.MessagePiece.(*HttpStreamedTraceSegment_ResponseTrailers); ok {
			return x.ResponseTrailers
		}
	}
	return nil
}

type isHttpStreamedTraceSegment_MessagePiece interface {
	isHttpStreamedTraceSegment_MessagePiece()
}

type HttpStreamedTraceSegment_RequestHeaders struct {
	// Request headers.
	RequestHeaders *v3.HeaderMap `protobuf:"bytes,2,opt,name=request_headers,json=requestHeaders,proto3,oneof"`
}

type HttpStreamedTraceSegment_RequestBodyChunk struct {
	// Request body chunk.
	RequestBodyChunk *Body `protobuf:"bytes,3,opt,name=request_body_chunk,json=requestBodyChunk,proto3,oneof"`
}

type HttpStreamedTraceSegment_RequestTrailers struct {
	// Request trailers.
	RequestTrailers *v3.HeaderMap `protobuf:"bytes,4,opt,name=request_trailers,json=requestTrailers,proto3,oneof"`
}

type HttpStreamedTraceSegment_ResponseHeaders struct {
	// Response headers.
	ResponseHeaders *v3.HeaderMap `protobuf:"bytes,5,opt,name=response_headers,json=responseHeaders,proto3,oneof"`
}

type HttpStreamedTraceSegment_ResponseBodyChunk struct {
	// Response body chunk.
	ResponseBodyChunk *Body `protobuf:"bytes,6,opt,name=response_body_chunk,json=responseBodyChunk,proto3,oneof"`
}

type HttpStreamedTraceSegment_ResponseTrailers struct {
	// Response trailers.
	ResponseTrailers *v3.HeaderMap `protobuf:"bytes,7,opt,name=response_trailers,json=responseTrailers,proto3,oneof"`
}

func (*HttpStreamedTraceSegment_RequestHeaders) isHttpStreamedTraceSegment_MessagePiece() {}

func (*HttpStreamedTraceSegment_RequestBodyChunk) isHttpStreamedTraceSegment_MessagePiece() {}

func (*HttpStreamedTraceSegment_RequestTrailers) isHttpStreamedTraceSegment_MessagePiece() {}

func (*HttpStreamedTraceSegment_ResponseHeaders) isHttpStreamedTraceSegment_MessagePiece() {}

func (*HttpStreamedTraceSegment_ResponseBodyChunk) isHttpStreamedTraceSegment_MessagePiece() {}

func (*HttpStreamedTraceSegment_ResponseTrailers) isHttpStreamedTraceSegment_MessagePiece() {}

// HTTP message wrapper.
type HttpBufferedTrace_Message struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Message headers.
	Headers []*v3.HeaderValue `protobuf:"bytes,1,rep,name=headers,proto3" json:"headers,omitempty"`
	// Message body.
	Body *Body `protobuf:"bytes,2,opt,name=body,proto3" json:"body,omitempty"`
	// Message trailers.
	Trailers []*v3.HeaderValue `protobuf:"bytes,3,rep,name=trailers,proto3" json:"trailers,omitempty"`
	// The timestamp after receiving the message headers.
	HeadersReceivedTime *timestamppb.Timestamp `protobuf:"bytes,4,opt,name=headers_received_time,json=headersReceivedTime,proto3" json:"headers_received_time,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *HttpBufferedTrace_Message) Reset() {
	*x = HttpBufferedTrace_Message{}
	mi := &file_envoy_data_tap_v3_http_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpBufferedTrace_Message) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpBufferedTrace_Message) ProtoMessage() {}

func (x *HttpBufferedTrace_Message) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_data_tap_v3_http_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpBufferedTrace_Message.ProtoReflect.Descriptor instead.
func (*HttpBufferedTrace_Message) Descriptor() ([]byte, []int) {
	return file_envoy_data_tap_v3_http_proto_rawDescGZIP(), []int{0, 0}
}

func (x *HttpBufferedTrace_Message) GetHeaders() []*v3.HeaderValue {
	if x != nil {
		return x.Headers
	}
	return nil
}

func (x *HttpBufferedTrace_Message) GetBody() *Body {
	if x != nil {
		return x.Body
	}
	return nil
}

func (x *HttpBufferedTrace_Message) GetTrailers() []*v3.HeaderValue {
	if x != nil {
		return x.Trailers
	}
	return nil
}

func (x *HttpBufferedTrace_Message) GetHeadersReceivedTime() *timestamppb.Timestamp {
	if x != nil {
		return x.HeadersReceivedTime
	}
	return nil
}

var File_envoy_data_tap_v3_http_proto protoreflect.FileDescriptor

const file_envoy_data_tap_v3_http_proto_rawDesc = "" +
	"\n" +
	"\x1cenvoy/data/tap/v3/http.proto\x12\x11envoy.data.tap.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a\x1eenvoy/data/tap/v3/common.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xb8\x05\n" +
	"\x11HttpBufferedTrace\x12F\n" +
	"\arequest\x18\x01 \x01(\v2,.envoy.data.tap.v3.HttpBufferedTrace.MessageR\arequest\x12H\n" +
	"\bresponse\x18\x02 \x01(\v2,.envoy.data.tap.v3.HttpBufferedTrace.MessageR\bresponse\x12R\n" +
	"\x15downstream_connection\x18\x03 \x01(\v2\x1d.envoy.data.tap.v3.ConnectionR\x14downstreamConnection\x12N\n" +
	"\x13upstream_connection\x18\x04 \x01(\v2\x1d.envoy.data.tap.v3.ConnectionR\x12upstreamConnection\x1a\xbb\x02\n" +
	"\aMessage\x12;\n" +
	"\aheaders\x18\x01 \x03(\v2!.envoy.config.core.v3.HeaderValueR\aheaders\x12+\n" +
	"\x04body\x18\x02 \x01(\v2\x17.envoy.data.tap.v3.BodyR\x04body\x12=\n" +
	"\btrailers\x18\x03 \x03(\v2!.envoy.config.core.v3.HeaderValueR\btrailers\x12N\n" +
	"\x15headers_received_time\x18\x04 \x01(\v2\x1a.google.protobuf.TimestampR\x13headersReceivedTime:7\x9aň\x1e2\n" +
	"0envoy.data.tap.v2alpha.HttpBufferedTrace.Message:/\x9aň\x1e*\n" +
	"(envoy.data.tap.v2alpha.HttpBufferedTrace\"\xca\x04\n" +
	"\x18HttpStreamedTraceSegment\x12\x19\n" +
	"\btrace_id\x18\x01 \x01(\x04R\atraceId\x12J\n" +
	"\x0frequest_headers\x18\x02 \x01(\v2\x1f.envoy.config.core.v3.HeaderMapH\x00R\x0erequestHeaders\x12G\n" +
	"\x12request_body_chunk\x18\x03 \x01(\v2\x17.envoy.data.tap.v3.BodyH\x00R\x10requestBodyChunk\x12L\n" +
	"\x10request_trailers\x18\x04 \x01(\v2\x1f.envoy.config.core.v3.HeaderMapH\x00R\x0frequestTrailers\x12L\n" +
	"\x10response_headers\x18\x05 \x01(\v2\x1f.envoy.config.core.v3.HeaderMapH\x00R\x0fresponseHeaders\x12I\n" +
	"\x13response_body_chunk\x18\x06 \x01(\v2\x17.envoy.data.tap.v3.BodyH\x00R\x11responseBodyChunk\x12N\n" +
	"\x11response_trailers\x18\a \x01(\v2\x1f.envoy.config.core.v3.HeaderMapH\x00R\x10responseTrailers:6\x9aň\x1e1\n" +
	"/envoy.data.tap.v2alpha.HttpStreamedTraceSegmentB\x0f\n" +
	"\rmessage_pieceBv\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\x1fio.envoyproxy.envoy.data.tap.v3B\tHttpProtoP\x01Z>github.com/envoyproxy/go-control-plane/envoy/data/tap/v3;tapv3b\x06proto3"

var (
	file_envoy_data_tap_v3_http_proto_rawDescOnce sync.Once
	file_envoy_data_tap_v3_http_proto_rawDescData []byte
)

func file_envoy_data_tap_v3_http_proto_rawDescGZIP() []byte {
	file_envoy_data_tap_v3_http_proto_rawDescOnce.Do(func() {
		file_envoy_data_tap_v3_http_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_data_tap_v3_http_proto_rawDesc), len(file_envoy_data_tap_v3_http_proto_rawDesc)))
	})
	return file_envoy_data_tap_v3_http_proto_rawDescData
}

var file_envoy_data_tap_v3_http_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_envoy_data_tap_v3_http_proto_goTypes = []any{
	(*HttpBufferedTrace)(nil),         // 0: envoy.data.tap.v3.HttpBufferedTrace
	(*HttpStreamedTraceSegment)(nil),  // 1: envoy.data.tap.v3.HttpStreamedTraceSegment
	(*HttpBufferedTrace_Message)(nil), // 2: envoy.data.tap.v3.HttpBufferedTrace.Message
	(*Connection)(nil),                // 3: envoy.data.tap.v3.Connection
	(*v3.HeaderMap)(nil),              // 4: envoy.config.core.v3.HeaderMap
	(*Body)(nil),                      // 5: envoy.data.tap.v3.Body
	(*v3.HeaderValue)(nil),            // 6: envoy.config.core.v3.HeaderValue
	(*timestamppb.Timestamp)(nil),     // 7: google.protobuf.Timestamp
}
var file_envoy_data_tap_v3_http_proto_depIdxs = []int32{
	2,  // 0: envoy.data.tap.v3.HttpBufferedTrace.request:type_name -> envoy.data.tap.v3.HttpBufferedTrace.Message
	2,  // 1: envoy.data.tap.v3.HttpBufferedTrace.response:type_name -> envoy.data.tap.v3.HttpBufferedTrace.Message
	3,  // 2: envoy.data.tap.v3.HttpBufferedTrace.downstream_connection:type_name -> envoy.data.tap.v3.Connection
	3,  // 3: envoy.data.tap.v3.HttpBufferedTrace.upstream_connection:type_name -> envoy.data.tap.v3.Connection
	4,  // 4: envoy.data.tap.v3.HttpStreamedTraceSegment.request_headers:type_name -> envoy.config.core.v3.HeaderMap
	5,  // 5: envoy.data.tap.v3.HttpStreamedTraceSegment.request_body_chunk:type_name -> envoy.data.tap.v3.Body
	4,  // 6: envoy.data.tap.v3.HttpStreamedTraceSegment.request_trailers:type_name -> envoy.config.core.v3.HeaderMap
	4,  // 7: envoy.data.tap.v3.HttpStreamedTraceSegment.response_headers:type_name -> envoy.config.core.v3.HeaderMap
	5,  // 8: envoy.data.tap.v3.HttpStreamedTraceSegment.response_body_chunk:type_name -> envoy.data.tap.v3.Body
	4,  // 9: envoy.data.tap.v3.HttpStreamedTraceSegment.response_trailers:type_name -> envoy.config.core.v3.HeaderMap
	6,  // 10: envoy.data.tap.v3.HttpBufferedTrace.Message.headers:type_name -> envoy.config.core.v3.HeaderValue
	5,  // 11: envoy.data.tap.v3.HttpBufferedTrace.Message.body:type_name -> envoy.data.tap.v3.Body
	6,  // 12: envoy.data.tap.v3.HttpBufferedTrace.Message.trailers:type_name -> envoy.config.core.v3.HeaderValue
	7,  // 13: envoy.data.tap.v3.HttpBufferedTrace.Message.headers_received_time:type_name -> google.protobuf.Timestamp
	14, // [14:14] is the sub-list for method output_type
	14, // [14:14] is the sub-list for method input_type
	14, // [14:14] is the sub-list for extension type_name
	14, // [14:14] is the sub-list for extension extendee
	0,  // [0:14] is the sub-list for field type_name
}

func init() { file_envoy_data_tap_v3_http_proto_init() }
func file_envoy_data_tap_v3_http_proto_init() {
	if File_envoy_data_tap_v3_http_proto != nil {
		return
	}
	file_envoy_data_tap_v3_common_proto_init()
	file_envoy_data_tap_v3_http_proto_msgTypes[1].OneofWrappers = []any{
		(*HttpStreamedTraceSegment_RequestHeaders)(nil),
		(*HttpStreamedTraceSegment_RequestBodyChunk)(nil),
		(*HttpStreamedTraceSegment_RequestTrailers)(nil),
		(*HttpStreamedTraceSegment_ResponseHeaders)(nil),
		(*HttpStreamedTraceSegment_ResponseBodyChunk)(nil),
		(*HttpStreamedTraceSegment_ResponseTrailers)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_data_tap_v3_http_proto_rawDesc), len(file_envoy_data_tap_v3_http_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_data_tap_v3_http_proto_goTypes,
		DependencyIndexes: file_envoy_data_tap_v3_http_proto_depIdxs,
		MessageInfos:      file_envoy_data_tap_v3_http_proto_msgTypes,
	}.Build()
	File_envoy_data_tap_v3_http_proto = out.File
	file_envoy_data_tap_v3_http_proto_goTypes = nil
	file_envoy_data_tap_v3_http_proto_depIdxs = nil
}
