// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/set_metadata/v3/set_metadata.proto

package set_metadatav3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	structpb "google.golang.org/protobuf/types/known/structpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Metadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The metadata namespace.
	MetadataNamespace string `protobuf:"bytes,1,opt,name=metadata_namespace,json=metadataNamespace,proto3" json:"metadata_namespace,omitempty"`
	// Allow the filter to overwrite or merge with an existing value in the namespace.
	AllowOverwrite bool `protobuf:"varint,2,opt,name=allow_overwrite,json=allowOverwrite,proto3" json:"allow_overwrite,omitempty"`
	// The value to place at the namespace. If “allow_overwrite“, this will
	// overwrite or merge with any existing values in that namespace. See
	// :ref:`the filter documentation <config_http_filters_set_metadata>` for
	// more information on how this value is merged with potentially existing
	// ones if “allow_overwrite“ is configured. Only one of “value“ and
	// “typed_value“ may be set.
	Value *structpb.Struct `protobuf:"bytes,3,opt,name=value,proto3" json:"value,omitempty"`
	// The value to place at the namespace. If “allow_overwrite“, this will
	// overwrite any existing values in that namespace. Only one of “value“ and
	// “typed_value“ may be set.
	TypedValue    *anypb.Any `protobuf:"bytes,4,opt,name=typed_value,json=typedValue,proto3" json:"typed_value,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Metadata) Reset() {
	*x = Metadata{}
	mi := &file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Metadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Metadata) ProtoMessage() {}

func (x *Metadata) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Metadata.ProtoReflect.Descriptor instead.
func (*Metadata) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDescGZIP(), []int{0}
}

func (x *Metadata) GetMetadataNamespace() string {
	if x != nil {
		return x.MetadataNamespace
	}
	return ""
}

func (x *Metadata) GetAllowOverwrite() bool {
	if x != nil {
		return x.AllowOverwrite
	}
	return false
}

func (x *Metadata) GetValue() *structpb.Struct {
	if x != nil {
		return x.Value
	}
	return nil
}

func (x *Metadata) GetTypedValue() *anypb.Any {
	if x != nil {
		return x.TypedValue
	}
	return nil
}

type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The metadata namespace.
	// This field is deprecated; please use “metadata“ as replacement.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/filters/http/set_metadata/v3/set_metadata.proto.
	MetadataNamespace string `protobuf:"bytes,1,opt,name=metadata_namespace,json=metadataNamespace,proto3" json:"metadata_namespace,omitempty"`
	// The untyped value to update the dynamic metadata namespace with. See
	// :ref:`the filter documentation <config_http_filters_set_metadata>` for
	// more information on how this value is merged with potentially existing
	// ones.
	// This field is deprecated; please use “metadata“ as replacement.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/filters/http/set_metadata/v3/set_metadata.proto.
	Value *structpb.Struct `protobuf:"bytes,2,opt,name=value,proto3" json:"value,omitempty"`
	// Defines changes to be made to dynamic metadata.
	Metadata      []*Metadata `protobuf:"bytes,3,rep,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDescGZIP(), []int{1}
}

// Deprecated: Marked as deprecated in envoy/extensions/filters/http/set_metadata/v3/set_metadata.proto.
func (x *Config) GetMetadataNamespace() string {
	if x != nil {
		return x.MetadataNamespace
	}
	return ""
}

// Deprecated: Marked as deprecated in envoy/extensions/filters/http/set_metadata/v3/set_metadata.proto.
func (x *Config) GetValue() *structpb.Struct {
	if x != nil {
		return x.Value
	}
	return nil
}

func (x *Config) GetMetadata() []*Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

var File_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDesc = "" +
	"\n" +
	"@envoy/extensions/filters/http/set_metadata/v3/set_metadata.proto\x12-envoy.extensions.filters.http.set_metadata.v3\x1a\x19google/protobuf/any.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd1\x01\n" +
	"\bMetadata\x126\n" +
	"\x12metadata_namespace\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x11metadataNamespace\x12'\n" +
	"\x0fallow_overwrite\x18\x02 \x01(\bR\x0eallowOverwrite\x12-\n" +
	"\x05value\x18\x03 \x01(\v2\x17.google.protobuf.StructR\x05value\x125\n" +
	"\vtyped_value\x18\x04 \x01(\v2\x14.google.protobuf.AnyR\n" +
	"typedValue\"\xd5\x01\n" +
	"\x06Config\x12:\n" +
	"\x12metadata_namespace\x18\x01 \x01(\tB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\x11metadataNamespace\x12:\n" +
	"\x05value\x18\x02 \x01(\v2\x17.google.protobuf.StructB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\x05value\x12S\n" +
	"\bmetadata\x18\x03 \x03(\v27.envoy.extensions.filters.http.set_metadata.v3.MetadataR\bmetadataB\xbe\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	";io.envoyproxy.envoy.extensions.filters.http.set_metadata.v3B\x10SetMetadataProtoP\x01Zcgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/set_metadata/v3;set_metadatav3b\x06proto3"

var (
	file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDesc), len(file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDescData
}

var file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_goTypes = []any{
	(*Metadata)(nil),        // 0: envoy.extensions.filters.http.set_metadata.v3.Metadata
	(*Config)(nil),          // 1: envoy.extensions.filters.http.set_metadata.v3.Config
	(*structpb.Struct)(nil), // 2: google.protobuf.Struct
	(*anypb.Any)(nil),       // 3: google.protobuf.Any
}
var file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.filters.http.set_metadata.v3.Metadata.value:type_name -> google.protobuf.Struct
	3, // 1: envoy.extensions.filters.http.set_metadata.v3.Metadata.typed_value:type_name -> google.protobuf.Any
	2, // 2: envoy.extensions.filters.http.set_metadata.v3.Config.value:type_name -> google.protobuf.Struct
	0, // 3: envoy.extensions.filters.http.set_metadata.v3.Config.metadata:type_name -> envoy.extensions.filters.http.set_metadata.v3.Metadata
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_init() }
func file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_init() {
	if File_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDesc), len(file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto = out.File
	file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_goTypes = nil
	file_envoy_extensions_filters_http_set_metadata_v3_set_metadata_proto_depIdxs = nil
}
