/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1alpha1

import (
	v1 "sigs.k8s.io/gateway-api/apis/v1"
	apisv1 "sigs.k8s.io/gateway-api/applyconfiguration/apis/v1"
)

// MeshSpecApplyConfiguration represents a declarative configuration of the MeshSpec type for use
// with apply.
//
// MeshSpec defines the desired state of an XMesh.
type MeshSpecApplyConfiguration struct {
	// ControllerName is the name of a controller that is managing Gateway API
	// resources for mesh traffic management. The value of this field MUST be a
	// domain prefixed path.
	//
	// Example: "example.com/awesome-mesh".
	//
	// This field is not mutable and cannot be empty.
	//
	// Support: Core
	ControllerName *v1.GatewayController `json:"controllerName,omitempty"`
	// ParametersRef is an optional reference to a resource that contains
	// implementation-specific configuration for this Mesh. If no
	// implementation-specific parameters are needed, this field MUST be
	// omitted.
	//
	// ParametersRef can reference a standard Kubernetes resource, i.e.
	// ConfigMap, or an implementation-specific custom resource. The resource
	// can be cluster-scoped or namespace-scoped.
	//
	// If the referent cannot be found, refers to an unsupported kind, or when
	// the data within that resource is malformed, the Mesh MUST be rejected
	// with the "Accepted" status condition set to "False" and an
	// "InvalidParameters" reason.
	//
	// Support: Implementation-specific
	ParametersRef *apisv1.ParametersReferenceApplyConfiguration `json:"parametersRef,omitempty"`
	// Description optionally provides a human-readable description of a Mesh.
	Description *string `json:"description,omitempty"`
}

// MeshSpecApplyConfiguration constructs a declarative configuration of the MeshSpec type for use with
// apply.
func MeshSpec() *MeshSpecApplyConfiguration {
	return &MeshSpecApplyConfiguration{}
}

// WithControllerName sets the ControllerName field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ControllerName field is set to the value of the last call.
func (b *MeshSpecApplyConfiguration) WithControllerName(value v1.GatewayController) *MeshSpecApplyConfiguration {
	b.ControllerName = &value
	return b
}

// WithParametersRef sets the ParametersRef field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ParametersRef field is set to the value of the last call.
func (b *MeshSpecApplyConfiguration) WithParametersRef(value *apisv1.ParametersReferenceApplyConfiguration) *MeshSpecApplyConfiguration {
	b.ParametersRef = value
	return b
}

// WithDescription sets the Description field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Description field is set to the value of the last call.
func (b *MeshSpecApplyConfiguration) WithDescription(value string) *MeshSpecApplyConfiguration {
	b.Description = &value
	return b
}
