/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	metav1 "k8s.io/client-go/applyconfigurations/meta/v1"
)

// ListenerSetStatusApplyConfiguration represents a declarative configuration of the ListenerSetStatus type for use
// with apply.
type ListenerSetStatusApplyConfiguration struct {
	// Conditions describe the current conditions of the ListenerSet.
	//
	// Implementations MUST express ListenerSet conditions using the
	// `ListenerSetConditionType` and `ListenerSetConditionReason`
	// constants so that operators and tools can converge on a common
	// vocabulary to describe ListenerSet state.
	//
	// Known condition types are:
	//
	// * "Accepted"
	// * "Programmed"
	Conditions []metav1.ConditionApplyConfiguration `json:"conditions,omitempty"`
	// Listeners provide status for each unique listener port defined in the Spec.
	Listeners []ListenerEntryStatusApplyConfiguration `json:"listeners,omitempty"`
}

// ListenerSetStatusApplyConfiguration constructs a declarative configuration of the ListenerSetStatus type for use with
// apply.
func ListenerSetStatus() *ListenerSetStatusApplyConfiguration {
	return &ListenerSetStatusApplyConfiguration{}
}

// WithConditions adds the given value to the Conditions field in the declarative configuration
// and returns the receiver, so that objects can be build by chaining "With" function invocations.
// If called multiple times, values provided by each call will be appended to the Conditions field.
func (b *ListenerSetStatusApplyConfiguration) WithConditions(values ...*metav1.ConditionApplyConfiguration) *ListenerSetStatusApplyConfiguration {
	for i := range values {
		if values[i] == nil {
			panic("nil value passed to WithConditions")
		}
		b.Conditions = append(b.Conditions, *values[i])
	}
	return b
}

// WithListeners adds the given value to the Listeners field in the declarative configuration
// and returns the receiver, so that objects can be build by chaining "With" function invocations.
// If called multiple times, values provided by each call will be appended to the Listeners field.
func (b *ListenerSetStatusApplyConfiguration) WithListeners(values ...*ListenerEntryStatusApplyConfiguration) *ListenerSetStatusApplyConfiguration {
	for i := range values {
		if values[i] == nil {
			panic("nil value passed to WithListeners")
		}
		b.Listeners = append(b.Listeners, *values[i])
	}
	return b
}
