// Code generated by protoc-gen-jsonshim. DO NOT EDIT.
package v1beta1

import (
	bytes "bytes"
	jsonpb "github.com/golang/protobuf/jsonpb"
)

// MarshalJSON is a custom marshaler for RequestAuthentication
func (this *RequestAuthentication) MarshalJSON() ([]byte, error) {
	str, err := RequestAuthenticationMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for RequestAuthentication
func (this *RequestAuthentication) UnmarshalJSON(b []byte) error {
	return RequestAuthenticationUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for JWTRule
func (this *JWTRule) MarshalJSON() ([]byte, error) {
	str, err := RequestAuthenticationMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for JWTRule
func (this *JWTRule) UnmarshalJSON(b []byte) error {
	return RequestAuthenticationUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for JWTHeader
func (this *JWTHeader) MarshalJSON() ([]byte, error) {
	str, err := RequestAuthenticationMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for JWTHeader
func (this *JWTHeader) UnmarshalJSON(b []byte) error {
	return RequestAuthenticationUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for ClaimToHeader
func (this *ClaimToHeader) MarshalJSON() ([]byte, error) {
	str, err := RequestAuthenticationMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for ClaimToHeader
func (this *ClaimToHeader) UnmarshalJSON(b []byte) error {
	return RequestAuthenticationUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

var (
	RequestAuthenticationMarshaler   = &jsonpb.Marshaler{}
	RequestAuthenticationUnmarshaler = &jsonpb.Unmarshaler{AllowUnknownFields: true}
)
