// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/filters/network/client_ssl_auth/v3/client_ssl_auth.proto

package client_ssl_authv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ClientSSLAuth struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The :ref:`cluster manager <arch_overview_cluster_manager>` cluster that runs
	// the authentication service. The filter will connect to the service every 60s to fetch the list
	// of principals. The service must support the expected :ref:`REST API
	// <config_network_filters_client_ssl_auth_rest_api>`.
	AuthApiCluster string `protobuf:"bytes,1,opt,name=auth_api_cluster,json=authApiCluster,proto3" json:"auth_api_cluster,omitempty"`
	// The prefix to use when emitting :ref:`statistics
	// <config_network_filters_client_ssl_auth_stats>`.
	StatPrefix string `protobuf:"bytes,2,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// Time in milliseconds between principal refreshes from the
	// authentication service. Default is 60000 (60s). The actual fetch time
	// will be this value plus a random jittered value between
	// 0-refresh_delay_ms milliseconds.
	RefreshDelay *durationpb.Duration `protobuf:"bytes,3,opt,name=refresh_delay,json=refreshDelay,proto3" json:"refresh_delay,omitempty"`
	// An optional list of IP address and subnet masks that should be white
	// listed for access by the filter. If no list is provided, there is no
	// IP allowlist.
	IpWhiteList   []*v3.CidrRange `protobuf:"bytes,4,rep,name=ip_white_list,json=ipWhiteList,proto3" json:"ip_white_list,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClientSSLAuth) Reset() {
	*x = ClientSSLAuth{}
	mi := &file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClientSSLAuth) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClientSSLAuth) ProtoMessage() {}

func (x *ClientSSLAuth) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClientSSLAuth.ProtoReflect.Descriptor instead.
func (*ClientSSLAuth) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDescGZIP(), []int{0}
}

func (x *ClientSSLAuth) GetAuthApiCluster() string {
	if x != nil {
		return x.AuthApiCluster
	}
	return ""
}

func (x *ClientSSLAuth) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *ClientSSLAuth) GetRefreshDelay() *durationpb.Duration {
	if x != nil {
		return x.RefreshDelay
	}
	return nil
}

func (x *ClientSSLAuth) GetIpWhiteList() []*v3.CidrRange {
	if x != nil {
		return x.IpWhiteList
	}
	return nil
}

var File_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDesc = "" +
	"\n" +
	"Qcontrib/envoy/extensions/filters/network/client_ssl_auth/v3/client_ssl_auth.proto\x123envoy.extensions.filters.network.client_ssl_auth.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xd2\x02\n" +
	"\rClientSSLAuth\x127\n" +
	"\x10auth_api_cluster\x18\x01 \x01(\tB\r\xfaB\n" +
	"r\b\x10\x01\xc8\x01\x00\xc0\x01\x02R\x0eauthApiCluster\x12(\n" +
	"\vstat_prefix\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12>\n" +
	"\rrefresh_delay\x18\x03 \x01(\v2\x19.google.protobuf.DurationR\frefreshDelay\x12Y\n" +
	"\rip_white_list\x18\x04 \x03(\v2\x1f.envoy.config.core.v3.CidrRangeB\x14\xf2\x98\xfe\x8f\x05\x0e\n" +
	"\fip_allowlistR\vipWhiteList:C\x9aň\x1e>\n" +
	"<envoy.config.filter.network.client_ssl_auth.v2.ClientSSLAuthB\xd7\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Aio.envoyproxy.envoy.extensions.filters.network.client_ssl_auth.v3B\x12ClientSslAuthProtoP\x01Ztgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/client_ssl_auth/v3;client_ssl_authv3b\x06proto3"

var (
	file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDescData
}

var file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_goTypes = []any{
	(*ClientSSLAuth)(nil),       // 0: envoy.extensions.filters.network.client_ssl_auth.v3.ClientSSLAuth
	(*durationpb.Duration)(nil), // 1: google.protobuf.Duration
	(*v3.CidrRange)(nil),        // 2: envoy.config.core.v3.CidrRange
}
var file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.client_ssl_auth.v3.ClientSSLAuth.refresh_delay:type_name -> google.protobuf.Duration
	2, // 1: envoy.extensions.filters.network.client_ssl_auth.v3.ClientSSLAuth.ip_white_list:type_name -> envoy.config.core.v3.CidrRange
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() {
	file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_init()
}
func file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_init() {
	if File_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_depIdxs,
		MessageInfos:      file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto = out.File
	file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_goTypes = nil
	file_contrib_envoy_extensions_filters_network_client_ssl_auth_v3_client_ssl_auth_proto_depIdxs = nil
}
