// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/proxy_protocol/v3/upstream_proxy_protocol.proto

package proxy_protocolv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for PROXY protocol socket
type ProxyProtocolUpstreamTransport struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The PROXY protocol settings
	Config *v3.ProxyProtocolConfig `protobuf:"bytes,1,opt,name=config,proto3" json:"config,omitempty"`
	// The underlying transport socket being wrapped.
	TransportSocket *v3.TransportSocket `protobuf:"bytes,2,opt,name=transport_socket,json=transportSocket,proto3" json:"transport_socket,omitempty"`
	// If this is set to true, the null addresses are allowed in the PROXY protocol header.
	// The proxy protocol header encodes the null addresses to AF_UNSPEC.
	// [#not-implemented-hide:]
	AllowUnspecifiedAddress bool `protobuf:"varint,3,opt,name=allow_unspecified_address,json=allowUnspecifiedAddress,proto3" json:"allow_unspecified_address,omitempty"`
	// If true, all the TLVs are encoded in the connection pool key.
	// [#not-implemented-hide:]
	TlvAsPoolKey  bool `protobuf:"varint,4,opt,name=tlv_as_pool_key,json=tlvAsPoolKey,proto3" json:"tlv_as_pool_key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ProxyProtocolUpstreamTransport) Reset() {
	*x = ProxyProtocolUpstreamTransport{}
	mi := &file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ProxyProtocolUpstreamTransport) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProxyProtocolUpstreamTransport) ProtoMessage() {}

func (x *ProxyProtocolUpstreamTransport) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProxyProtocolUpstreamTransport.ProtoReflect.Descriptor instead.
func (*ProxyProtocolUpstreamTransport) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDescGZIP(), []int{0}
}

func (x *ProxyProtocolUpstreamTransport) GetConfig() *v3.ProxyProtocolConfig {
	if x != nil {
		return x.Config
	}
	return nil
}

func (x *ProxyProtocolUpstreamTransport) GetTransportSocket() *v3.TransportSocket {
	if x != nil {
		return x.TransportSocket
	}
	return nil
}

func (x *ProxyProtocolUpstreamTransport) GetAllowUnspecifiedAddress() bool {
	if x != nil {
		return x.AllowUnspecifiedAddress
	}
	return false
}

func (x *ProxyProtocolUpstreamTransport) GetTlvAsPoolKey() bool {
	if x != nil {
		return x.TlvAsPoolKey
	}
	return false
}

var File_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDesc = "" +
	"\n" +
	"Renvoy/extensions/transport_sockets/proxy_protocol/v3/upstream_proxy_protocol.proto\x124envoy.extensions.transport_sockets.proxy_protocol.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a)envoy/config/core/v3/proxy_protocol.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xa2\x02\n" +
	"\x1eProxyProtocolUpstreamTransport\x12A\n" +
	"\x06config\x18\x01 \x01(\v2).envoy.config.core.v3.ProxyProtocolConfigR\x06config\x12Z\n" +
	"\x10transport_socket\x18\x02 \x01(\v2%.envoy.config.core.v3.TransportSocketB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x0ftransportSocket\x12:\n" +
	"\x19allow_unspecified_address\x18\x03 \x01(\bR\x17allowUnspecifiedAddress\x12%\n" +
	"\x0ftlv_as_pool_key\x18\x04 \x01(\bR\ftlvAsPoolKeyB\xd8\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Bio.envoyproxy.envoy.extensions.transport_sockets.proxy_protocol.v3B\x1aUpstreamProxyProtocolProtoP\x01Zlgithub.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/proxy_protocol/v3;proxy_protocolv3b\x06proto3"

var (
	file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDescOnce sync.Once
	file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDescData []byte
)

func file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDescGZIP() []byte {
	file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDesc), len(file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDesc)))
	})
	return file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDescData
}

var file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_goTypes = []any{
	(*ProxyProtocolUpstreamTransport)(nil), // 0: envoy.extensions.transport_sockets.proxy_protocol.v3.ProxyProtocolUpstreamTransport
	(*v3.ProxyProtocolConfig)(nil),         // 1: envoy.config.core.v3.ProxyProtocolConfig
	(*v3.TransportSocket)(nil),             // 2: envoy.config.core.v3.TransportSocket
}
var file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.transport_sockets.proxy_protocol.v3.ProxyProtocolUpstreamTransport.config:type_name -> envoy.config.core.v3.ProxyProtocolConfig
	2, // 1: envoy.extensions.transport_sockets.proxy_protocol.v3.ProxyProtocolUpstreamTransport.transport_socket:type_name -> envoy.config.core.v3.TransportSocket
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_init()
}
func file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_init() {
	if File_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDesc), len(file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_msgTypes,
	}.Build()
	File_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto = out.File
	file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_proxy_protocol_v3_upstream_proxy_protocol_proto_depIdxs = nil
}
